@@ -320,7 +320,12 @@ project_mode (const char *file)
 }
 #endif
 
-#ifdef GDK_WINDOWING_WIN32
+#undef WANT_SYNAPTICS_FIX
+#if defined(GDK_WINDOWING_WIN32) && !GTK_CHECK_VERSION(2,24,8)
+#define WANT_SYNAPTICS_FIX 1
+#endif
+
+#ifdef WANT_SYNAPTICS_FIX
 
 static GdkWindow *
 _moo_get_toplevel_window_at_pointer (void)
@@ -512,7 +517,7 @@ hookup_synaptics_touchpad (void)
     gdk_window_add_filter (NULL, touchpad_filter_func, NULL);
 }
 
-#endif // GDK_WINDOWING_WIN32
+#endif // WANT_SYNAPTICS_FIX
 
 static void
 unit_test_func (void)
@@ -536,6 +541,22 @@ run_script_func (void)
         moo_app_run_script (moo_app_instance(), *p);
 }
 
+static void
+install_log_handlers (void)
+{
+    if (medit_opts.log_file)
+        moo_set_log_func_file (medit_opts.log_file);
+    else if (medit_opts.log_window)
+        moo_set_log_func_window (TRUE);
+#ifdef __WIN32__
+    // this will install do-nothing log and print handlers plus
+    // a fatal error win32 message handler (it will also turn off
+    // console output, but that is not visible anyway)
+    else
+        moo_set_log_func_silent ();
+#endif
+}
+
 static int
 medit_main (int argc, char *argv[])
 {
@@ -558,7 +579,7 @@ medit_main (int argc, char *argv[])
 
     stamp = get_time_stamp ();
 
-#ifdef GDK_WINDOWING_WIN32
+#ifdef WANT_SYNAPTICS_FIX
     hookup_synaptics_touchpad ();
 #endif
 
@@ -636,13 +657,7 @@ medit_main (int argc, char *argv[])
     gdk_threads_init ();
     gdk_threads_enter ();
 
-    if (medit_opts.log_file || medit_opts.log_window)
-    {
-        if (medit_opts.log_file)
-            moo_set_log_func_file (medit_opts.log_file);
-        else
-            moo_set_log_func_window (TRUE);
-    }
+    install_log_handlers ();
 
     app = MOO_APP (g_object_new (medit_app_get_type (),
                                  "run-input", run_input,
