@@ -14,7 +14,7 @@
  */
 
 /**
- * class:MooApp: (parent GObject)
+ * class:MooApp: (parent GObject): application object
  */
 
 #ifdef HAVE_CONFIG_H
@@ -47,7 +47,7 @@
 #include "mooutils/moohelp.h"
 #include "mooutils/moocompat.h"
 #include "mooutils/mooutils-script.h"
-#include <glib/gmappedfile.h>
+#include <glib.h>
 #include <string.h>
 #include <stdio.h>
 #include <errno.h>
@@ -358,7 +358,6 @@ moo_app_constructor (GType           type,
                      GObjectConstructParam *params)
 {
     GObject *object;
-    MooApp *app;
 
     if (moo_app_data.instance != NULL)
     {
@@ -368,7 +367,6 @@ moo_app_constructor (GType           type,
     }
 
     object = moo_app_parent_class->constructor (type, n_params, params);
-    app = MOO_APP (object);
 
 #if defined(HAVE_SIGNAL) && defined(SIGINT)
     setup_signals (sigint_handler);
@@ -600,7 +598,7 @@ init_plugins (MooApp *app)
 static void
 moo_app_init_editor (MooApp *app)
 {
-    app->priv->editor = moo_editor_create (FALSE);
+    app->priv->editor = moo_editor_create_instance (FALSE);
 
     g_signal_connect_swapped (app->priv->editor, "will-close-window",
                               G_CALLBACK (editor_will_close_window), app);
