@@ -1,7 +1,7 @@
 /*
  *   mooedit-fileops.c
  *
- *   Copyright (C) 2004-2009 by Yevgen Muntyan <muntyan@tamu.edu>
+ *   Copyright (C) 2004-2010 by Yevgen Muntyan <emuntyan@users.sourceforge.net>
  *
  *   This file is part of medit.  medit is free software; you can
  *   redistribute it and/or modify it under the terms of the
@@ -19,7 +19,7 @@
 
 #define MOOEDIT_COMPILATION
 #include "mooedit/mooedit-private.h"
-#include "mooedit/mooeditor-private.h"
+#include "mooedit/mooeditor-impl.h"
 #include "mooedit/mooedit-fileops.h"
 #include "mooedit/mooeditdialogs.h"
 #include "mooedit/mootextbuffer.h"
@@ -29,7 +29,9 @@
 #include "mooutils/mooencodings.h"
 #include "mooutils/mooi18n.h"
 #include "mooutils/mootype-macros.h"
-#include "mooutils/mooutils-messages.h"
+#include "mooutils/mooutils.h"
+#include "mooutils/mooutils-fs.h"
+#include "mooutils/moocompat.h"
 #include <string.h>
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -40,9 +42,7 @@
 #include <stdio.h>
 #include <errno.h>
 
-#if GLIB_CHECK_VERSION(2,6,0)
-# include <glib/gstdio.h>
-#endif
+#include <glib/gstdio.h>
 
 #undef LOAD_BINARY
 #define ENCODING_LOCALE "LOCALE"
@@ -58,9 +58,7 @@ static GHashTable *UNTITLED_NO = NULL;
 
 static void     block_buffer_signals        (MooEdit        *edit);
 static void     unblock_buffer_signals      (MooEdit        *edit);
-static gboolean focus_in_cb                 (MooEdit        *edit);
-static void     check_file_status           (MooEdit        *edit,
-                                             gboolean        in_focus_only);
+static void     check_file_status           (MooEdit        *edit);
 static void     file_modified_on_disk       (MooEdit        *edit);
 static void     file_deleted                (MooEdit        *edit);
 static void     add_status                  (MooEdit        *edit,
@@ -69,6 +67,7 @@ static void     add_status                  (MooEdit        *edit,
 static gboolean moo_edit_load_local         (MooEdit        *edit,
                                              GFile          *file,
                                              const char     *encoding,
+                                             const char     *cached_encoding,
                                              GError        **error);
 static gboolean moo_edit_reload_local       (MooEdit        *edit,
                                              const char     *encoding,
@@ -86,6 +85,14 @@ static gboolean moo_edit_save_copy_local    (MooEdit        *edit,
 static void     _moo_edit_start_file_watch  (MooEdit        *edit);
 
 
+gboolean
+_moo_is_file_error_cancelled (GError *error)
+{
+    return error && error->domain == MOO_EDIT_FILE_ERROR &&
+           error->code == MOO_EDIT_FILE_ERROR_CANCELLED;
+}
+
+
 static const char *
 normalize_encoding (const char *encoding,
                     gboolean    for_save)
@@ -96,27 +103,86 @@ normalize_encoding (const char *encoding,
 }
 
 
+gboolean
+_moo_edit_file_is_new (GFile *file)
+{
+    gboolean is_new;
+    char *filename;
+    g_return_val_if_fail (G_IS_FILE (file), FALSE);
+    filename = g_file_get_path (file);
+    g_return_val_if_fail (filename != NULL, FALSE);
+    is_new = !g_file_test (filename, G_FILE_TEST_EXISTS);
+    g_free (filename);
+    return is_new;
+}
+
+
 gboolean
 _moo_edit_load_file (MooEdit      *edit,
                      GFile        *file,
                      const char   *encoding,
+                     const char   *cached_encoding,
                      GError      **error)
 {
-    char *encoding_copy;
+    char *freeme1 = NULL;
+    char *freeme2 = NULL;
     gboolean result;
     GError *error_here = NULL;
 
-    moo_return_val_if_fail (MOO_IS_EDIT (edit), FALSE);
-    moo_return_val_if_fail (G_IS_FILE (file), FALSE);
+    moo_return_error_if_fail (MOO_IS_EDIT (edit));
+    moo_return_error_if_fail (G_IS_FILE (file));
+    moo_return_error_if_fail (!MOO_EDIT_IS_BUSY (edit));
 
-    encoding_copy = g_strdup (normalize_encoding (encoding, FALSE));
+    encoding = freeme1 = g_strdup (normalize_encoding (encoding, FALSE));
+    cached_encoding = freeme2 = cached_encoding ? g_strdup (normalize_encoding (cached_encoding, FALSE)) : NULL;
 
-    result = moo_edit_load_local (edit, file, encoding_copy, &error_here);
+    while (TRUE)
+    {
+        char *new_encoding = NULL;
+        MooEditTryEncodingResponse response;
+
+        result = moo_edit_load_local (edit, file, encoding, cached_encoding, &error_here);
+
+        if (result)
+            break;
+
+        if (!error_here || error_here->domain != MOO_EDIT_FILE_ERROR ||
+            error_here->code != MOO_EDIT_FILE_ERROR_ENCODING)
+                break;
+
+        response = _moo_edit_try_encoding_dialog (file, encoding, &new_encoding);
+
+        switch (response)
+        {
+            case MOO_EDIT_TRY_ENCODING_RESPONSE_CANCEL:
+                g_error_free (error_here);
+                error_here = g_error_new (MOO_EDIT_FILE_ERROR,
+                                          MOO_EDIT_FILE_ERROR_CANCELLED,
+                                          "Cancelled");
+                g_free (new_encoding);
+                new_encoding = NULL;
+                break;
+
+            case MOO_EDIT_TRY_ENCODING_RESPONSE_TRY_ANOTHER:
+                g_error_free (error_here);
+                error_here = NULL;
+                g_assert (new_encoding != NULL);
+                break;
+        }
+
+        if (!new_encoding)
+            break;
+
+        g_free (freeme1);
+        encoding = freeme1 = new_encoding;
+        cached_encoding = NULL;
+    }
 
     if (error_here)
         g_propagate_error (error, error_here);
 
-    g_free (encoding_copy);
+    g_free (freeme1);
+    g_free (freeme2);
     return result;
 }
 
@@ -129,7 +195,7 @@ _moo_edit_reload_file (MooEdit    *edit,
     GError *error_here = NULL;
     gboolean result;
 
-    moo_return_val_if_fail (MOO_IS_EDIT (edit), FALSE);
+    moo_return_error_if_fail (MOO_IS_EDIT (edit));
 
     result = moo_edit_reload_local (edit, encoding, &error_here);
 
@@ -151,8 +217,8 @@ _moo_edit_save_file (MooEdit        *edit,
     char *encoding_copy;
     gboolean result;
 
-    moo_return_val_if_fail (MOO_IS_EDIT (edit), FALSE);
-    moo_return_val_if_fail (G_IS_FILE (file), FALSE);
+    moo_return_error_if_fail (MOO_IS_EDIT (edit));
+    moo_return_error_if_fail (G_IS_FILE (file));
 
     encoding_copy = g_strdup (normalize_encoding (encoding, TRUE));
 
@@ -176,8 +242,8 @@ _moo_edit_save_file_copy (MooEdit        *edit,
     char *encoding_copy;
     gboolean result;
 
-    moo_return_val_if_fail (MOO_IS_EDIT (edit), FALSE);
-    moo_return_val_if_fail (G_IS_FILE (file), FALSE);
+    moo_return_error_if_fail (MOO_IS_EDIT (edit));
+    moo_return_error_if_fail (G_IS_FILE (file));
 
     encoding_copy = g_strdup (normalize_encoding (encoding, TRUE));
 
@@ -200,11 +266,17 @@ set_encoding_error (GError **error)
 }
 
 
+/**
+ * moo_edit_get_line_end_type:
+ **/
 MooLineEndType
 moo_edit_get_line_end_type (MooEdit *edit)
 {
-    moo_return_val_if_fail (MOO_IS_EDIT (edit), MOO_LE_DEFAULT);
-    return edit->priv->line_end_type;
+    g_return_val_if_fail (MOO_IS_EDIT (edit), MOO_LE_NATIVE);
+    if (edit->priv->line_end_type == MOO_LE_NONE)
+        return MOO_LE_NATIVE;
+    else
+        return edit->priv->line_end_type;
 }
 
 static void
@@ -212,8 +284,8 @@ moo_edit_set_line_end_type_full (MooEdit        *edit,
                                  MooLineEndType  le,
                                  gboolean        quiet)
 {
-    moo_return_if_fail (MOO_IS_EDIT (edit));
-    moo_return_if_fail (le > 0);
+    g_return_if_fail (MOO_IS_EDIT (edit));
+    g_return_if_fail (le > 0);
 
     if (edit->priv->line_end_type != le)
     {
@@ -223,11 +295,14 @@ moo_edit_set_line_end_type_full (MooEdit        *edit,
     }
 }
 
+/**
+ * moo_edit_set_line_end_type:
+ **/
 void
 moo_edit_set_line_end_type (MooEdit        *edit,
                             MooLineEndType  le)
 {
-    moo_return_if_fail (MOO_IS_EDIT (edit));
+    g_return_if_fail (MOO_IS_EDIT (edit));
     moo_edit_set_line_end_type_full (edit, le, FALSE);
 }
 
@@ -252,27 +327,6 @@ static LoadResult   load_binary (MooEdit      *edit,
                                  GError      **error);
 #endif
 
-const char *
-_moo_get_default_encodings (void)
-{
-    /* Translators: if translated, it should be a comma-separated list
-       of encodings to try when opening files. Encodings names should be
-       those understood by iconv, or "LOCALE" which means user's locale
-       charset. For instance, the default value is "UTF-8,LOCALE,ISO_8859-15,ISO_8859-1".
-       You want to add common preferred non-UTF8 encodings used in your locale.
-       Do not remove ISO_8859-15 and ISO_8859-1, instead leave them at the end,
-       these are common source files encodings. */
-    const char *to_translate = N_("encodings_list");
-    const char *encodings;
-
-    encodings = _(to_translate);
-
-    if (!strcmp (encodings, to_translate))
-        encodings = "UTF-8," ENCODING_LOCALE ",ISO_8859-1,ISO_8859-15";
-
-    return encodings;
-}
-
 static GSList *
 get_encodings (void)
 {
@@ -308,7 +362,7 @@ get_encodings (void)
 
     if (!result)
     {
-        g_critical ("%s: oops", G_STRLOC);
+        g_critical ("oops");
         result = g_slist_prepend (NULL, g_strdup ("UTF-8"));
     }
 
@@ -327,17 +381,17 @@ try_load (MooEdit      *edit,
     gboolean enable_highlight;
     LoadResult result = ERROR_FILE;
 
-    moo_return_val_if_fail (MOO_IS_EDIT (edit), result);
-    moo_return_val_if_fail (G_IS_FILE (file), result);
-    moo_return_val_if_fail (encoding && encoding[0], result);
+    g_return_val_if_fail (MOO_IS_EDIT (edit), result);
+    g_return_val_if_fail (G_IS_FILE (file), result);
+    g_return_val_if_fail (encoding && encoding[0], result);
 
-    buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (edit));
+    buffer = moo_edit_get_buffer (edit);
     gtk_text_buffer_set_text (buffer, "", 0);
 
-    g_object_get (edit, "enable-highlight", &enable_highlight, (char*) 0);
-    g_object_set (edit, "enable-highlight", FALSE, (char*) 0);
+    g_object_get (buffer, "highlight-syntax", &enable_highlight, (char*) 0);
+    g_object_set (buffer, "highlight-syntax", FALSE, (char*) 0);
     result = do_load (edit, file, encoding, error);
-    g_object_set (edit, "enable-highlight", enable_highlight, (char*) 0);
+    g_object_set (buffer, "highlight-syntax", enable_highlight, (char*) 0);
 
     return result;
 }
@@ -374,18 +428,18 @@ static gboolean
 moo_edit_load_local (MooEdit     *edit,
                      GFile       *file,
                      const char  *encoding,
+                     const char  *cached_encoding,
                      GError     **error)
 {
     GtkTextIter start;
     GtkTextBuffer *buffer;
-    MooTextView *view;
     gboolean undo;
     LoadResult result = ERROR_FILE;
     char *freeme = NULL;
     MooLineEndType saved_le;
 
-    moo_return_val_if_fail (MOO_IS_EDIT (edit), FALSE);
-    moo_return_val_if_fail (file != NULL, FALSE);
+    moo_return_error_if_fail (MOO_IS_EDIT (edit));
+    moo_return_error_if_fail (G_IS_FILE (file));
 
     if (!check_regular (file, error))
         return FALSE;
@@ -395,15 +449,14 @@ moo_edit_load_local (MooEdit     *edit,
     else
         undo = TRUE;
 
-    view = MOO_TEXT_VIEW (edit);
-    buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (edit));
+    buffer = moo_edit_get_buffer (edit);
 
     block_buffer_signals (edit);
 
     if (undo)
         gtk_text_buffer_begin_user_action (buffer);
     else
-        moo_text_view_begin_not_undoable_action (view);
+        moo_text_buffer_begin_non_undoable_action (MOO_TEXT_BUFFER (buffer));
 
     moo_text_buffer_begin_non_interactive_action (MOO_TEXT_BUFFER (buffer));
 
@@ -414,6 +467,8 @@ moo_edit_load_local (MooEdit     *edit,
         GSList *encodings;
 
         encodings = get_encodings ();
+        if (cached_encoding)
+            encodings = g_slist_prepend (encodings, g_strdup (cached_encoding));
         result = ERROR_ENCODING;
 
         while (encodings)
@@ -479,7 +534,7 @@ moo_edit_load_local (MooEdit     *edit,
     if (undo)
         gtk_text_buffer_end_user_action (buffer);
     else
-        moo_text_view_end_not_undoable_action (view);
+        moo_text_buffer_end_non_undoable_action (MOO_TEXT_BUFFER (buffer));
 
     moo_text_buffer_end_non_interactive_action (MOO_TEXT_BUFFER (buffer));
 
@@ -498,13 +553,14 @@ do_load (MooEdit      *edit,
     GIOStatus status;
     GtkTextBuffer *buffer;
     MooLineEndType le = MOO_LE_NONE;
+    gboolean mixed_le = FALSE;
     GString *text = NULL;
     char *line = NULL;
     LoadResult result = ERROR_FILE;
     char *path;
 
-    moo_return_val_if_fail (G_IS_FILE (gfile), ERROR_FILE);
-    moo_return_val_if_fail (encoding != NULL, ERROR_FILE);
+    g_return_val_if_fail (G_IS_FILE (gfile), ERROR_FILE);
+    g_return_val_if_fail (encoding != NULL, ERROR_FILE);
 
     if (!(path = g_file_get_path (gfile)))
     {
@@ -527,7 +583,7 @@ do_load (MooEdit      *edit,
     }
 
     text = g_string_new (NULL);
-    buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (edit));
+    buffer = moo_edit_get_buffer (edit);
 
     while (TRUE)
     {
@@ -548,7 +604,17 @@ do_load (MooEdit      *edit,
 
         if (line)
         {
-            if (!g_utf8_validate (line, len, NULL))
+            const char *invalid;
+            gboolean valid_utf8 = g_utf8_validate (line, len, &invalid);
+
+            // allow trailing zero byte
+            if (!valid_utf8 && invalid + 1 == line + len && *invalid == 0)
+            {
+                valid_utf8 = TRUE;
+                len -= 1;
+            }
+
+            if (!valid_utf8)
             {
                 result = ERROR_ENCODING;
                 g_set_error (error, G_CONVERT_ERROR,
@@ -584,8 +650,8 @@ do_load (MooEdit      *edit,
 
                 if (le_here)
                 {
-                    if (le && le != le_here)
-                        le = MOO_LE_MIX;
+                    if (mixed_le || (le && le != le_here))
+                        mixed_le = TRUE;
                     else
                         le = le_here;
                 }
@@ -618,14 +684,11 @@ do_load (MooEdit      *edit,
     if (text->len)
         gtk_text_buffer_insert_at_cursor (buffer, text->str, text->len);
 
-    switch (edit->priv->line_end_type)
-    {
-        case MOO_LE_MIX:
-            break;
-        default:
-            moo_edit_set_line_end_type_full (edit, le, TRUE);
-            break;
-    }
+    if (mixed_le)
+        le = MOO_LE_NATIVE;
+
+    if (le != MOO_LE_NONE)
+        moo_edit_set_line_end_type_full (edit, le, TRUE);
 
     g_string_free (text, TRUE);
     g_clear_error (error);
@@ -659,9 +722,9 @@ load_binary (MooEdit     *edit,
     GIOStatus status;
     GtkTextBuffer *buffer;
 
-    moo_return_val_if_fail (filename != NULL, FALSE);
+    moo_return_error_if_fail (filename != NULL);
 
-    buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (edit));
+    buffer = moo_edit_get_buffer (edit);
 
     file = g_io_channel_new_file (filename, "r", error);
 
@@ -740,24 +803,25 @@ moo_edit_reload_local (MooEdit    *edit,
     gboolean result, enable_highlight;
     GFile *file;
 
-    file = _moo_edit_get_file (edit);
-    moo_return_val_if_fail (file != NULL, FALSE);
+    file = moo_edit_get_file (edit);
+    moo_return_error_if_fail (G_IS_FILE (file));
 
-    buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (edit));
+    buffer = moo_edit_get_buffer (edit);
 
     block_buffer_signals (edit);
     gtk_text_buffer_begin_user_action (buffer);
 
     gtk_text_buffer_get_bounds (buffer, &start, &end);
     gtk_text_buffer_delete (buffer, &start, &end);
-    g_object_get (edit, "enable-highlight", &enable_highlight, (char*) 0);
-    g_object_set (edit, "enable-highlight", FALSE, (char*) 0);
+    g_object_get (buffer, "highlight-syntax", &enable_highlight, (char*) 0);
+    g_object_set (buffer, "highlight-syntax", FALSE, (char*) 0);
 
     result = _moo_edit_load_file (edit, file,
                                   encoding ? encoding : edit->priv->encoding,
+                                  NULL,
                                   error);
 
-    g_object_set (edit, "enable-highlight", enable_highlight, (char*) 0);
+    g_object_set (buffer, "highlight-syntax", enable_highlight, (char*) 0);
     gtk_text_buffer_end_user_action (buffer);
     unblock_buffer_signals (edit);
 
@@ -816,56 +880,30 @@ get_contents_with_fixed_line_end (GtkTextBuffer *buffer, const char *le, guint l
 static char *
 get_contents (MooEdit *edit)
 {
-    gboolean normalize_le = FALSE;
     const char *le = "\n";
     gsize le_len = 1;
-    MooLineEndType line_end_type;
     GtkTextBuffer *buffer;
-    char *contents;
 
-    line_end_type = edit->priv->line_end_type;
-    if (!line_end_type)
-        line_end_type = MOO_LE_DEFAULT;
-
-    switch (line_end_type)
+    switch (moo_edit_get_line_end_type (edit))
     {
         case MOO_LE_UNIX:
-            normalize_le = TRUE;
             le = "\n";
             le_len = 1;
             break;
         case MOO_LE_WIN32:
-            normalize_le = TRUE;
             le = "\r\n";
             le_len = 2;
             break;
         case MOO_LE_MAC:
-            normalize_le = TRUE;
             le = "\r";
             le_len = 1;
             break;
-
-        case MOO_LE_MIX:
-            break;
-
         default:
             moo_assert_not_reached ();
     }
 
-    buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (edit));
-
-    if (normalize_le)
-    {
-        contents = get_contents_with_fixed_line_end (buffer, le, le_len);
-    }
-    else
-    {
-        GtkTextIter start, end;
-        gtk_text_buffer_get_bounds (buffer, &start, &end);
-        contents = gtk_text_buffer_get_text (buffer, &start, &end, TRUE);
-    }
-
-    return contents;
+    buffer = moo_edit_get_buffer (edit);
+    return get_contents_with_fixed_line_end (buffer, le, le_len);
 }
 
 static gboolean
@@ -879,7 +917,7 @@ do_write (GFile             *file,
     MooFileWriterFlags writer_flags;
     gboolean success = FALSE;
 
-    moo_return_val_if_fail (G_IS_FILE (file), FALSE);
+    moo_return_error_if_fail (G_IS_FILE (file));
 
     writer_flags = (flags & MOO_EDIT_SAVE_BACKUP) ? MOO_FILE_WRITER_SAVE_BACKUP : (MooFileWriterFlags) 0;
 
@@ -897,18 +935,15 @@ do_save_local (MooEdit        *edit,
                GFile          *file,
                const char     *encoding,
                MooEditSaveFlags flags,
-               GError        **error,
-               gboolean       *retval)
+               GError        **error)
 {
     char *utf8_contents;
     const char *to_save;
     gsize to_save_size;
     GError *encoding_error = NULL;
     char *freeme = NULL;
-    gboolean success;
+    gboolean success = TRUE;
 
-    success = TRUE;
-    *retval = TRUE;
     utf8_contents = get_contents (edit);
 
     moo_release_assert (utf8_contents != NULL);
@@ -929,12 +964,13 @@ do_save_local (MooEdit        *edit,
         {
             to_save = encoded;
             to_save_size = bytes_written;
+            freeme = encoded;
         }
         else
         {
-            *retval = FALSE;
-            to_save = utf8_contents;
-            to_save_size = strlen (utf8_contents);
+            g_propagate_error (error, encoding_error);
+            set_encoding_error (error);
+            success = FALSE;
         }
     }
     else
@@ -943,16 +979,8 @@ do_save_local (MooEdit        *edit,
         to_save_size = strlen (utf8_contents);
     }
 
-    if (!do_write (file, to_save, to_save_size, flags, error))
-    {
-        *retval = FALSE;
+    if (success && !do_write (file, to_save, to_save_size, flags, error))
         success = FALSE;
-    }
-    else if (encoding_error)
-    {
-        g_propagate_error (error, encoding_error);
-        set_encoding_error (error);
-    }
 
     g_free (freeme);
     g_free (utf8_contents);
@@ -967,21 +995,17 @@ moo_edit_save_local (MooEdit        *edit,
                      MooEditSaveFlags flags,
                      GError        **error)
 {
-    gboolean result;
+    moo_return_error_if_fail (MOO_IS_EDIT (edit));
+    moo_return_error_if_fail (G_IS_FILE (file));
 
-    moo_return_val_if_fail (MOO_IS_EDIT (edit), FALSE);
-    moo_return_val_if_fail (G_IS_FILE (file), FALSE);
+    if (!do_save_local (edit, file, encoding, flags, error))
+        return FALSE;
 
-    if (do_save_local (edit, file, encoding, flags, error, &result))
-    {
-        edit->priv->status = (MooEditStatus) 0;
-        _moo_edit_set_file (edit, file,
-                           result ? encoding : MOO_ENCODING_UTF8);
-        moo_edit_set_modified (edit, FALSE);
-        _moo_edit_start_file_watch (edit);
-    }
-
-    return result;
+    edit->priv->status = (MooEditStatus) 0;
+    _moo_edit_set_file (edit, file, encoding);
+    moo_edit_set_modified (edit, FALSE);
+    _moo_edit_start_file_watch (edit);
+    return TRUE;
 }
 
 
@@ -992,11 +1016,9 @@ moo_edit_save_copy_local (MooEdit        *edit,
                           MooEditSaveFlags flags,
                           GError        **error)
 {
-    gboolean result;
-    moo_return_val_if_fail (MOO_IS_EDIT (edit), FALSE);
-    moo_return_val_if_fail (G_IS_FILE (file), FALSE);
-    do_save_local (edit, file, encoding, flags, error, &result);
-    return result;
+    moo_return_error_if_fail (MOO_IS_EDIT (edit));
+    moo_return_error_if_fail (G_IS_FILE (file));
+    return do_save_local (edit, file, encoding, flags, error);
 }
 
 
@@ -1007,16 +1029,14 @@ moo_edit_save_copy_local (MooEdit        *edit,
 static void
 block_buffer_signals (MooEdit *edit)
 {
-    GtkTextBuffer *buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (edit));
-    g_signal_handler_block (buffer, edit->priv->modified_changed_handler_id);
+    g_signal_handler_block (moo_edit_get_buffer (edit), edit->priv->modified_changed_handler_id);
 }
 
 
 static void
 unblock_buffer_signals (MooEdit *edit)
 {
-    GtkTextBuffer *buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (edit));
-    g_signal_handler_unblock (buffer, edit->priv->modified_changed_handler_id);
+    g_signal_handler_unblock (moo_edit_get_buffer (edit), edit->priv->modified_changed_handler_id);
 }
 
 
@@ -1027,10 +1047,10 @@ file_watch_callback (G_GNUC_UNUSED MooFileWatch *watch,
 {
     MooEdit *edit = MOO_EDIT (data);
 
-    moo_return_if_fail (MOO_IS_EDIT (data));
-    moo_return_if_fail (event->monitor_id == edit->priv->file_monitor_id);
-    moo_return_if_fail (edit->priv->filename != NULL);
-    moo_return_if_fail (!(edit->priv->status & MOO_EDIT_CHANGED_ON_DISK));
+    g_return_if_fail (MOO_IS_EDIT (data));
+    g_return_if_fail (event->monitor_id == edit->priv->file_monitor_id);
+    g_return_if_fail (edit->priv->filename != NULL);
+    g_return_if_fail (!(edit->priv->status & MOO_EDIT_STATUS_CHANGED_ON_DISK));
 
     switch (event->code)
     {
@@ -1048,11 +1068,11 @@ file_watch_callback (G_GNUC_UNUSED MooFileWatch *watch,
             break;
 
         case MOO_FILE_EVENT_CREATED:
-            g_critical ("%s: oops", G_STRLOC);
+            g_critical ("oops");
             break;
     }
 
-    check_file_status (edit, FALSE);
+    check_file_status (edit);
 }
 
 
@@ -1063,14 +1083,14 @@ _moo_edit_start_file_watch (MooEdit *edit)
     GError *error = NULL;
 
     watch = _moo_editor_get_file_watch (edit->priv->editor);
-    moo_return_if_fail (watch != NULL);
+    g_return_if_fail (watch != NULL);
 
     if (edit->priv->file_monitor_id)
         moo_file_watch_cancel_monitor (watch, edit->priv->file_monitor_id);
     edit->priv->file_monitor_id = 0;
 
-    moo_return_if_fail ((edit->priv->status & MOO_EDIT_CHANGED_ON_DISK) == 0);
-    moo_return_if_fail (edit->priv->filename != NULL);
+    g_return_if_fail ((edit->priv->status & MOO_EDIT_STATUS_CHANGED_ON_DISK) == 0);
+    g_return_if_fail (edit->priv->filename != NULL);
 
     edit->priv->file_monitor_id =
         moo_file_watch_create_monitor (watch, edit->priv->filename,
@@ -1079,23 +1099,11 @@ _moo_edit_start_file_watch (MooEdit *edit)
 
     if (!edit->priv->file_monitor_id)
     {
-        g_warning ("%s: could not start watch for '%s'",
-                   G_STRLOC, edit->priv->filename);
-
-        if (error)
-        {
-            g_warning ("%s: %s", G_STRLOC, error->message);
-            g_error_free (error);
-        }
-
+        g_warning ("could not start watch for '%s': %s",
+                   edit->priv->filename, moo_error_message (error));
+        g_error_free (error);
         return;
     }
-
-    if (!edit->priv->focus_in_handler_id)
-        edit->priv->focus_in_handler_id =
-                g_signal_connect (edit, "focus-in-event",
-                                  G_CALLBACK (focus_in_cb),
-                                  NULL);
 }
 
 
@@ -1105,37 +1113,19 @@ _moo_edit_stop_file_watch (MooEdit *edit)
     MooFileWatch *watch;
 
     watch = _moo_editor_get_file_watch (edit->priv->editor);
-    moo_return_if_fail (watch != NULL);
+    g_return_if_fail (watch != NULL);
 
     if (edit->priv->file_monitor_id)
         moo_file_watch_cancel_monitor (watch, edit->priv->file_monitor_id);
     edit->priv->file_monitor_id = 0;
-
-    if (edit->priv->focus_in_handler_id)
-    {
-        g_signal_handler_disconnect (edit, edit->priv->focus_in_handler_id);
-        edit->priv->focus_in_handler_id = 0;
-    }
-}
-
-
-static gboolean
-focus_in_cb (MooEdit *edit)
-{
-    check_file_status (edit, TRUE);
-    return FALSE;
 }
 
 
 static void
-check_file_status (MooEdit  *edit,
-                   gboolean  in_focus_only)
+check_file_status (MooEdit *edit)
 {
-    if (in_focus_only && !GTK_WIDGET_HAS_FOCUS (edit))
-        return;
-
-    moo_return_if_fail (edit->priv->filename != NULL);
-    moo_return_if_fail (!(edit->priv->status & MOO_EDIT_CHANGED_ON_DISK));
+    g_return_if_fail (edit->priv->filename != NULL);
+    g_return_if_fail (!(edit->priv->status & MOO_EDIT_STATUS_CHANGED_ON_DISK));
 
     if (edit->priv->deleted_from_disk)
         file_deleted (edit);
@@ -1147,22 +1137,22 @@ check_file_status (MooEdit  *edit,
 static void
 file_modified_on_disk (MooEdit *edit)
 {
-    moo_return_if_fail (edit->priv->filename != NULL);
+    g_return_if_fail (edit->priv->filename != NULL);
     edit->priv->modified_on_disk = FALSE;
     edit->priv->deleted_from_disk = FALSE;
     _moo_edit_stop_file_watch (edit);
-    add_status (edit, MOO_EDIT_MODIFIED_ON_DISK);
+    add_status (edit, MOO_EDIT_STATUS_MODIFIED_ON_DISK);
 }
 
 
 static void
 file_deleted (MooEdit *edit)
 {
-    moo_return_if_fail (edit->priv->filename != NULL);
+    g_return_if_fail (edit->priv->filename != NULL);
     edit->priv->modified_on_disk = FALSE;
     edit->priv->deleted_from_disk = FALSE;
     _moo_edit_stop_file_watch (edit);
-    add_status (edit, MOO_EDIT_DELETED);
+    add_status (edit, MOO_EDIT_STATUS_DELETED);
 }
 
 
@@ -1175,20 +1165,15 @@ add_status (MooEdit        *edit,
 }
 
 
-static void
-remove_untitled (gpointer    destroyed,
-                 MooEdit    *edit)
+void
+_moo_edit_remove_untitled (MooEdit *doc)
 {
-    gpointer n = g_hash_table_lookup (UNTITLED_NO, edit);
+    gpointer n = g_hash_table_lookup (UNTITLED_NO, doc);
 
     if (n)
     {
         UNTITLED = g_slist_remove (UNTITLED, n);
-        g_hash_table_remove (UNTITLED_NO, edit);
-        if (!destroyed)
-            g_object_weak_unref (G_OBJECT (edit),
-                                 (GWeakNotify) remove_untitled,
-                                 GINT_TO_POINTER (TRUE));
+        g_hash_table_remove (UNTITLED_NO, doc);
     }
 }
 
@@ -1210,39 +1195,22 @@ add_untitled (MooEdit *edit)
         }
 
         g_hash_table_insert (UNTITLED_NO, edit, GINT_TO_POINTER (n));
-        g_object_weak_ref (G_OBJECT (edit),
-                           (GWeakNotify) remove_untitled,
-                           GINT_TO_POINTER (TRUE));
     }
 
     return n;
 }
 
 
-const char *
-_moo_edit_get_default_encoding (void)
-{
-    return moo_prefs_get_string (moo_edit_setting (MOO_EDIT_PREFS_ENCODING_SAVE));
-}
-
-
-char *
-_moo_file_get_display_name (GFile *file)
-{
-    moo_return_val_if_fail (G_IS_FILE (file), NULL);
-    return g_file_get_parse_name (file);
-}
-
-char *
-_moo_file_get_display_basename (GFile *file)
+static char *
+moo_file_get_display_basename (GFile *file)
 {
     char *name;
     const char *slash;
 
-    moo_return_val_if_fail (G_IS_FILE (file), NULL);
+    g_return_val_if_fail (G_IS_FILE (file), NULL);
 
-    name = _moo_file_get_display_name (file);
-    moo_return_val_if_fail (name != NULL, NULL);
+    name = moo_file_get_display_name (file);
+    g_return_val_if_fail (name != NULL, NULL);
 
     slash = strrchr (name, '/');
 
@@ -1260,6 +1228,53 @@ _moo_file_get_display_basename (GFile *file)
     return name;
 }
 
+char *
+_moo_edit_normalize_filename_for_comparison (const char *filename)
+{
+    g_return_val_if_fail (filename != NULL, NULL);
+#ifdef __WIN32__
+    /* XXX */
+    char *tmp = g_utf8_normalize (filename, -1, G_NORMALIZE_ALL_COMPOSE);
+    char *ret = g_utf8_strdown (tmp, -1);
+    g_free (tmp);
+    return ret;
+#else
+    return g_strdup (filename);
+#endif
+}
+
+char *_moo_edit_normalize_uri_for_comparison (const char *uri)
+{
+    return _moo_edit_normalize_filename_for_comparison (uri);
+}
+
+char *
+_moo_file_get_normalized_name (GFile *file)
+{
+    char *ret;
+    char *tmp = NULL;
+    char *tmp2 = NULL;
+
+    g_return_val_if_fail (G_IS_FILE (file), NULL);
+
+    tmp = g_file_get_path (file);
+
+    if (tmp)
+    {
+        tmp2 = _moo_normalize_file_path (tmp);
+        ret = _moo_edit_normalize_filename_for_comparison (tmp2);
+    }
+    else
+    {
+        tmp = g_file_get_uri (file);
+        g_return_val_if_fail (tmp != NULL, NULL);
+        ret = _moo_edit_normalize_uri_for_comparison (tmp);
+    }
+
+    g_free (tmp2);
+    g_free (tmp);
+    return ret;
+}
 
 void
 _moo_edit_set_file (MooEdit    *edit,
@@ -1267,12 +1282,14 @@ _moo_edit_set_file (MooEdit    *edit,
                     const char *encoding)
 {
     GFile *tmp;
-    char *tmp2, *tmp3, *tmp4;
+    GSList *free_list = NULL;
 
     tmp = edit->priv->file;
-    tmp2 = edit->priv->filename;
-    tmp3 = edit->priv->display_filename;
-    tmp4 = edit->priv->display_basename;
+
+    free_list = g_slist_prepend (free_list, edit->priv->filename);
+    free_list = g_slist_prepend (free_list, edit->priv->norm_name);
+    free_list = g_slist_prepend (free_list, edit->priv->display_filename);
+    free_list = g_slist_prepend (free_list, edit->priv->display_basename);
 
     if (!UNTITLED_NO)
         UNTITLED_NO = g_hash_table_new (g_direct_hash, g_direct_equal);
@@ -1283,6 +1300,7 @@ _moo_edit_set_file (MooEdit    *edit,
 
         edit->priv->file = NULL;
         edit->priv->filename = NULL;
+        edit->priv->norm_name = NULL;
 
         if (n == 1)
             edit->priv->display_filename = g_strdup (_("Untitled"));
@@ -1293,11 +1311,12 @@ _moo_edit_set_file (MooEdit    *edit,
     }
     else
     {
-        remove_untitled (NULL, edit);
+        _moo_edit_remove_untitled (edit);
         edit->priv->file = g_file_dup (file);
         edit->priv->filename = g_file_get_path (file);
-        edit->priv->display_filename = _moo_file_get_display_name (file);
-        edit->priv->display_basename = _moo_file_get_display_basename (file);
+        edit->priv->norm_name = _moo_file_get_normalized_name (file);
+        edit->priv->display_filename = moo_file_get_display_name (file);
+        edit->priv->display_basename = moo_file_get_display_basename (file);
     }
 
     if (!encoding)
@@ -1305,14 +1324,14 @@ _moo_edit_set_file (MooEdit    *edit,
     else
         moo_edit_set_encoding (edit, encoding);
 
-    g_signal_emit_by_name (edit, "filename-changed", edit->priv->filename, NULL);
-    moo_edit_status_changed (edit);
+    g_signal_emit_by_name (edit, "filename-changed", NULL);
+    _moo_edit_status_changed (edit);
+    _moo_edit_queue_recheck_config (edit);
 
-    if (tmp)
-        g_object_unref (tmp);
-    g_free (tmp2);
-    g_free (tmp3);
-    g_free (tmp4);
+    moo_file_free (tmp);
+
+    g_slist_foreach (free_list, (GFunc) g_free, NULL);
+    g_slist_free (free_list);
 }
 
 
@@ -1322,9 +1341,9 @@ _moo_edit_get_icon (MooEdit     *doc,
                     GtkIconSize  size)
 {
     if (doc->priv->filename)
-        return moo_get_icon_for_file (doc->priv->filename, widget, size);
+        return moo_get_icon_for_path (doc->priv->filename, widget, size);
     else if (doc->priv->file)
-        return moo_get_icon_for_file (doc->priv->display_basename, widget, size);
+        return moo_get_icon_for_path (doc->priv->display_basename, widget, size);
     else
-        return moo_get_icon_for_file (NULL, widget, size);
+        return moo_get_icon_for_path (NULL, widget, size);
 }
