@@ -303,7 +303,7 @@ moo_edit_constructor (GType                  type,
     doc = MOO_EDIT (object);
 
     view = _moo_edit_view_new (doc);
-    g_assert (doc->priv->views->n_elms == 1 && doc->priv->views->elms[0] == view);
+    (void) view; g_assert (doc->priv->views->n_elms == 1 && doc->priv->views->elms[0] == view);
 
     _moo_edit_add_class_actions (doc);
     _moo_edit_instances = moo_edit_list_prepend (_moo_edit_instances, doc);
@@ -594,7 +594,7 @@ moo_edit_is_empty (MooEdit *edit)
 
     g_return_val_if_fail (MOO_IS_EDIT (edit), FALSE);
 
-    if (MOO_EDIT_IS_BUSY (edit) || MOO_EDIT_IS_MODIFIED (edit) || !MOO_EDIT_IS_UNTITLED (edit))
+    if (MOO_EDIT_IS_BUSY (edit) || moo_edit_is_modified (edit) || !MOO_EDIT_IS_UNTITLED (edit))
         return FALSE;
 
     gtk_text_buffer_get_bounds (moo_edit_get_buffer (edit), &start, &end);
@@ -619,7 +619,7 @@ gboolean
 moo_edit_is_modified (MooEdit *edit)
 {
     g_return_val_if_fail (MOO_IS_EDIT (edit), FALSE);
-    return MOO_EDIT_IS_MODIFIED (edit);
+    return (moo_edit_get_status (edit) & MOO_EDIT_STATUS_MODIFIED) != 0;
 }
 
 /**
@@ -1404,6 +1404,13 @@ moo_edit_save (MooEdit *doc,
 
 /**
  * moo_edit_save_as:
+ *
+ * @doc:
+ * @info: (allow-none):
+ * @error:
+ *
+ * Save document with new filename and/or encoding. If @info is
+ * missing or %NULL then the user is asked for a new filename first.
  **/
 gboolean
 moo_edit_save_as (MooEdit     *doc,
@@ -1707,8 +1714,11 @@ block_uncomment (GtkTextBuffer *buffer,
 }
 
 
+/**
+ * moo_edit_comment_selection:
+ **/
 void
-_moo_edit_comment (MooEdit *edit)
+moo_edit_comment_selection (MooEdit *edit)
 {
     MooLang *lang;
     GtkTextIter start, end;
@@ -1763,8 +1773,11 @@ _moo_edit_comment (MooEdit *edit)
 }
 
 
+/**
+ * moo_edit_uncomment_selection:
+ **/
 void
-_moo_edit_uncomment (MooEdit *edit)
+moo_edit_uncomment_selection (MooEdit *edit)
 {
     MooLang *lang;
     GtkTextIter start, end;
