@@ -1,13 +1,16 @@
 /*
  *   mooeditconfig.c
  *
- *   Copyright (C) 2004-2008 by Yevgen Muntyan <muntyan@tamu.edu>
+ *   Copyright (C) 2004-2010 by Yevgen Muntyan <emuntyan@users.sourceforge.net>
  *
- *   This library is free software; you can redistribute it and/or
- *   modify it under the terms of the GNU Lesser General Public
- *   License version 2.1 as published by the Free Software Foundation.
+ *   This file is part of medit.  medit is free software; you can
+ *   redistribute it and/or modify it under the terms of the
+ *   GNU Lesser General Public License as published by the
+ *   Free Software Foundation; either version 2.1 of the License,
+ *   or (at your option) any later version.
  *
- *   See COPYING file that comes with this distribution.
+ *   You should have received a copy of the GNU Lesser General Public
+ *   License along with medit.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #define MOOEDIT_COMPILATION
@@ -102,7 +105,7 @@ moo_edit_config_init (MooEditConfig *config)
 
     config->priv = G_TYPE_INSTANCE_GET_PRIVATE (config, MOO_TYPE_EDIT_CONFIG, MooEditConfigPrivate);
 
-    MOO_IP_ARRAY_INIT (config->priv, values, vars->len);
+    MOO_IP_ARRAY_INIT (Value, config->priv, values, vars->len);
 
     if (global)
     {
@@ -142,7 +145,7 @@ global_init (void)
     {
         guint i;
 
-        global = g_object_new (MOO_TYPE_EDIT_CONFIG, NULL);
+        global = MOO_EDIT_CONFIG (g_object_new (MOO_TYPE_EDIT_CONFIG, (const char*) NULL));
 
         for (i = 1; i < vars->len; ++i)
         {
@@ -162,7 +165,7 @@ MooEditConfig *
 moo_edit_config_new (void)
 {
     global_init ();
-    return g_object_new (MOO_TYPE_EDIT_CONFIG, NULL);
+    return MOO_EDIT_CONFIG (g_object_new (MOO_TYPE_EDIT_CONFIG, (const char*) NULL));
 }
 
 
@@ -228,7 +231,7 @@ update_prop_from_global (MooEditConfig *config,
 
     if (prop_id == config->priv->n_values)
     {
-        MOO_IP_ARRAY_GROW (config->priv, values, 1);
+        MOO_IP_ARRAY_GROW (Value, config->priv, values, 1);
         g_value_init (GVALUE (config, prop_id),
                       G_VALUE_TYPE (GVALUE (global, prop_id)));
         VALUE (config, prop_id)->source = VALUE (global, prop_id)->source;
@@ -267,7 +270,7 @@ global_add_prop (GParamSpec *pspec,
 {
     g_assert (global->priv->n_values == prop_id);
 
-    MOO_IP_ARRAY_GROW (global->priv, values, 1);
+    MOO_IP_ARRAY_GROW (Value, global->priv, values, 1);
 
     g_value_init (GVALUE (global, prop_id), G_PARAM_SPEC_VALUE_TYPE (pspec));
     g_param_value_set_default (pspec, GVALUE (global, prop_id));
@@ -289,8 +292,8 @@ moo_edit_config_install_setting (GParamSpec *pspec)
 
     if (moo_edit_config_lookup_spec (pspec->name, NULL, TRUE))
     {
-        g_warning ("%s: setting named '%s' already exists",
-                   G_STRLOC, pspec->name);
+        g_warning ("setting named '%s' already exists",
+                   pspec->name);
         return 0;
     }
 
@@ -479,8 +482,7 @@ moo_edit_config_set_valist (MooEditConfig  *config,
 
         if (!pspec)
         {
-            g_warning ("%s: there is no setting named `%s'",
-                       G_STRLOC, name);
+            g_warning ("there is no setting named `%s'", name);
             break;
         }
 
@@ -489,7 +491,7 @@ moo_edit_config_set_valist (MooEditConfig  *config,
 
         if (error)
         {
-            g_warning ("%s: %s", G_STRLOC, error);
+            g_warning ("%s", error);
             g_free (error);
             g_value_unset (&value);
             break;
@@ -727,7 +729,7 @@ moo_edit_config_parse_one (MooEditConfig  *config,
 
     if (!pspec)
     {
-        g_warning ("%s: no setting named '%s'", G_STRLOC, name);
+        g_warning ("no setting named '%s'", name);
         return;
     }
 
@@ -742,8 +744,8 @@ moo_edit_config_parse_one (MooEditConfig  *config,
     if (result)
         moo_edit_config_set_value (config, prop_id, source, &gval);
     else
-        g_warning ("%s: could not convert '%s' to type '%s'",
-                   G_STRLOC, value, g_type_name (G_VALUE_TYPE (&gval)));
+        g_warning ("could not convert '%s' to type '%s'",
+                   value, g_type_name (G_VALUE_TYPE (&gval)));
 
     g_value_unset (&gval);
 }
@@ -813,8 +815,7 @@ moo_edit_config_parse (MooEditConfig  *config,
 
         if (!colon && !assign)
         {
-            g_warning ("%s: invalid config string '%s'",
-                       G_STRLOC, string);
+            g_warning ("invalid config string '%s'", string);
             return;
         }
 
@@ -828,8 +829,7 @@ moo_edit_config_parse (MooEditConfig  *config,
 
         if (name_end == name_start)
         {
-            g_warning ("%s: invalid config string '%s'",
-                       G_STRLOC, string);
+            g_warning ("invalid config string '%s'", string);
             return;
         }
 
@@ -839,8 +839,7 @@ moo_edit_config_parse (MooEditConfig  *config,
 
             if (!*value_start || *value_start == ';' || *value_start == ',')
             {
-                g_warning ("%s: invalid config string '%s'",
-                           G_STRLOC, string);
+                g_warning ("invalid config string '%s'", string);
                 return;
             }
 
@@ -859,8 +858,7 @@ moo_edit_config_parse (MooEditConfig  *config,
 
             if (!(separator = g_utf8_get_char (value_start)))
             {
-                g_warning ("%s: invalid config string '%s'",
-                           G_STRLOC, string);
+                g_warning ("invalid config string '%s'", string);
                 return;
             }
 
@@ -869,8 +867,7 @@ moo_edit_config_parse (MooEditConfig  *config,
                                        separator);
             if (!value_end)
             {
-                g_warning ("%s: unterminated value in '%s'",
-                           G_STRLOC, string);
+                g_warning ("unterminated value in '%s'", string);
                 return;
             }
 
@@ -902,19 +899,19 @@ moo_edit_config_install_alias (const char     *name,
 
     if (!pspec)
     {
-        g_warning ("%s: no setting named '%s'", G_STRLOC, name);
+        g_warning ("no setting named '%s'", name);
         return;
     }
 
     if (moo_edit_config_lookup_spec (alias, NULL, TRUE))
     {
-        g_warning ("%s: setting named '%s' already exists", G_STRLOC, alias);
+        g_warning ("setting named '%s' already exists", alias);
         return;
     }
 
     if (g_hash_table_lookup (aliases, alias))
     {
-        g_warning ("%s: alias '%s' already exists", G_STRLOC, alias);
+        g_warning ("alias '%s' already exists", alias);
         return;
     }
 
