@@ -27,7 +27,7 @@
 #include "mooedit/mooeditsavemult-gxml.h"
 #include "mooedit/mootryencoding-gxml.h"
 #include <gtk/gtk.h>
-#include <glib/gregex.h>
+#include <glib.h>
 #include <string.h>
 
 
