@@ -148,7 +148,7 @@ _moo_edit_init_prefs (void)
 
     NEW_KEY_BOOL (MOO_EDIT_PREFS_DIALOGS_OPEN_FOLLOWS_DOC, FALSE);
 
-    NEW_KEY_STRING (MOO_EDIT_PREFS_COLOR_SCHEME, "kate");
+    NEW_KEY_STRING (MOO_EDIT_PREFS_COLOR_SCHEME, "medit");
 
     NEW_KEY_BOOL (MOO_EDIT_PREFS_SMART_HOME_END, TRUE);
     NEW_KEY_BOOL (MOO_EDIT_PREFS_WRAP_ENABLE, FALSE);
