@@ -1,23 +1,32 @@
 /*
  *   moolinemark.c
  *
- *   Copyright (C) 2004-2007 by Yevgen Muntyan <muntyan@math.tamu.edu>
+ *   Copyright (C) 2004-2010 by Yevgen Muntyan <emuntyan@users.sourceforge.net>
+ *   Copyright (C) 2008 by Daniel Poelzleithner <mooedit@poelzi.org>
  *
- *   This library is free software; you can redistribute it and/or
- *   modify it under the terms of the GNU Lesser General Public
- *   License version 2.1 as published by the Free Software Foundation.
+ *   This file is part of medit.  medit is free software; you can
+ *   redistribute it and/or modify it under the terms of the
+ *   GNU Lesser General Public License as published by the
+ *   Free Software Foundation; either version 2.1 of the License,
+ *   or (at your option) any later version.
  *
- *   See COPYING file that comes with this distribution.
+ *   You should have received a copy of the GNU Lesser General Public
+ *   License along with medit.  If not, see <http://www.gnu.org/licenses/>.
  */
 
+/**
+ * class:MooLineMark: (parent GObject) (constructable) (moo.private 1)
+ **/
+
 #define MOOEDIT_COMPILATION
 #include "mooedit/mootext-private.h"
 #include "mooedit/mootextbuffer.h"
 #include "marshals.h"
 #include "mooutils/mooutils-misc.h"
+#include "mooutils/moocompat.h"
 
 
-struct _MooLineMarkPrivate {
+struct MooLineMarkPrivate {
     GdkColor background;
     GdkGC *background_gc;
 
@@ -77,7 +86,8 @@ enum {
     PROP_PIXBUF,
     PROP_STOCK_ID,
     PROP_MARKUP,
-    PROP_VISIBLE
+    PROP_VISIBLE,
+    PROP_FOLD
 };
 
 
@@ -110,7 +120,7 @@ moo_line_mark_class_init (MooLineMarkClass *klass)
                                              "background-gdk",
                                              "background-gdk",
                                              GDK_TYPE_COLOR,
-                                             G_PARAM_READWRITE));
+                                             (GParamFlags) G_PARAM_READWRITE));
 
     g_object_class_install_property (gobject_class,
                                      PROP_BACKGROUND_SET,
@@ -118,7 +128,7 @@ moo_line_mark_class_init (MooLineMarkClass *klass)
                                              "background-set",
                                              "background-set",
                                              FALSE,
-                                             G_PARAM_READWRITE));
+                                             (GParamFlags) G_PARAM_READWRITE));
 
     g_object_class_install_property (gobject_class,
                                      PROP_MARKUP,
@@ -126,7 +136,7 @@ moo_line_mark_class_init (MooLineMarkClass *klass)
                                              "markup",
                                              "markup",
                                              NULL,
-                                             G_PARAM_READWRITE));
+                                             (GParamFlags) G_PARAM_READWRITE));
 
     g_object_class_install_property (gobject_class,
                                      PROP_VISIBLE,
@@ -134,7 +144,7 @@ moo_line_mark_class_init (MooLineMarkClass *klass)
                                              "visible",
                                              "visible",
                                              FALSE,
-                                             G_PARAM_READWRITE));
+                                             (GParamFlags) G_PARAM_READWRITE));
 
     g_object_class_install_property (gobject_class,
                                      PROP_PIXBUF,
@@ -142,7 +152,7 @@ moo_line_mark_class_init (MooLineMarkClass *klass)
                                              "pixbuf",
                                              "pixbuf",
                                              GDK_TYPE_PIXBUF,
-                                             G_PARAM_READWRITE));
+                                             (GParamFlags) G_PARAM_READWRITE));
 
     g_object_class_install_property (gobject_class,
                                      PROP_STOCK_ID,
@@ -150,7 +160,15 @@ moo_line_mark_class_init (MooLineMarkClass *klass)
                                              "stock-id",
                                              "stock-id",
                                              NULL,
-                                             G_PARAM_READWRITE));
+                                             (GParamFlags) G_PARAM_READWRITE));
+
+    g_object_class_install_property (gobject_class,
+                                     PROP_FOLD,
+                                     g_param_spec_object ("fold",
+                                             "fold",
+                                             "fold",
+                                             MOO_TYPE_FOLD,
+                                             G_PARAM_READABLE));
 
     signals[CHANGED] =
             g_signal_new ("changed",
@@ -272,6 +290,10 @@ moo_line_mark_get_property (GObject        *object,
             g_value_set_string (value, mark->priv->stock_id);
             break;
 
+        case PROP_FOLD:
+            g_value_set_object (value, mark->priv->fold);
+            break;
+
         default:
             G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
             break;
@@ -347,8 +369,7 @@ moo_line_mark_set_background (MooLineMark    *mark,
         if (gdk_color_parse (color, &gdk_color))
             moo_line_mark_set_background_gdk (mark, &gdk_color);
         else
-            g_warning ("%s: could not parse color '%s'",
-                       G_STRLOC, color);
+            g_warning ("could not parse color '%s'", color);
     }
     else
     {
