@@ -1,15 +1,22 @@
 /*
  *   mootextbuffer.c
  *
- *   Copyright (C) 2004-2007 by Yevgen Muntyan <muntyan@math.tamu.edu>
+ *   Copyright (C) 2004-2010 by Yevgen Muntyan <emuntyan@users.sourceforge.net>
  *
- *   This library is free software; you can redistribute it and/or
- *   modify it under the terms of the GNU Lesser General Public
- *   License version 2.1 as published by the Free Software Foundation.
+ *   This file is part of medit.  medit is free software; you can
+ *   redistribute it and/or modify it under the terms of the
+ *   GNU Lesser General Public License as published by the
+ *   Free Software Foundation; either version 2.1 of the License,
+ *   or (at your option) any later version.
  *
- *   See COPYING file that comes with this distribution.
+ *   You should have received a copy of the GNU Lesser General Public
+ *   License along with medit.  If not, see <http://www.gnu.org/licenses/>.
  */
 
+/**
+ * class:MooTextBuffer: (parent GtkTextBuffer) (moo.private 1)
+ **/
+
 #define MOOEDIT_COMPILATION
 #include "mooedit/mootextiter.h"
 #include "mooedit/mootext-private.h"
@@ -21,7 +28,7 @@
 #include <string.h>
 
 
-struct _MooTextBufferPrivate {
+struct MooTextBufferPrivate {
     gboolean has_selection;
     gboolean has_text;
 
@@ -110,6 +117,8 @@ static MooUndoAction *delete_action_new             (GtkTextBuffer      *buffer,
 static void     before_undo_redo                    (MooTextBuffer      *buffer);
 #endif
 static void     after_undo_redo                     (MooTextBuffer      *buffer);
+static void     proxy_notify_can_undo_redo          (MooTextBuffer      *buffer,
+                                                     GParamSpec         *pspec);
 
 static void     line_mark_moved                     (MooTextBuffer      *buffer,
                                                      MooLineMark        *mark);
@@ -142,7 +151,9 @@ enum {
     PROP_BRACKET_MISMATCH_STYLE,
     PROP_HAS_TEXT,
     PROP_HAS_SELECTION,
-    PROP_LANG
+    PROP_LANG,
+    PROP_CAN_UNDO,
+    PROP_CAN_REDO
 };
 
 
@@ -179,7 +190,7 @@ moo_text_buffer_class_init (MooTextBufferClass *klass)
                                              "highlight-syntax",
                                              "highlight-syntax",
                                              TRUE,
-                                             G_PARAM_READWRITE));
+                                             (GParamFlags) G_PARAM_READWRITE));
 
     g_object_class_install_property (gobject_class,
                                      PROP_HIGHLIGHT_MATCHING_BRACKETS,
@@ -187,7 +198,7 @@ moo_text_buffer_class_init (MooTextBufferClass *klass)
                                              "highlight-matching-brackets",
                                              "highlight-matching-brackets",
                                              TRUE,
-                                             G_PARAM_READWRITE));
+                                             (GParamFlags) G_PARAM_READWRITE));
 
     g_object_class_install_property (gobject_class,
                                      PROP_HIGHLIGHT_MISMATCHING_BRACKETS,
@@ -195,7 +206,7 @@ moo_text_buffer_class_init (MooTextBufferClass *klass)
                                              "highlight-mismatching-brackets",
                                              "highlight-mismatching-brackets",
                                              FALSE,
-                                             G_PARAM_READWRITE));
+                                             (GParamFlags) G_PARAM_READWRITE));
 
     g_object_class_install_property (gobject_class,
                                      PROP_BRACKET_MATCH_STYLE,
@@ -203,7 +214,7 @@ moo_text_buffer_class_init (MooTextBufferClass *klass)
                                              "bracket-match-style",
                                              "bracket-match-style",
                                              MOO_TYPE_TEXT_STYLE,
-                                             G_PARAM_CONSTRUCT | G_PARAM_WRITABLE));
+                                             (GParamFlags) (G_PARAM_CONSTRUCT | G_PARAM_WRITABLE)));
 
     g_object_class_install_property (gobject_class,
                                      PROP_BRACKET_MISMATCH_STYLE,
@@ -211,7 +222,7 @@ moo_text_buffer_class_init (MooTextBufferClass *klass)
                                              "bracket-mismatch-style",
                                              "bracket-mismatch-style",
                                              MOO_TYPE_TEXT_STYLE,
-                                             G_PARAM_CONSTRUCT | G_PARAM_WRITABLE));
+                                             (GParamFlags) (G_PARAM_CONSTRUCT | G_PARAM_WRITABLE)));
 
     g_object_class_install_property (gobject_class,
                                      PROP_HAS_TEXT,
@@ -235,7 +246,23 @@ moo_text_buffer_class_init (MooTextBufferClass *klass)
                                              "lang",
                                              "lang",
                                              MOO_TYPE_LANG,
-                                             G_PARAM_READWRITE));
+                                             (GParamFlags) G_PARAM_READWRITE));
+
+    g_object_class_install_property (gobject_class,
+                                     PROP_CAN_UNDO,
+                                     g_param_spec_boolean ("can-undo",
+                                             "can-undo",
+                                             "can-undo",
+                                             FALSE,
+                                             G_PARAM_READABLE));
+
+    g_object_class_install_property (gobject_class,
+                                     PROP_CAN_REDO,
+                                     g_param_spec_boolean ("can-redo",
+                                             "can-redo",
+                                             "can-redo",
+                                             FALSE,
+                                             G_PARAM_READABLE));
 
     signals[HIGHLIGHT_UPDATED] =
             g_signal_new ("highlight_updated",
@@ -358,6 +385,11 @@ moo_text_buffer_init (MooTextBuffer *buffer)
                            buffer, NULL,
                            G_CONNECT_AFTER | G_CONNECT_SWAPPED);
 
+    g_signal_connect_swapped (buffer->priv->undo_stack, "notify::can-undo",
+                              G_CALLBACK (proxy_notify_can_undo_redo), buffer);
+    g_signal_connect_swapped (buffer->priv->undo_stack, "notify::can-redo",
+                              G_CALLBACK (proxy_notify_can_undo_redo), buffer);
+
     moo_text_buffer_set_brackets (buffer, NULL);
 }
 
@@ -422,11 +454,12 @@ moo_text_buffer_dispose (GObject *object)
 }
 
 
-GtkTextBuffer*
+GtkTextBuffer *
 moo_text_buffer_new (GtkTextTagTable *table)
 {
-    return g_object_new (MOO_TYPE_TEXT_BUFFER,
-                         "tag-table", table, NULL);
+    return GTK_TEXT_BUFFER (g_object_new (MOO_TYPE_TEXT_BUFFER,
+                                          "tag-table", table,
+                                          (const char*) NULL));
 }
 
 
@@ -438,18 +471,18 @@ _moo_text_buffer_get_undo_stack (MooTextBuffer *buffer)
 }
 
 
-void
-moo_text_buffer_begin_not_undoable_action (MooTextBuffer *buffer)
+gboolean
+moo_text_buffer_can_redo (MooTextBuffer *buffer)
 {
-    g_return_if_fail (MOO_IS_TEXT_BUFFER (buffer));
-    moo_undo_stack_freeze (buffer->priv->undo_stack);
+    g_return_val_if_fail (MOO_IS_TEXT_BUFFER (buffer), FALSE);
+    return moo_undo_stack_can_redo (buffer->priv->undo_stack);
 }
 
-void
-moo_text_buffer_end_not_undoable_action (MooTextBuffer *buffer)
+gboolean
+moo_text_buffer_can_undo (MooTextBuffer *buffer)
 {
-    g_return_if_fail (MOO_IS_TEXT_BUFFER (buffer));
-    moo_undo_stack_thaw (buffer->priv->undo_stack);
+    g_return_val_if_fail (MOO_IS_TEXT_BUFFER (buffer), FALSE);
+    return moo_undo_stack_can_undo (buffer->priv->undo_stack);
 }
 
 
@@ -770,6 +803,13 @@ after_undo_redo (MooTextBuffer *buffer)
 #endif
 }
 
+static void
+proxy_notify_can_undo_redo (MooTextBuffer *buffer,
+                            GParamSpec    *pspec)
+{
+    g_object_notify (G_OBJECT (buffer), pspec->name);
+}
+
 
 void
 moo_text_buffer_set_lang (MooTextBuffer  *buffer,
@@ -1022,6 +1062,21 @@ moo_text_buffer_thaw (MooTextBuffer *buffer)
 }
 
 
+void
+moo_text_buffer_begin_non_undoable_action (MooTextBuffer *buffer)
+{
+    g_return_if_fail (MOO_IS_TEXT_BUFFER (buffer));
+    moo_undo_stack_freeze (buffer->priv->undo_stack);
+}
+
+void
+moo_text_buffer_end_non_undoable_action (MooTextBuffer *buffer)
+{
+    g_return_if_fail (MOO_IS_TEXT_BUFFER (buffer));
+    moo_undo_stack_thaw (buffer->priv->undo_stack);
+}
+
+
 void
 moo_text_buffer_begin_non_interactive_action (MooTextBuffer *buffer)
 {
@@ -2105,3 +2160,11 @@ moo_text_buffer_toggle_fold (MooTextBuffer *buffer,
 
     g_signal_emit (buffer, signals[FOLD_TOGGLED], 0, fold);
 }
+
+void
+moo_text_buffer_toggle_folds (MooTextBuffer *buffer)
+{
+    g_return_if_fail (MOO_IS_TEXT_BUFFER (buffer));
+    if (_moo_fold_tree_toggle (buffer->priv->fold_tree))
+        g_signal_emit (buffer, signals[FOLD_TOGGLED], 0, NULL);
+}
