@@ -1,13 +1,16 @@
 /*
  *   mootextfind.c
  *
- *   Copyright (C) 2004-2007 by Yevgen Muntyan <muntyan@math.tamu.edu>
+ *   Copyright (C) 2004-2010 by Yevgen Muntyan <emuntyan@users.sourceforge.net>
  *
- *   This library is free software; you can redistribute it and/or
- *   modify it under the terms of the GNU Lesser General Public
- *   License version 2.1 as published by the Free Software Foundation.
+ *   This file is part of medit.  medit is free software; you can
+ *   redistribute it and/or modify it under the terms of the
+ *   GNU Lesser General Public License as published by the
+ *   Free Software Foundation; either version 2.1 of the License,
+ *   or (at your option) any later version.
  *
- *   See COPYING file that comes with this distribution.
+ *   You should have received a copy of the GNU Lesser General Public
+ *   License along with medit.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "mooedit/mootextfind.h"
@@ -21,9 +24,10 @@
 #include "mooutils/moodialogs.h"
 #include "mooutils/mooi18n.h"
 #include "mooutils/moohelp.h"
-#include "glade/mootextfind-gxml.h"
-#include "glade/mootextgotoline-gxml.h"
-#include "help-sections.h"
+#include "mooutils/moocompat.h"
+#include "mooedit/mootextfind-gxml.h"
+#include "mooedit/mootextgotoline-gxml.h"
+#include "moo-help-sections.h"
 #include <gtk/gtk.h>
 #include <glib/gprintf.h>
 
@@ -91,8 +95,8 @@ init_find_history (void)
         last_search = moo_history_list_get_last_item (search_history);
 
         moo_prefs_create_key (moo_edit_setting (MOO_EDIT_PREFS_SEARCH_FLAGS), MOO_PREFS_STATE,
-                                                MOO_TYPE_FIND_FLAGS, MOO_FIND_CASELESS);
-        last_search_flags = moo_prefs_get_flags (moo_edit_setting (MOO_EDIT_PREFS_SEARCH_FLAGS));
+                                                G_TYPE_INT, MOO_FIND_CASELESS);
+        last_search_flags = moo_prefs_get_int (moo_edit_setting (MOO_EDIT_PREFS_SEARCH_FLAGS));
     }
 }
 
@@ -115,7 +119,7 @@ moo_find_class_init (MooFindClass *klass)
                                              "replace",
                                              "replace",
                                              TRUE,
-                                             G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+                                             (GParamFlags) (G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY)));
 }
 
 
@@ -158,13 +162,13 @@ moo_find_constructor (GType           type,
     {
         use_replace = TRUE;
         stock_id = GTK_STOCK_FIND_AND_REPLACE;
-        title = "Replace";
+        title = C_("Dialog title", "Replace");
     }
     else
     {
         use_replace = FALSE;
         stock_id = GTK_STOCK_FIND;
-        title = "Find";
+        title = C_("Dialog title", "Find");
     }
 
     gtk_widget_set_sensitive (GTK_WIDGET (find->xml->backwards), !use_replace);
@@ -246,7 +250,7 @@ moo_find_finalize (GObject *object)
 static GtkWidget *
 moo_find_new (gboolean replace)
 {
-    return g_object_new (MOO_TYPE_FIND, "replace", replace, NULL);
+    return GTK_WIDGET (g_object_new (MOO_TYPE_FIND, "replace", replace, (const char*) NULL));
 }
 
 
@@ -509,7 +513,7 @@ moo_find_run (MooFind        *find,
         }
 
         last_search_flags = flags;
-        moo_prefs_set_flags (moo_edit_setting (MOO_EDIT_PREFS_SEARCH_FLAGS), flags);
+        moo_prefs_set_int (moo_edit_setting (MOO_EDIT_PREFS_SEARCH_FLAGS), flags);
         g_free (last_search);
         last_search = g_strdup (search_for);
         REGEX_FREE (last_regex);
@@ -1360,6 +1364,7 @@ moo_text_view_run_goto_line (GtkTextView *view)
         return;
     }
 
+    gtk_spin_button_update (xml->spin);
     line = gtk_spin_button_get_value (xml->spin) - 1;
     gtk_widget_destroy (GTK_WIDGET (dialog));
 
