@@ -1,27 +1,31 @@
 /*
  *   mootextprint.c
  *
- *   Copyright (C) 2004-2007 by Yevgen Muntyan <muntyan@math.tamu.edu>
+ *   Copyright (C) 2004-2010 by Yevgen Muntyan <emuntyan@users.sourceforge.net>
  *
- *   This library is free software; you can redistribute it and/or
- *   modify it under the terms of the GNU Lesser General Public
- *   License version 2.1 as published by the Free Software Foundation.
+ *   This file is part of medit.  medit is free software; you can
+ *   redistribute it and/or modify it under the terms of the
+ *   GNU Lesser General Public License as published by the
+ *   Free Software Foundation; either version 2.1 of the License,
+ *   or (at your option) any later version.
  *
- *   See COPYING file that comes with this distribution.
+ *   You should have received a copy of the GNU Lesser General Public
+ *   License along with medit.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #define MOOEDIT_COMPILATION
 #include "mooedit/mootextprint-private.h"
 #include "mooedit/mooedit.h"
-#include "mooedit/mooedit-private.h"
+#include "mooedit/mooedit-impl.h"
+#include "mooedit/mooeditview-impl.h"
 #include "mooedit/mooeditprefs.h"
 #include "mooedit/mootext-private.h"
-#include "mooedit/mooprintpreview.h"
 #include "mooutils/moodialogs.h"
 #include "mooutils/mooi18n.h"
 #include "mooutils/mooutils-misc.h"
 #include "mooutils/mooutils-debug.h"
-#include "glade/mooprint-gxml.h"
+#include "mooutils/mootype-macros.h"
+#include "mooedit/mooprint-gxml.h"
 #include <sys/types.h>
 #include <time.h>
 #include <errno.h>
@@ -32,7 +36,7 @@
 #include <cairo-win32.h>
 #endif
 
-MOO_DEBUG_INIT(printing, TRUE)
+MOO_DEBUG_INIT (printing, FALSE)
 
 #define SEPARATOR_POINTS 1.
 
@@ -67,13 +71,11 @@ typedef struct {
     double ln_space;
 } Page;
 
-struct _MooPrintOperationPrivate {
+struct MooPrintOperationPrivate {
     GtkWindow *parent;
     GtkTextView *doc;
     GtkTextBuffer *buffer;
 
-    MooPrintPreview *preview;
-
     /* print settings */
     int first_line;
     int last_line;          /* -1 to print everything after first_line */
@@ -103,7 +105,7 @@ G_STMT_START {                                      \
 #define GET_OPTION(op, opt) (((op)->priv->settings->flags & (opt)) != 0)
 
 
-typedef struct _HFFormat HFFormat;
+typedef struct HFFormat HFFormat;
 static HFFormat *hf_format_parse    (const char         *strformat);
 static void      hf_format_free     (HFFormat           *format);
 static char     *hf_format_eval     (HFFormat           *format,
@@ -137,10 +139,6 @@ static void moo_print_operation_draw_page   (GtkPrintOperation  *operation,
 static void moo_print_operation_end_print   (GtkPrintOperation  *operation,
                                              GtkPrintContext    *context);
 static void moo_print_operation_status_changed (GtkPrintOperation *operation);
-static gboolean moo_print_operation_preview (GtkPrintOperation  *operation,
-                                             GtkPrintOperationPreview *preview,
-                                             GtkPrintContext    *context,
-                                             GtkWindow          *parent);
 static GtkWidget *moo_print_operation_create_custom_widget (GtkPrintOperation *operation);
 static void moo_print_operation_custom_widget_apply (GtkPrintOperation *operation,
                                                      GtkWidget *widget);
@@ -260,7 +258,7 @@ moo_print_settings_new_default (void)
 
 
 static MooPrintSettings *
-moo_print_settings_copy (MooPrintSettings *settings)
+_moo_print_settings_copy (MooPrintSettings *settings)
 {
     MooPrintSettings *copy;
 
@@ -276,9 +274,8 @@ moo_print_settings_copy (MooPrintSettings *settings)
     return copy;
 }
 
-
 static void
-moo_print_settings_free (MooPrintSettings *settings)
+_moo_print_settings_free (MooPrintSettings *settings)
 {
     if (settings)
     {
@@ -290,19 +287,7 @@ moo_print_settings_free (MooPrintSettings *settings)
     }
 }
 
-
-GType
-_moo_print_settings_get_type (void)
-{
-    static GType t = 0;
-
-    if (G_UNLIKELY (!t))
-        t = g_boxed_type_register_static ("MooPrintSettings",
-                                          (GBoxedCopyFunc) moo_print_settings_copy,
-                                          (GBoxedFreeFunc) moo_print_settings_free);
-
-    return t;
-}
+MOO_DEFINE_BOXED_TYPE_C (MooPrintSettings, _moo_print_settings)
 
 
 static void
@@ -313,10 +298,10 @@ moo_print_operation_set_settings (MooPrintOperation *op,
 
     if (op->priv->settings != settings)
     {
-        moo_print_settings_free (op->priv->settings);
+        _moo_print_settings_free (op->priv->settings);
 
         if (settings)
-            op->priv->settings = moo_print_settings_copy (settings);
+            op->priv->settings = _moo_print_settings_copy (settings);
         else
             op->priv->settings = moo_print_settings_new_default ();
 
@@ -338,7 +323,7 @@ moo_print_operation_finalize (GObject *object)
     if (op->priv->buffer)
         g_object_unref (op->priv->buffer);
 
-    moo_print_settings_free (op->priv->settings);
+    _moo_print_settings_free (op->priv->settings);
     g_free (op->priv->filename);
     g_free (op->priv->basename);
     g_free (op->priv->tm);
@@ -422,7 +407,6 @@ _moo_print_operation_class_init (MooPrintOperationClass *klass)
     print_class->create_custom_widget = moo_print_operation_create_custom_widget;
     print_class->custom_widget_apply = moo_print_operation_custom_widget_apply;
     print_class->status_changed = moo_print_operation_status_changed;
-    print_class->preview = moo_print_operation_preview;
 
     g_object_class_install_property (object_class,
                                      PROP_DOC,
@@ -430,7 +414,7 @@ _moo_print_operation_class_init (MooPrintOperationClass *klass)
                                              "doc",
                                              "doc",
                                              GTK_TYPE_TEXT_VIEW,
-                                             G_PARAM_READWRITE));
+                                             (GParamFlags) G_PARAM_READWRITE));
 
     g_object_class_install_property (object_class,
                                      PROP_BUFFER,
@@ -438,7 +422,7 @@ _moo_print_operation_class_init (MooPrintOperationClass *klass)
                                              "buffer",
                                              "buffer",
                                              GTK_TYPE_TEXT_BUFFER,
-                                             G_PARAM_READWRITE));
+                                             (GParamFlags) G_PARAM_READWRITE));
 
     g_object_class_install_property (object_class,
                                      PROP_SETTINGS,
@@ -446,7 +430,7 @@ _moo_print_operation_class_init (MooPrintOperationClass *klass)
                                              "settings",
                                              "settings",
                                              MOO_TYPE_PRINT_SETTINGS,
-                                             G_PARAM_READWRITE));
+                                             (GParamFlags) G_PARAM_READWRITE));
 }
 
 
@@ -722,7 +706,7 @@ moo_print_operation_calc_page_size (MooPrintOperation    *op,
 
     if (page->height < 0)
     {
-        g_critical ("%s: page messed up", G_STRLOC);
+        g_critical ("page messed up");
         page->y = 0.;
         page->height = gtk_print_context_get_height (context);
     }
@@ -931,18 +915,14 @@ moo_print_operation_begin_print (GtkPrintOperation *operation,
 
     timer = g_timer_new ();
 
-    if (MOO_IS_EDIT (op->priv->doc))
+    if (MOO_IS_EDIT_VIEW (op->priv->doc))
     {
-        if (!op->priv->preview)
-            _moo_edit_set_state (MOO_EDIT (op->priv->doc),
-                                 MOO_EDIT_STATE_PRINTING,
-                                 "Printing",
-                                 (GDestroyNotify) gtk_print_operation_cancel,
-                                 op);
-        else
-            _moo_edit_set_state (MOO_EDIT (op->priv->doc),
-                                 MOO_EDIT_STATE_NORMAL,
-                                 NULL, NULL, NULL);
+        MooEdit *doc = moo_edit_view_get_doc (MOO_EDIT_VIEW (op->priv->doc));
+        _moo_edit_set_state (doc,
+                             MOO_EDIT_STATE_PRINTING,
+                             "Printing",
+                             (GDestroyNotify) gtk_print_operation_cancel,
+                             op);
     }
 
     if (op->priv->last_line < 0)
@@ -1019,9 +999,6 @@ moo_print_operation_begin_print (GtkPrintOperation *operation,
         op->priv->tm = NULL;
     }
 
-    if (op->priv->preview)
-        _moo_print_preview_start (op->priv->preview);
-
     moo_dmsg ("moo_print_operation_begin_print done");
 }
 
@@ -1469,7 +1446,7 @@ moo_print_operation_draw_page (GtkPrintOperation *operation,
     }
 #endif
 
-#if 0 && defined(MOO_DEBUG_ENABLED) && !defined(__WIN32__)
+#if 0 && defined(MOO_DEBUG) && !defined(__WIN32__)
     cairo_save (cr);
     cairo_set_line_width (cr, 1.);
     cairo_set_source_rgb (cr, 1., 0., 0.);
@@ -1511,8 +1488,8 @@ moo_print_operation_end_print (GtkPrintOperation  *operation,
 
     moo_dmsg ("moo_print_operation_end_print");
 
-    if (MOO_IS_EDIT (op->priv->doc))
-        _moo_edit_set_state (MOO_EDIT (op->priv->doc),
+    if (MOO_IS_EDIT_VIEW (op->priv->doc))
+        _moo_edit_set_state (moo_edit_view_get_doc (MOO_EDIT_VIEW (op->priv->doc)),
                              MOO_EDIT_STATE_NORMAL,
                              NULL, NULL, NULL);
 
@@ -1531,13 +1508,15 @@ update_progress (GtkPrintOperation *operation,
     MooPrintOperation *op = MOO_PRINT_OPERATION (operation);
     GtkPrintStatus status;
     MooEditWindow *window;
+    MooEditView *view;
     MooEdit *doc;
 
-    if (!MOO_IS_EDIT (op->priv->doc))
+    if (!MOO_IS_EDIT_VIEW (op->priv->doc))
         return;
 
-    doc = MOO_EDIT (op->priv->doc);
-    window = moo_edit_get_window (doc);
+    view = MOO_EDIT_VIEW (op->priv->doc);
+    doc = moo_edit_view_get_doc (view);
+    window = moo_edit_view_get_window (view);
     status = gtk_print_operation_get_status (operation);
 
     if (status == GTK_PRINT_STATUS_FINISHED)
@@ -1558,7 +1537,7 @@ update_progress (GtkPrintOperation *operation,
     if (window)
         moo_window_message (MOO_WINDOW (window), NULL);
 
-    if (moo_edit_get_state (doc) == MOO_EDIT_STATE_PRINTING)
+    if (_moo_edit_get_state (doc) == MOO_EDIT_STATE_PRINTING)
         _moo_edit_set_progress_text (doc, text);
     else if (window)
         moo_window_message (MOO_WINDOW (window), text);
@@ -1627,7 +1606,7 @@ moo_print_operation_load_prefs (MooPrintOperation *op)
 {
     MooPrintSettings *settings = get_settings_from_prefs ();
     moo_print_operation_set_settings (op, settings);
-    moo_print_settings_free (settings);
+    _moo_print_settings_free (settings);
 }
 
 
@@ -1645,10 +1624,10 @@ do_print_operation (GtkTextView            *view,
     g_return_if_fail (GTK_IS_TEXT_VIEW (view));
     g_return_if_fail (!exp_filename || action == GTK_PRINT_OPERATION_ACTION_EXPORT);
 
-    op = g_object_new (MOO_TYPE_PRINT_OPERATION,
-                       "doc", view,
-                       "export-filename", exp_filename,
-                       NULL);
+    op = MOO_PRINT_OPERATION (g_object_new (MOO_TYPE_PRINT_OPERATION,
+                                            "doc", view,
+                                            "export-filename", exp_filename,
+                                            (const char*) NULL));
 
     if (!parent)
         parent = GTK_WIDGET (view);
@@ -1659,11 +1638,12 @@ do_print_operation (GtkTextView            *view,
     if (GTK_IS_WINDOW (parent))
         op->priv->parent = GTK_WINDOW (parent);
 
-    if (MOO_IS_EDIT (view))
+    if (MOO_IS_EDIT_VIEW (view))
     {
+        MooEdit *doc = moo_edit_view_get_doc (MOO_EDIT_VIEW (view));
         const char *filename, *basename;
-        filename = moo_edit_get_display_name (MOO_EDIT (view));
-        basename = moo_edit_get_display_basename (MOO_EDIT (view));
+        filename = moo_edit_get_display_name (doc);
+        basename = moo_edit_get_display_basename (doc);
         moo_print_operation_set_filename (op, filename, basename);
     }
 
@@ -1678,7 +1658,7 @@ do_print_operation (GtkTextView            *view,
                                                    GTK_MESSAGE_ERROR,
                                                    GTK_BUTTONS_CLOSE,
                                                    "Error:\n%s",
-                                                   error ? error->message : "");
+                                                   moo_error_message (error));
             gtk_dialog_run (GTK_DIALOG (error_dialog));
             gtk_widget_destroy (error_dialog);
             g_error_free (error);
@@ -1696,12 +1676,6 @@ do_print_operation (GtkTextView            *view,
             break;
     }
 
-#if !GTK_CHECK_VERSION(2,11,6)
-    /* older gtk crashes in unref() */
-    if (action == GTK_PRINT_OPERATION_ACTION_EXPORT)
-        g_object_ref (op);
-#endif
-
     g_object_unref (op);
 }
 
@@ -1730,16 +1704,6 @@ _moo_edit_export_pdf (GtkTextView *view,
 }
 
 
-void
-_moo_print_operation_set_preview (MooPrintOperation *op,
-                                  MooPrintPreview   *preview)
-{
-    g_return_if_fail (MOO_IS_PRINT_OPERATION (op));
-    g_return_if_fail (!preview || MOO_IS_PRINT_PREVIEW (preview));
-    op->priv->preview = preview;
-}
-
-
 GtkWindow *
 _moo_print_operation_get_parent (MooPrintOperation *op)
 {
@@ -1763,7 +1727,7 @@ _moo_print_operation_get_n_pages (MooPrintOperation *op)
 #define SET_TEXT(wid,setting)                                           \
 G_STMT_START {                                                          \
     const char *s__ = moo_prefs_get_string (setting);                   \
-    gtk_entry_set_text (xml->wid, s__ ? s__ : "");                      \
+    gtk_entry_set_text (xml->wid, MOO_NZS (s__));                       \
 } G_STMT_END
 
 #define GET_TEXT(wid,setting)                                           \
@@ -1867,7 +1831,9 @@ get_options (PrintWidgetXml *xml)
     if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (xml->line_numbers)))
     {
         GtkSpinButton *btn = xml->line_numbers_step;
-        int step = gtk_spin_button_get_value_as_int (btn);
+        int step;
+        gtk_spin_button_update (btn);
+        step = gtk_spin_button_get_value_as_int (btn);
         if (step < 1)
         {
             step = 1;
@@ -1911,14 +1877,14 @@ static GtkWidget *
 moo_print_operation_create_custom_widget (G_GNUC_UNUSED GtkPrintOperation *operation)
 {
     PrintWidgetXml *xml;
-    GtkWidget *font, *line_numbers_hbox;
 
     xml = print_widget_xml_new ();
 
-    font = GTK_WIDGET (xml->font);
-    moo_bind_sensitive (GTK_WIDGET (xml->use_custom_font), &font, 1, FALSE);
-    line_numbers_hbox = GTK_WIDGET (xml->line_numbers_hbox);
-    moo_bind_sensitive (GTK_WIDGET (xml->line_numbers), &line_numbers_hbox, 1, FALSE);
+    moo_bind_sensitive (GTK_WIDGET (xml->wrap), GTK_WIDGET (xml->ellipsize), TRUE);
+    moo_bind_sensitive (GTK_WIDGET (xml->print_header), GTK_WIDGET (xml->header_alignment), FALSE);
+    moo_bind_sensitive (GTK_WIDGET (xml->print_footer), GTK_WIDGET (xml->footer_alignment), FALSE);
+    moo_bind_sensitive (GTK_WIDGET (xml->use_custom_font), GTK_WIDGET (xml->font), FALSE);
+    moo_bind_sensitive (GTK_WIDGET (xml->line_numbers), GTK_WIDGET (xml->line_numbers_hbox), FALSE);
 
     set_options (xml);
     return GTK_WIDGET (xml->PrintWidget);
@@ -1933,47 +1899,6 @@ moo_print_operation_custom_widget_apply (G_GNUC_UNUSED GtkPrintOperation *print,
 }
 
 
-/*****************************************************************************/
-/* Preview
- */
-
-static void
-preview_response (MooPrintOperation *op,
-                  int                response,
-                  MooPrintPreview   *preview)
-{
-    gtk_widget_hide (GTK_WIDGET (preview));
-    op->priv->preview = NULL;
-
-    gtk_print_operation_preview_end_preview (_moo_print_preview_get_gtk_preview (preview));
-
-    if (response == MOO_PRINT_PREVIEW_RESPONSE_PRINT)
-        _moo_edit_print (op->priv->doc,
-                         op->priv->parent ? GTK_WIDGET (op->priv->parent) : NULL);
-
-    gtk_widget_destroy (GTK_WIDGET (preview));
-}
-
-static gboolean
-moo_print_operation_preview (GtkPrintOperation        *op,
-                             GtkPrintOperationPreview *preview,
-                             GtkPrintContext          *context,
-                             G_GNUC_UNUSED GtkWindow  *parent)
-{
-    GtkWidget *dialog;
-
-    moo_dmsg ("moo_print_operation_preview");
-
-    dialog = _moo_print_preview_new (MOO_PRINT_OPERATION (op), preview, context);
-    gtk_widget_show (dialog);
-    g_signal_connect_swapped (dialog, "response", G_CALLBACK (preview_response), op);
-
-    moo_dmsg ("moo_print_operation_preview done");
-
-    return TRUE;
-}
-
-
 /*****************************************************************************/
 /* Format string
  */
@@ -1991,7 +1916,7 @@ typedef struct {
     char *string;
 } HFFormatChunk;
 
-struct _HFFormat {
+struct HFFormat {
     GSList *chunks;
 };
 
@@ -2062,8 +1987,8 @@ hf_format_parse (const char *format_string)
                 switch (p[2])
                 {
                     case 0:
-                        g_warning ("%s: trailing '%%E' in %s",
-                                   G_STRLOC, format_string);
+                        g_warning ("trailing '%%E' in %s",
+                                   format_string);
                         goto error;
                     case 'f':
                         ADD_CHUNK (format, HF_FORMAT_BASENAME, NULL, 0);
@@ -2082,8 +2007,8 @@ hf_format_parse (const char *format_string)
                         str = p = p + 3;
                         break;
                     default:
-                        g_warning ("%s: unknown format '%%E%c' in %s",
-                                   G_STRLOC, p[2], format_string);
+                        g_warning ("unknown format '%%E%c' in %s",
+                                   p[2], format_string);
                         goto error;
                 }
                 break;
@@ -2123,7 +2048,7 @@ eval_strftime (GString    *dest,
 
     if (!result)
     {
-        g_warning ("%s: strftime failed", G_STRLOC);
+        g_warning ("strftime failed");
         return;
     }
 
@@ -2131,7 +2056,7 @@ eval_strftime (GString    *dest,
 
     if (!retval)
     {
-        g_warning ("%s: could not convert result of strftime to UTF8", G_STRLOC);
+        g_warning ("could not convert result of strftime to UTF8");
         return;
     }
 
@@ -2172,13 +2097,13 @@ hf_format_eval (HFFormat         *format,
                 break;
             case HF_FORMAT_FILENAME:
                 if (!filename)
-                    g_critical ("%s: filename is NULL", G_STRLOC);
+                    g_critical ("filename is NULL");
                 else
                     g_string_append (string, filename);
                 break;
             case HF_FORMAT_BASENAME:
                 if (!basename)
-                    g_critical ("%s: basename is NULL", G_STRLOC);
+                    g_critical ("basename is NULL");
                 else
                     g_string_append (string, basename);
                 break;
