@@ -1,13 +1,16 @@
 /*
  *   moobookmarkmgr.c
  *
- *   Copyright (C) 2004-2007 by Yevgen Muntyan <muntyan@math.tamu.edu>
+ *   Copyright (C) 2004-2010 by Yevgen Muntyan <emuntyan@users.sourceforge.net>
  *
- *   This library is free software; you can redistribute it and/or
- *   modify it under the terms of the GNU Lesser General Public
- *   License version 2.1 as published by the Free Software Foundation.
+ *   This file is part of medit.  medit is free software; you can
+ *   redistribute it and/or modify it under the terms of the
+ *   GNU Lesser General Public License as published by the
+ *   Free Software Foundation; either version 2.1 of the License,
+ *   or (at your option) any later version.
  *
- *   See COPYING file that comes with this distribution.
+ *   You should have received a copy of the GNU Lesser General Public
+ *   License along with medit.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -22,17 +25,15 @@
 #include "mooutils/mooactionfactory.h"
 #include "mooutils/mooutils-misc.h"
 #include "mooutils/moostock.h"
-#include "glade/moobookmark-editor-gxml.h"
+#include "mooutils/mootype-macros.h"
+#include "moofileview/moobookmark-editor-gxml.h"
 #include <string.h>
 #include <errno.h>
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
 #include <gtk/gtk.h>
-
-#if GLIB_CHECK_VERSION(2,6,0)
-# include <glib/gstdio.h>
-#endif
+#include <glib/gstdio.h>
 
 #define COLUMN_BOOKMARK MOO_BOOKMARK_MGR_COLUMN_BOOKMARK
 
@@ -63,7 +64,7 @@ static gboolean mgr_update_menus            (MooBookmarkMgr *mgr);
 static MooBookmark *_moo_bookmark_copy      (MooBookmark    *bookmark);
 
 
-/* MOO_TYPE_BOOKMARK_MGR */
+MOO_DEFINE_BOXED_TYPE_C (MooBookmark, _moo_bookmark)
 G_DEFINE_TYPE (MooBookmarkMgr, _moo_bookmark_mgr, G_TYPE_OBJECT)
 
 enum {
@@ -165,7 +166,7 @@ moo_bookmark_mgr_changed (MooBookmarkMgr *mgr)
     if (!mgr->priv->loading)
         moo_bookmark_mgr_save (mgr);
     if (!mgr->priv->update_idle)
-        mgr->priv->update_idle = moo_idle_add ((GSourceFunc) mgr_update_menus, mgr);
+        mgr->priv->update_idle = gdk_threads_add_idle ((GSourceFunc) mgr_update_menus, mgr);
 }
 
 
@@ -195,10 +196,10 @@ moo_bookmark_mgr_add_separator (MooBookmarkMgr *mgr)
 }
 
 
-MooBookmarkMgr*
+MooBookmarkMgr *
 _moo_bookmark_mgr_new (void)
 {
-    MooBookmarkMgr *mgr = g_object_new (MOO_TYPE_BOOKMARK_MGR, NULL);
+    MooBookmarkMgr *mgr = MOO_BOOKMARK_MGR (g_object_new (MOO_TYPE_BOOKMARK_MGR, (const char*) NULL));
     moo_bookmark_mgr_load (mgr);
     return mgr;
 }
@@ -266,18 +267,6 @@ _moo_bookmark_free (MooBookmark *bookmark)
 }
 
 
-GType
-_moo_bookmark_get_type (void)
-{
-    static GType type = 0;
-    if (G_UNLIKELY (!type))
-        type = g_boxed_type_register_static ("MooBookmark",
-                                             (GBoxedCopyFunc) _moo_bookmark_copy,
-                                             (GBoxedFreeFunc) _moo_bookmark_free);
-    return type;
-}
-
-
 #if 0
 void
 _moo_bookmark_set_path (MooBookmark    *bookmark,
@@ -328,13 +317,13 @@ _moo_bookmark_set_display_path (MooBookmark  *bookmark,
 static void
 moo_bookmark_mgr_load (MooBookmarkMgr *mgr)
 {
-    MooMarkupDoc *xml;
+    MooMarkupNode *xml;
     MooMarkupNode *root, *node;
 
     xml = moo_prefs_get_markup (MOO_PREFS_RC);
     g_return_if_fail (xml != NULL);
 
-    root = moo_markup_get_element (MOO_MARKUP_NODE (xml), BOOKMARKS_ROOT);
+    root = moo_markup_get_element (xml, BOOKMARKS_ROOT);
 
     if (!root)
         return;
@@ -356,7 +345,7 @@ moo_bookmark_mgr_load (MooBookmarkMgr *mgr)
 
             if (!path_utf8 || !path_utf8[0])
             {
-                g_warning ("%s: empty path in bookmark", G_STRLOC);
+                g_warning ("empty path in bookmark");
                 continue;
             }
 
@@ -364,8 +353,7 @@ moo_bookmark_mgr_load (MooBookmarkMgr *mgr)
 
             if (!path)
             {
-                g_warning ("%s: could not convert '%s' to filename encoding",
-                           G_STRLOC, path_utf8);
+                g_warning ("could not convert '%s' to filename encoding", path_utf8);
                 continue;
             }
 
@@ -381,7 +369,7 @@ moo_bookmark_mgr_load (MooBookmarkMgr *mgr)
         }
         else
         {
-            g_warning ("%s: invalid '%s' element", G_STRLOC, node->name);
+            g_warning ("invalid '%s' element", node->name);
         }
     }
 
@@ -392,7 +380,7 @@ moo_bookmark_mgr_load (MooBookmarkMgr *mgr)
 static void
 moo_bookmark_mgr_save (MooBookmarkMgr *mgr)
 {
-    MooMarkupDoc *xml;
+    MooMarkupNode *xml;
     MooMarkupNode *root;
     GtkTreeModel *model;
     GtkTreeIter iter;
@@ -402,7 +390,7 @@ moo_bookmark_mgr_save (MooBookmarkMgr *mgr)
 
     model = GTK_TREE_MODEL (mgr->priv->store);
 
-    root = moo_markup_get_element (MOO_MARKUP_NODE (xml), BOOKMARKS_ROOT);
+    root = moo_markup_get_element (xml, BOOKMARKS_ROOT);
 
     if (root)
         moo_markup_delete_node (root);
@@ -410,7 +398,7 @@ moo_bookmark_mgr_save (MooBookmarkMgr *mgr)
     if (!gtk_tree_model_get_iter_first (model, &iter))
         return;
 
-    root = moo_markup_create_element (MOO_MARKUP_NODE (xml), BOOKMARKS_ROOT);
+    root = moo_markup_create_element (xml, BOOKMARKS_ROOT);
     g_return_if_fail (root != NULL);
 
     do
@@ -445,7 +433,7 @@ moo_bookmark_mgr_save (MooBookmarkMgr *mgr)
 
 struct _UserInfo {
     GObject *user;
-    MooUIXML *xml;
+    MooUiXml *xml;
     MooActionCollection *actions;
     char *path;
     guint user_id;
@@ -456,7 +444,7 @@ struct _UserInfo {
 static UserInfo*
 user_info_new (GObject             *user,
                MooActionCollection *actions,
-               MooUIXML            *xml,
+               MooUiXml            *xml,
                const char          *path,
                guint                user_id)
 {
@@ -621,7 +609,7 @@ void
 _moo_bookmark_mgr_add_user (MooBookmarkMgr *mgr,
                             gpointer        user,
                             MooActionCollection *actions,
-                            MooUIXML       *xml,
+                            MooUiXml       *xml,
                             const char     *path)
 {
     UserInfo *info;
@@ -824,7 +812,7 @@ _moo_bookmark_mgr_get_editor (MooBookmarkMgr *mgr)
                       G_CALLBACK (dialog_show), mgr);
 
     mgr->priv->editor = dialog;
-    MOO_OBJECT_REF_SINK (dialog);
+    g_object_ref_sink (dialog);
 
     return dialog;
 }
@@ -938,8 +926,6 @@ static void     path_edited         (GtkCellRenderer    *cell,
                                      BkEditorXml        *xml);
 static void     path_editing_started(GtkCellRenderer    *cell,
                                      GtkCellEditable    *editable);
-static void     path_entry_realize  (GtkWidget          *entry);
-static void     path_entry_unrealize(GtkWidget          *entry);
 
 static void     init_icon_combo     (GtkComboBox        *combo,
                                      BkEditorXml        *xml);
@@ -972,7 +958,8 @@ init_editor_dialog (BkEditorXml *xml)
 
     /* Icon */
     cell = gtk_cell_renderer_pixbuf_new ();
-    column = gtk_tree_view_column_new_with_attributes ("Icon", cell, NULL);
+    /* Column label in file selector bookmark editor */
+    column = gtk_tree_view_column_new_with_attributes (C_("fileview-bookmark-editor", "Icon"), cell, NULL);
     gtk_tree_view_column_set_cell_data_func (column, cell,
                                              (GtkTreeCellDataFunc) icon_data_func,
                                              NULL, NULL);
@@ -987,7 +974,8 @@ init_editor_dialog (BkEditorXml *xml)
     g_signal_connect (cell, "edited",
                       G_CALLBACK (label_edited), xml);
 
-    column = gtk_tree_view_column_new_with_attributes ("Label", cell, NULL);
+    /* Column label in file selector bookmark editor */
+    column = gtk_tree_view_column_new_with_attributes (C_("fileview-bookmark-editor", "Label"), cell, NULL);
     gtk_tree_view_column_set_cell_data_func (column, cell,
                                              (GtkTreeCellDataFunc) label_data_func,
                                              NULL, NULL);
@@ -1004,7 +992,8 @@ init_editor_dialog (BkEditorXml *xml)
     g_signal_connect (cell, "editing-started",
                       G_CALLBACK (path_editing_started), NULL);
 
-    column = gtk_tree_view_column_new_with_attributes ("Path", cell, NULL);
+    /* Column label in file selector bookmark editor */
+    column = gtk_tree_view_column_new_with_attributes (C_("fileview-bookmark-editor", "Path"), cell, NULL);
     gtk_tree_view_column_set_cell_data_func (column, cell,
                                              (GtkTreeCellDataFunc) path_data_func,
                                              NULL, NULL);
@@ -1013,10 +1002,11 @@ init_editor_dialog (BkEditorXml *xml)
                        "moo-bookmarks-path-column",
                        column);
 
-    completion = g_object_new (MOO_TYPE_FILE_ENTRY_COMPLETION,
-                               "directories-only", TRUE,
-                               "show-hidden", FALSE,
-                               NULL);
+    completion = MOO_FILE_ENTRY_COMPLETION (
+        g_object_new (MOO_TYPE_FILE_ENTRY_COMPLETION,
+                      "directories-only", TRUE,
+                      "show-hidden", FALSE,
+                      (const char*) NULL));
     g_object_set_data_full (G_OBJECT (cell), "moo-file-entry-completion",
                             completion, g_object_unref);
 }
@@ -1341,6 +1331,7 @@ path_editing_started (GtkCellRenderer    *cell,
 }
 
 
+#if 0
 static void
 path_entry_realize (GtkWidget *entry)
 {
@@ -1390,6 +1381,7 @@ path_entry_unrealize (GtkWidget *entry)
                                           NULL);
     g_object_set_data (G_OBJECT (entry), "moo-stupid-entry-workaround", NULL);
 }
+#endif
 
 
 static void combo_icon_data_func    (GtkCellLayout      *cell_layout,
