@@ -1,13 +1,16 @@
 /*
  *   moofilesystem.c
  *
- *   Copyright (C) 2004-2007 by Yevgen Muntyan <muntyan@math.tamu.edu>
+ *   Copyright (C) 2004-2010 by Yevgen Muntyan <emuntyan@users.sourceforge.net>
  *
- *   This library is free software; you can redistribute it and/or
- *   modify it under the terms of the GNU Lesser General Public
- *   License version 2.1 as published by the Free Software Foundation.
+ *   This file is part of medit.  medit is free software; you can
+ *   redistribute it and/or modify it under the terms of the
+ *   GNU Lesser General Public License as published by the
+ *   Free Software Foundation; either version 2.1 of the License,
+ *   or (at your option) any later version.
  *
- *   See COPYING file that comes with this distribution.
+ *   You should have received a copy of the GNU Lesser General Public
+ *   License along with medit.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <config.h>
@@ -17,7 +20,9 @@
 #include "moofolder-private.h"
 #include "mooutils/mooutils-fs.h"
 #include "mooutils/mooutils-mem.h"
+#include "mooutils/mooutils.h"
 #include "marshals.h"
+#include <gio/gio.h>
 #include <errno.h>
 #include <stdio.h>
 #ifndef __WIN32__
@@ -28,7 +33,7 @@
 #include <shellapi.h>
 #endif
 
-#if 0 && MOO_DEBUG_ENABLED
+#if 0 && MOO_DEBUG
 #define DEBUG_MESSAGE g_message
 #else
 static void G_GNUC_PRINTF(1,2) DEBUG_MESSAGE (G_GNUC_UNUSED const char *format, ...)
@@ -72,7 +77,7 @@ static MooFolder   *get_parent_folder       (MooFileSystem  *fs,
                                              MooFileFlags    flags);
 static gboolean     delete_file             (MooFileSystem  *fs,
                                              const char     *path,
-                                             gboolean        recursive,
+                                             MooDeleteFileFlags flags,
                                              GError        **error);
 
 static gboolean     move_file_unix          (MooFileSystem  *fs,
@@ -278,7 +283,7 @@ _moo_file_system_create (void)
 {
     if (!fs_instance)
     {
-        fs_instance = MOO_FILE_SYSTEM (g_object_new (MOO_TYPE_FILE_SYSTEM, NULL));
+        fs_instance = MOO_FILE_SYSTEM (g_object_new (MOO_TYPE_FILE_SYSTEM, (const char*) NULL));
         g_object_weak_ref (G_OBJECT (fs_instance),
                            (GWeakNotify) g_nullify_pointer, &fs_instance);
         return fs_instance;
@@ -333,14 +338,14 @@ _moo_file_system_create_folder (MooFileSystem  *fs,
 
 
 gboolean
-_moo_file_system_delete_file (MooFileSystem  *fs,
-                              const char     *path,
-                              gboolean        recursive,
-                              GError        **error)
+_moo_file_system_delete_file (MooFileSystem       *fs,
+                              const char          *path,
+                              MooDeleteFileFlags   flags,
+                              GError             **error)
 {
     g_return_val_if_fail (MOO_IS_FILE_SYSTEM (fs), FALSE);
     g_return_val_if_fail (path != NULL, FALSE);
-    return MOO_FILE_SYSTEM_GET_CLASS(fs)->delete_file (fs, path, recursive, error);
+    return MOO_FILE_SYSTEM_GET_CLASS(fs)->delete_file (fs, path, flags, error);
 }
 
 
@@ -418,8 +423,7 @@ _moo_file_system_get_file_watch (MooFileSystem *fs)
         fs->priv->fam = moo_file_watch_new (&error);
         if (!fs->priv->fam)
         {
-            g_warning ("%s: moo_fam_open failed", G_STRLOC);
-            g_warning ("%s: %s", G_STRLOC, error->message);
+            g_warning ("moo_fam_open failed: %s", moo_error_message (error));
             g_error_free (error);
         }
     }
@@ -722,17 +726,26 @@ delete_file_win32 (const char     *path,
 
 static gboolean
 delete_file (G_GNUC_UNUSED MooFileSystem *fs,
-             const char     *path,
-             gboolean        recursive,
-             GError        **error)
+             const char          *path,
+             MooDeleteFileFlags   flags,
+             GError             **error)
 {
     gboolean isdir;
 
     g_return_val_if_fail (path != NULL, FALSE);
     g_return_val_if_fail (_moo_path_is_absolute (path), FALSE);
+    g_return_val_if_fail (!error || !*error, FALSE);
+
+    if (flags & MOO_DELETE_TO_TRASH)
+    {
+        GFile *file = g_file_new_for_path (path);
+        gboolean retval = g_file_trash (file, NULL, error);
+        g_object_unref (file);
+        return retval;
+    }
 
 #if defined(__WIN32__) && 0
-    return delete_file_win32 (path, recursive, error);
+    return delete_file_win32 (path, (flags & MOO_DELETE_RECURSIVE) != 0, error);
 #endif
 
     if (g_file_test (path, G_FILE_TEST_IS_SYMLINK))
@@ -741,7 +754,7 @@ delete_file (G_GNUC_UNUSED MooFileSystem *fs,
         isdir = g_file_test (path, G_FILE_TEST_IS_DIR);
 
     if (isdir)
-        return _moo_remove_dir (path, recursive, error);
+        return _moo_remove_dir (path, (flags & MOO_DELETE_RECURSIVE) != 0, error);
 
     if (_moo_remove (path))
     {
@@ -984,9 +997,7 @@ static MooFolder *
 get_root_folder_win32 (MooFileSystem  *fs,
                        MooFileFlags    wanted)
 {
-#ifdef __GNUC__
-#warning "Implement get_root_folder_win32()"
-#endif
+    MOO_IMPLEMENT_ME
     return _moo_file_system_get_folder (fs, "c:\\", wanted, NULL);
 }
 
@@ -998,9 +1009,7 @@ move_file_win32 (G_GNUC_UNUSED MooFileSystem  *fs,
                  G_GNUC_UNUSED const char *new_path,
                  GError        **error)
 {
-#ifdef __GNUC__
-#warning "Implement move_file_win32()"
-#endif
+    MOO_IMPLEMENT_ME
     g_set_error (error, MOO_FILE_ERROR,
                  MOO_FILE_ERROR_NOT_IMPLEMENTED,
                  "Renaming files is not implemented on win32");
@@ -1130,9 +1139,7 @@ parse_path_win32 (MooFileSystem  *fs,
                   char          **display_basename_p,
                   GError        **error)
 {
-#ifdef __GNUC__
-#warning "Implement parse_path_win32()"
-#endif
+    MOO_IMPLEMENT_ME
     const char *separator;
     char *norm_dirname = NULL, *dirname = NULL, *basename = NULL;
     gsize len;
