@@ -1,13 +1,16 @@
 /*
  *   mootreeview.c
  *
- *   Copyright (C) 2004-2007 by Yevgen Muntyan <muntyan@math.tamu.edu>
+ *   Copyright (C) 2004-2010 by Yevgen Muntyan <emuntyan@users.sourceforge.net>
  *
- *   This library is free software; you can redistribute it and/or
- *   modify it under the terms of the GNU Lesser General Public
- *   License version 2.1 as published by the Free Software Foundation.
+ *   This file is part of medit.  medit is free software; you can
+ *   redistribute it and/or modify it under the terms of the
+ *   GNU Lesser General Public License as published by the
+ *   Free Software Foundation; either version 2.1 of the License,
+ *   or (at your option) any later version.
  *
- *   See COPYING file that comes with this distribution.
+ *   You should have received a copy of the GNU Lesser General Public
+ *   License along with medit.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #define MOO_FILE_VIEW_COMPILATION
@@ -86,7 +89,7 @@ _moo_tree_view_class_init (MooTreeViewClass *klass)
                                              "model",
                                              "model",
                                              GTK_TYPE_TREE_MODEL,
-                                             G_PARAM_READWRITE));
+                                             (GParamFlags) G_PARAM_READWRITE));
 
     signals[BUTTON_PRESS_EVENT] =
             g_signal_new ("button-press-event",
@@ -159,7 +162,7 @@ MooTreeView *
 _moo_tree_view_new (GtkTreeModel *model)
 {
     g_return_val_if_fail (!model || GTK_IS_TREE_MODEL (model), NULL);
-    return g_object_new (MOO_TYPE_TREE_VIEW, "model", model, NULL);
+    return MOO_TREE_VIEW (g_object_new (MOO_TYPE_TREE_VIEW, "model", model, (const char*) NULL));
 }
 
 
@@ -704,12 +707,7 @@ _moo_tree_view_widget_to_abs_coords (gpointer        view,
                                      int            *absy)
 {
     if (GTK_IS_TREE_VIEW (view))
-        /* Nice! */
-#if !GTK_CHECK_VERSION(2,12,0)
-        gtk_tree_view_widget_to_tree_coords (view, wx, wy, absx, absy);
-#else
         gtk_tree_view_convert_widget_to_bin_window_coords (view, wx, wy, absx, absy);
-#endif
     else if (MOO_IS_ICON_VIEW (view))
         _moo_icon_view_widget_to_abs_coords (view, wx, wy, absx, absy);
     else
