@@ -791,4 +791,8 @@ union luai_Cast { double l_d; long l_l; };
 #define L_GNUC_NO_INSTRUMENT
 #endif  /* !__GNUC__ */
 
+#define SLNUNICODE_AS_STRING 1
+#define STRING_WITH_METAT
+#define SLN_UNICODENAME "unicode"
+
 #endif
