@@ -79,7 +79,6 @@ static void moo_combo_cell_layout_reorder               (GtkCellLayout      *cel
 
 static void     moo_combo_class_init        (MooComboClass  *klass);
 static void     moo_combo_init              (MooCombo       *combo);
-static void     moo_combo_finalize          (GObject        *object);
 static void     moo_combo_destroy           (GtkObject      *object);
 static void     moo_combo_set_property      (GObject        *object,
                                              guint           prop_id,
@@ -104,7 +103,9 @@ static gboolean moo_combo_popup_key_press   (MooCombo       *combo,
 static void     create_arrow_button         (MooCombo       *combo);
 static void     button_clicked              (MooCombo       *combo);
 
-static void     create_popup                (MooCombo       *combo);
+static void     create_popup_tree_view      (MooCombo       *combo);
+static void     create_popup_window         (MooCombo       *combo);
+static void     destroy_popup_window        (MooCombo       *combo);
 static gboolean resize_popup                (MooCombo       *combo);
 static gboolean entry_focus_out             (MooCombo       *combo);
 static gboolean popup_button_press          (MooCombo       *combo,
@@ -183,9 +184,10 @@ moo_combo_class_init (MooComboClass *klass)
     GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);
     GtkBindingSet *binding_set;
 
+    g_type_class_add_private (klass, sizeof (MooComboPrivate));
+
     moo_combo_parent_class = g_type_class_peek_parent (klass);
 
-    gobject_class->finalize = moo_combo_finalize;
     gobject_class->set_property = moo_combo_set_property;
     gobject_class->get_property = moo_combo_get_property;
 
@@ -274,7 +276,7 @@ moo_combo_cell_layout_init (GtkCellLayoutIface *iface)
 static void
 moo_combo_init (MooCombo *combo)
 {
-    combo->priv = g_new0 (MooComboPrivate, 1);
+    combo->priv = G_TYPE_INSTANCE_GET_PRIVATE (combo, MOO_TYPE_COMBO, MooComboPrivate);
 
     combo->priv->use_button = TRUE;
 
@@ -297,7 +299,7 @@ moo_combo_init (MooCombo *combo)
                               G_CALLBACK (entry_changed), combo);
 
     create_arrow_button (combo);
-    create_popup (combo);
+    create_popup_tree_view (combo);
 }
 
 
@@ -334,15 +336,26 @@ moo_combo_destroy (GtkObject *object)
 {
     MooCombo *combo = MOO_COMBO (object);
 
+    moo_combo_set_model (combo, NULL);
+
     combo->priv->button = NULL;
     combo->entry = NULL;
 
+    gtk_tree_row_reference_free (combo->priv->active_row);
+    combo->priv->active_row = NULL;
+
     if (combo->priv->model)
     {
         g_object_unref (combo->priv->model);
         combo->priv->model = NULL;
     }
 
+    if (combo->priv->treeview)
+    {
+        g_object_unref (combo->priv->treeview);
+        combo->priv->treeview = NULL;
+    }
+
     if (combo->priv->popup)
     {
         gtk_widget_destroy (combo->priv->popup);
@@ -411,16 +424,6 @@ moo_combo_get_property (GObject        *object,
 }
 
 
-static void
-moo_combo_finalize (GObject *object)
-{
-    MooCombo *combo = MOO_COMBO (object);
-    gtk_tree_row_reference_free (combo->priv->active_row);
-    g_free (combo->priv);
-    G_OBJECT_CLASS (moo_combo_parent_class)->finalize (object);
-}
-
-
 GtkWidget *
 moo_combo_new (void)
 {
@@ -429,10 +432,12 @@ moo_combo_new (void)
 
 
 static void
-create_popup (MooCombo *combo)
+create_popup_window (MooCombo *combo)
 {
     GtkWidget *scrolled_window, *frame;
-    GtkTreeSelection *selection;
+
+    if (combo->priv->popup)
+        return;
 
     combo->priv->popup = gtk_window_new (GTK_WINDOW_POPUP);
     gtk_widget_set_size_request (combo->priv->popup, -1, -1);
@@ -440,17 +445,6 @@ create_popup (MooCombo *combo)
     gtk_window_set_resizable (GTK_WINDOW (combo->priv->popup), FALSE);
     gtk_widget_add_events (combo->priv->popup, GDK_KEY_PRESS_MASK | GDK_BUTTON_PRESS_MASK);
 
-    combo->priv->column = gtk_tree_view_column_new ();
-
-    combo->priv->treeview = GTK_TREE_VIEW (gtk_tree_view_new ());
-    gtk_widget_set_size_request (GTK_WIDGET (combo->priv->treeview), -1, -1);
-    gtk_tree_view_append_column (combo->priv->treeview, combo->priv->column);
-    gtk_tree_view_set_headers_visible (combo->priv->treeview, FALSE);
-    gtk_tree_view_set_hover_selection (combo->priv->treeview, TRUE);
-
-    selection = gtk_tree_view_get_selection (combo->priv->treeview);
-    gtk_tree_selection_set_mode (selection, GTK_SELECTION_SINGLE);
-
     scrolled_window = gtk_scrolled_window_new (NULL, NULL);
     gtk_widget_set_size_request (scrolled_window, -1, -1);
     gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_window),
@@ -470,6 +464,37 @@ create_popup (MooCombo *combo)
     gtk_container_add (GTK_CONTAINER (combo->priv->popup), frame);
 }
 
+static void
+destroy_popup_window (MooCombo *combo)
+{
+    if (combo->priv->popup)
+    {
+        GtkWidget *tree_view = GTK_WIDGET (combo->priv->treeview);
+        gtk_container_remove (GTK_CONTAINER (gtk_widget_get_parent (tree_view)), tree_view);
+        gtk_widget_destroy (combo->priv->popup);
+        combo->priv->popup = NULL;
+    }
+}
+
+static void
+create_popup_tree_view (MooCombo *combo)
+{
+    GtkTreeSelection *selection;
+
+    combo->priv->column = gtk_tree_view_column_new ();
+
+    combo->priv->treeview = GTK_TREE_VIEW (gtk_tree_view_new ());
+    g_object_ref_sink (combo->priv->treeview);
+
+    gtk_widget_set_size_request (GTK_WIDGET (combo->priv->treeview), -1, -1);
+    gtk_tree_view_append_column (combo->priv->treeview, combo->priv->column);
+    gtk_tree_view_set_headers_visible (combo->priv->treeview, FALSE);
+    gtk_tree_view_set_hover_selection (combo->priv->treeview, TRUE);
+
+    selection = gtk_tree_view_get_selection (combo->priv->treeview);
+    gtk_tree_selection_set_mode (selection, GTK_SELECTION_SINGLE);
+}
+
 
 void
 moo_combo_popup (MooCombo   *combo)
@@ -552,6 +577,8 @@ moo_combo_popup_real (MooCombo *combo)
     window = gtk_widget_get_toplevel (GTK_WIDGET (combo->entry));
     g_return_if_fail (GTK_IS_WINDOW (window));
 
+    create_popup_window (combo);
+
     gtk_widget_realize (combo->priv->popup);
 
     if (GTK_WINDOW (window)->group)
@@ -1187,6 +1214,7 @@ static void
 moo_combo_unmap (GtkWidget      *widget)
 {
     moo_combo_popdown (MOO_COMBO (widget));
+    destroy_popup_window (MOO_COMBO (widget));
     GTK_WIDGET_CLASS(moo_combo_parent_class)->unmap (widget);
 }
 
@@ -1195,7 +1223,7 @@ static void
 moo_combo_unrealize (GtkWidget      *widget)
 {
     moo_combo_popdown (MOO_COMBO (widget));
-    gtk_widget_unrealize (MOO_COMBO (widget)->priv->popup);
+    destroy_popup_window (MOO_COMBO (widget));
     GTK_WIDGET_CLASS(moo_combo_parent_class)->unrealize (widget);
 }
 
