@@ -1,63 +1,33 @@
 /*
  *   moomarkup.c
  *
- *   Copyright (C) 2004-2007 by Yevgen Muntyan <muntyan@math.tamu.edu>
+ *   Copyright (C) 2004-2010 by Yevgen Muntyan <emuntyan@users.sourceforge.net>
  *
- *   This library is free software; you can redistribute it and/or
- *   modify it under the terms of the GNU Lesser General Public
- *   License version 2.1 as published by the Free Software Foundation.
+ *   This file is part of medit.  medit is free software; you can
+ *   redistribute it and/or modify it under the terms of the
+ *   GNU Lesser General Public License as published by the
+ *   Free Software Foundation; either version 2.1 of the License,
+ *   or (at your option) any later version.
  *
- *   See COPYING file that comes with this distribution.
+ *   You should have received a copy of the GNU Lesser General Public
+ *   License along with medit.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "mooutils/moomarkup.h"
 #include "mooutils/mooutils-fs.h"
 #include "mooutils/mooutils-gobject.h"
 #include "mooutils/mooutils-misc.h"
+#include "mooutils/mootype-macros.h"
 #include <string.h>
 #include <glib.h>
 
 
-GType
-moo_markup_doc_get_type (void)
-{
-    static GType type = 0;
-
-    if (G_UNLIKELY (!type))
-        type = g_boxed_type_register_static ("MooMarkupDoc",
-                                             (GBoxedCopyFunc) moo_markup_doc_ref,
-                                             (GBoxedFreeFunc) moo_markup_doc_unref);
-
-    return type;
-}
-
-
-typedef void (*markup_start_element_func)   (GMarkupParseContext    *context,
-                                             const gchar            *element_name,
-                                             const gchar           **attribute_names,
-                                             const gchar           **attribute_values,
-                                             gpointer                user_data,
-                                             GError                **error);
-typedef void (*markup_end_element_func)     (GMarkupParseContext    *context,
-                                             const gchar            *element_name,
-                                             gpointer                user_data,
-                                             GError                **error);
-typedef void (*markup_text_func)            (GMarkupParseContext    *context,
-                                             const gchar            *text,
-                                             gsize                   text_len,
-                                             gpointer                user_data,
-                                             GError                **error);
-typedef void (*markup_passthrough_func)     (GMarkupParseContext    *context,
-                                             const gchar            *passthrough_text,
-                                             gsize                   text_len,
-                                             gpointer                user_data,
-                                             GError                **error);
-
 typedef struct {
     MooMarkupDoc   *doc;
     MooMarkupNode  *current;
 } ParserState;
 
+MOO_DEFINE_BOXED_TYPE_R (MooMarkupDoc, moo_markup_doc)
 
 static MooMarkupDoc     *moo_markup_doc_new_priv    (const char         *name);
 static void              moo_markup_doc_set_name    (MooMarkupDoc       *doc,
@@ -128,7 +98,7 @@ set_modified (MooMarkupNode *node,
 {
     if (node->doc)
     {
-#ifdef MOO_DEBUG_ENABLED
+#ifdef MOO_DEBUG
         if (modified && !node->doc->modified && node->doc->track_modified)
         {
             g_message ("markup doc %s modified",
@@ -173,10 +143,10 @@ markup_parse_memory (const char     *buffer,
                      G_GNUC_UNUSED const char *filename,
                      GError        **error)
 {
-    GMarkupParser parser = { (markup_start_element_func) start_element,
-                             (markup_end_element_func) end_element,
-                             (markup_text_func) text,
-                             (markup_passthrough_func) passthrough,
+    GMarkupParser parser = { (MooMarkupStartElementFunc) start_element,
+                             (MooMarkupEndElementFunc) end_element,
+                             (MooMarkupTextFunc) text,
+                             (MooMarkupPassthroughFunc) passthrough,
                              NULL};
 
     ParserState state;
@@ -189,7 +159,7 @@ markup_parse_memory (const char     *buffer,
     _moo_markup_set_modified (state.doc, TRUE);
 
     state.current = MOO_MARKUP_NODE (state.doc);
-    context = g_markup_parse_context_new (&parser, (GMarkupParseFlags) 0, &state, NULL);
+    context = g_markup_parse_context_new (&parser, G_MARKUP_TREAT_CDATA_AS_TEXT, &state, NULL);
 
     if (!g_markup_parse_context_parse (context, buffer, len, error) ||
          !g_markup_parse_context_end_parse (context, error))
@@ -223,6 +193,8 @@ moo_markup_parse_file (const char     *filename,
 
     g_return_val_if_fail (filename != NULL, NULL);
 
+    /* XXX use moo_parse_markup_file here */
+
     if (!g_file_get_contents (filename, &content, &len, error))
         return NULL;
 
@@ -304,10 +276,10 @@ passthrough (G_GNUC_UNUSED GMarkupParseContext    *ctx,
 static MooMarkupDoc *
 moo_markup_doc_new_priv (const char *name)
 {
-    MooMarkupDoc *doc = moo_new0 (MooMarkupDoc);
+    MooMarkupDoc *doc = g_slice_new0 (MooMarkupDoc);
 
     doc->type = MOO_MARKUP_DOC_NODE;
-    doc->name = g_strdup (name ? name : "");
+    doc->name = g_strdup (MOO_NZS (name));
     doc->doc = doc;
     doc->ref_count = 1;
 
@@ -320,7 +292,7 @@ moo_markup_doc_set_name (MooMarkupDoc *doc,
                          const char   *name)
 {
     g_return_if_fail (MOO_MARKUP_IS_DOC (doc));
-    MOO_ASSIGN_STRING (doc->name, name ? name : "");
+    MOO_ASSIGN_STRING (doc->name, MOO_NZS (name));
 }
 
 
@@ -340,7 +312,7 @@ moo_markup_element_new (MooMarkupDoc   *doc,
 {
     MooMarkupElement *elm;
 
-    elm = moo_new0 (MooMarkupElement);
+    elm = g_slice_new0 (MooMarkupElement);
     add_node (doc, parent, MOO_MARKUP_NODE (elm));
 
     elm->type = MOO_MARKUP_ELEMENT_NODE;
@@ -368,7 +340,7 @@ moo_markup_text_node_new_take_string (MooMarkupNodeType   type,
 
     g_assert (type == MOO_MARKUP_TEXT_NODE || type == MOO_MARKUP_COMMENT_NODE);
 
-    node = moo_new0 (MooMarkupText);
+    node = g_slice_new0 (MooMarkupText);
     add_node (doc, parent, MOO_MARKUP_NODE (node));
 
     node->type = type;
@@ -508,19 +480,19 @@ moo_markup_node_free (MooMarkupNode *node)
     {
         case MOO_MARKUP_DOC_NODE:
             moo_markup_doc_unref_private (MOO_MARKUP_DOC (node));
-            moo_free (MooMarkupDoc, (MooMarkupDoc*) node);
+            g_slice_free (MooMarkupDoc, (MooMarkupDoc*) node);
             break;
         case MOO_MARKUP_ELEMENT_NODE:
             moo_markup_element_free (MOO_MARKUP_ELEMENT (node));
-            moo_free (MooMarkupElement, (MooMarkupElement*) node);
+            g_slice_free (MooMarkupElement, (MooMarkupElement*) node);
             break;
         case MOO_MARKUP_TEXT_NODE:
             moo_markup_text_node_free (node);
-            moo_free (MooMarkupText, (MooMarkupText*) node);
+            g_slice_free (MooMarkupText, (MooMarkupText*) node);
             break;
         case MOO_MARKUP_COMMENT_NODE:
             moo_markup_text_node_free (node);
-            moo_free (MooMarkupComment, (MooMarkupComment*) node);
+            g_slice_free (MooMarkupComment, (MooMarkupComment*) node);
             break;
         default:
             g_assert_not_reached ();
@@ -648,7 +620,7 @@ moo_markup_text_node_print (MooMarkupNode  *node,
 
     if (!g_utf8_validate (text->text, -1, (const char**) &p))
     {
-        g_critical ("%s: invalid UTF8", G_STRLOC);
+        g_critical ("invalid UTF8");
         *p = 0;
     }
 
@@ -667,11 +639,11 @@ moo_markup_comment_node_print (MooMarkupNode *node,
 
     if (!g_utf8_validate (text->text, -1, (const char**) &p))
     {
-        g_critical ("%s: invalid UTF8", G_STRLOC);
+        g_critical ("invalid UTF8");
         *p = 0;
     }
 
-    g_string_append_printf (str, "<!--%s-->", text->text);
+    g_string_append (str, text->text);
 }
 
 
@@ -849,9 +821,9 @@ moo_markup_set_prop (MooMarkupNode      *elm,
 
 
 int
-moo_markup_get_int_prop (MooMarkupNode      *node,
-                         const char         *prop_name,
-                         int                 default_val)
+moo_markup_int_prop (MooMarkupNode *node,
+                     const char    *prop_name,
+                     int            default_val)
 {
     g_return_val_if_fail (MOO_MARKUP_IS_ELEMENT (node), default_val);
     g_return_val_if_fail (prop_name != NULL, default_val);
@@ -859,21 +831,21 @@ moo_markup_get_int_prop (MooMarkupNode      *node,
                                        default_val);
 }
 
-
-void
-moo_markup_set_int_prop (MooMarkupNode      *node,
-                         const char         *prop_name,
-                         int                 val)
+guint
+moo_markup_uint_prop (MooMarkupNode *node,
+                      const char    *prop_name,
+                      guint          default_val)
 {
-    moo_markup_set_prop (node, prop_name,
-                         _moo_convert_int_to_string (val));
+    g_return_val_if_fail (MOO_MARKUP_IS_ELEMENT (node), default_val);
+    g_return_val_if_fail (prop_name != NULL, default_val);
+    return _moo_convert_string_to_uint (moo_markup_get_prop (node, prop_name),
+                                        default_val);
 }
 
-
 gboolean
-moo_markup_get_bool_prop (MooMarkupNode      *node,
-                          const char         *prop_name,
-                          gboolean            default_val)
+moo_markup_bool_prop (MooMarkupNode *node,
+                      const char    *prop_name,
+                      gboolean       default_val)
 {
     g_return_val_if_fail (MOO_MARKUP_IS_ELEMENT (node), default_val);
     g_return_val_if_fail (prop_name != NULL, default_val);
@@ -883,9 +855,9 @@ moo_markup_get_bool_prop (MooMarkupNode      *node,
 
 
 void
-moo_markup_set_bool_prop (MooMarkupNode      *node,
-                          const char         *prop_name,
-                          gboolean            val)
+moo_markup_set_bool_prop (MooMarkupNode *node,
+                          const char    *prop_name,
+                          gboolean       val)
 {
     moo_markup_set_prop (node, prop_name,
                          _moo_convert_bool_to_string (val));
@@ -1052,14 +1024,15 @@ moo_markup_create_text_element (MooMarkupNode      *parent,
     g_return_val_if_fail (MOO_MARKUP_IS_ELEMENT (parent) ||
                           MOO_MARKUP_IS_DOC (parent), NULL);
     g_return_val_if_fail (path != NULL, NULL);
-    g_return_val_if_fail (content != NULL, NULL);
-    g_return_val_if_fail (g_utf8_validate (content, -1, NULL), NULL);
+    g_return_val_if_fail (!content || g_utf8_validate (content, -1, NULL), NULL);
 
     elm = (MooMarkupElement*) moo_markup_create_element (parent, path);
 
     if (!elm)
         return NULL;
 
+    content = MOO_NZS (content);
+
     moo_markup_text_node_new (MOO_MARKUP_TEXT_NODE,
                               elm->doc, MOO_MARKUP_NODE (elm),
                               content, strlen (content));
@@ -1108,8 +1081,8 @@ moo_markup_create_file_element (MooMarkupNode      *parent,
 
     if (!filename_utf8)
     {
-        g_warning ("%s: could not convert '%s' to utf8",
-                   G_STRLOC, filename);
+        g_warning ("could not convert '%s' to utf8",
+                   filename);
         return NULL;
     }
 
@@ -1239,7 +1212,7 @@ moo_markup_write_pretty (MooMarkupDoc  *doc,
     g_return_val_if_fail (MOO_MARKUP_IS_DOC (doc), FALSE);
     g_return_val_if_fail (writer != NULL, FALSE);
 
-    result = moo_file_writer_write (writer, "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n", -1);
+    result = moo_file_writer_write (writer, "<?xml version=\"1.0\" encoding=\"UTF-8\"?><!-- -*- xml -*- -->\n", -1);
 
     for (child = doc->children; result && child != NULL; child = child->next)
         if (MOO_MARKUP_IS_ELEMENT (child))
@@ -1318,3 +1291,52 @@ MOO_MARKUP_COMMENT_CHECK_CAST (gpointer n)
     g_return_val_if_fail (node->type == MOO_MARKUP_COMMENT_NODE, NULL);
     return (MooMarkupComment*) n;
 }
+
+
+gboolean
+moo_parse_markup_file (const char         *filename,
+                       const GMarkupParser *parser,
+                       gpointer            data,
+                       GError            **error)
+{
+    GMarkupParseContext *ctx;
+    MooFileReader *reader;
+    char buf[1024];
+    gsize size;
+    gboolean seen_error = FALSE;
+
+    g_return_val_if_fail (filename != NULL, FALSE);
+    g_return_val_if_fail (parser != NULL, FALSE);
+    g_return_val_if_fail (!error || !*error, FALSE);
+
+    if (!(reader = moo_text_reader_new (filename, error)))
+        return FALSE;
+
+    ctx = g_markup_parse_context_new (parser, G_MARKUP_TREAT_CDATA_AS_TEXT, data, NULL);
+
+    while (TRUE)
+    {
+        if (!moo_file_reader_read (reader, buf, sizeof buf, &size, error))
+        {
+            seen_error = TRUE;
+            break;
+        }
+
+        if (!size)
+            break;
+
+        if (!g_markup_parse_context_parse (ctx, buf, size, error))
+        {
+            seen_error = TRUE;
+            break;
+        }
+    }
+
+    if (!seen_error && !g_markup_parse_context_end_parse (ctx, error))
+        seen_error = TRUE;
+
+    g_markup_parse_context_free (ctx);
+    moo_file_reader_close (reader);
+
+    return !seen_error;
+}
