@@ -1,15 +1,22 @@
 /*
  *   moopane.c
  *
- *   Copyright (C) 2004-2008 by Yevgen Muntyan <muntyan@tamu.edu>
+ *   Copyright (C) 2004-2010 by Yevgen Muntyan <emuntyan@users.sourceforge.net>
  *
- *   This library is free software; you can redistribute it and/or
- *   modify it under the terms of the GNU Lesser General Public
- *   License version 2.1 as published by the Free Software Foundation.
+ *   This file is part of medit.  medit is free software; you can
+ *   redistribute it and/or modify it under the terms of the
+ *   GNU Lesser General Public License as published by the
+ *   Free Software Foundation; either version 2.1 of the License,
+ *   or (at your option) any later version.
  *
- *   See COPYING file that comes with this distribution.
+ *   You should have received a copy of the GNU Lesser General Public
+ *   License along with medit.  If not, see <http://www.gnu.org/licenses/>.
  */
 
+/**
+ * class:MooPane: (parent GtkObject) (moo.lua 0) (moo.private 1)
+ **/
+
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
@@ -28,6 +35,7 @@
 #ifdef MOO_COMPILATION
 
 #include "mooutils-misc.h"
+#include "moocompat.h"
 #include "moohelp.h"
 #include "mooutils-gobject.h"
 #include "mooi18n.h"
@@ -80,12 +88,6 @@ _moo_window_set_icon_from_stock (GtkWindow  *window,
     }
 }
 
-#if GLIB_CHECK_VERSION(2,10,0)
-#define MOO_OBJECT_REF_SINK(obj) g_object_ref_sink (obj)
-#else
-#define MOO_OBJECT_REF_SINK(obj) gtk_object_sink (g_object_ref (obj))
-#endif
-
 #endif
 
 
@@ -100,6 +102,7 @@ struct _MooPane {
     MooPaneLabel *label;
     GtkWidget    *frame;
     GtkWidget    *handle;
+    GtkWidget    *small_handle;
     GtkWidget    *button;
     GtkWidget    *label_widget;
     GtkWidget    *icon_widget;
@@ -107,6 +110,11 @@ struct _MooPane {
     GtkWidget    *detach_button;
     GtkWidget    *close_button;
 
+    GtkWidget    *frame_label_embedded;
+    GtkWidget    *frame_label_window;
+    char         *frame_label_text;
+    gboolean      frame_label_markup;
+
     /* XXX weak pointer */
     gpointer      focus_child;
 
@@ -194,6 +202,9 @@ update_label_widgets (MooPane *pane)
     set_pane_window_icon_and_title (pane);
 }
 
+/**
+ * moo_pane_set_label:
+ **/
 void
 moo_pane_set_label (MooPane      *pane,
                     MooPaneLabel *label)
@@ -213,6 +224,9 @@ moo_pane_set_label (MooPane      *pane,
 }
 
 
+/**
+ * moo_pane_get_params:
+ **/
 MooPaneParams *
 moo_pane_get_params (MooPane *pane)
 {
@@ -220,6 +234,9 @@ moo_pane_get_params (MooPane *pane)
     return moo_pane_params_copy (pane->params);
 }
 
+/**
+ * moo_pane_get_label:
+ **/
 MooPaneLabel *
 moo_pane_get_label (MooPane *pane)
 {
@@ -228,6 +245,9 @@ moo_pane_get_label (MooPane *pane)
 }
 
 
+/**
+ * moo_pane_set_params:
+ **/
 void
 moo_pane_set_params (MooPane       *pane,
                      MooPaneParams *params)
@@ -256,6 +276,9 @@ moo_pane_set_params (MooPane       *pane,
 }
 
 
+/**
+ * moo_pane_set_detachable:
+ **/
 void
 moo_pane_set_detachable (MooPane  *pane,
                          gboolean  detachable)
@@ -277,6 +300,9 @@ moo_pane_set_detachable (MooPane  *pane,
 }
 
 
+/**
+ * moo_pane_set_removable:
+ **/
 void
 moo_pane_set_removable (MooPane  *pane,
                         gboolean  removable)
@@ -295,6 +321,9 @@ moo_pane_set_removable (MooPane  *pane,
 }
 
 
+/**
+ * moo_pane_get_detachable:
+ **/
 gboolean
 moo_pane_get_detachable (MooPane *pane)
 {
@@ -302,6 +331,9 @@ moo_pane_get_detachable (MooPane *pane)
     return pane->detachable;
 }
 
+/**
+ * moo_pane_get_removable:
+ **/
 gboolean
 moo_pane_get_removable (MooPane *pane)
 {
@@ -382,6 +414,7 @@ moo_pane_init (MooPane *pane)
     pane->child_holder = NULL;
     pane->frame = NULL;
     pane->handle = NULL;
+    pane->small_handle = NULL;
     pane->button = NULL;
     pane->label_widget = NULL;
     pane->icon_widget = NULL;
@@ -416,7 +449,6 @@ moo_pane_dispose (GObject *object)
     {
         GtkWidget *tmp = pane->child;
         pane->child = NULL;
-        gtk_widget_destroy (tmp);
         g_object_unref (tmp);
     }
 
@@ -457,19 +489,19 @@ moo_pane_class_init (MooPaneClass *klass)
 
     g_object_class_install_property (gobject_class, PROP_LABEL,
         g_param_spec_boxed ("label", "label", "label",
-                            MOO_TYPE_PANE_LABEL, G_PARAM_READWRITE));
+                            MOO_TYPE_PANE_LABEL, (GParamFlags) G_PARAM_READWRITE));
 
     g_object_class_install_property (gobject_class, PROP_PARAMS,
         g_param_spec_boxed ("params", "params", "params",
-                            MOO_TYPE_PANE_PARAMS, G_PARAM_READWRITE));
+                            MOO_TYPE_PANE_PARAMS, (GParamFlags) G_PARAM_READWRITE));
 
     g_object_class_install_property (gobject_class, PROP_DETACHABLE,
         g_param_spec_boolean ("detachable", "detachable", "detachable",
-                              TRUE, G_PARAM_READWRITE));
+                              TRUE, (GParamFlags) G_PARAM_READWRITE));
 
     g_object_class_install_property (gobject_class, PROP_REMOVABLE,
         g_param_spec_boolean ("removable", "removable", "removable",
-                              TRUE, G_PARAM_READWRITE));
+                              TRUE, (GParamFlags) G_PARAM_READWRITE));
 
     signals[REMOVE] =
             g_signal_new ("remove",
@@ -535,6 +567,63 @@ update_sticky_button (MooPane *pane)
 }
 
 
+/**
+ * moo_pane_set_frame_markup:
+ *
+ * @pane:
+ * @markup: (type const-utf8) (allow-none):
+ **/
+void
+moo_pane_set_frame_markup (MooPane    *pane,
+                           const char *text)
+{
+    char *tmp;
+
+    g_return_if_fail (MOO_IS_PANE (pane));
+
+    if (!text)
+    {
+        moo_pane_set_frame_text (pane, NULL);
+        return;
+    }
+
+    tmp = pane->frame_label_text;
+    pane->frame_label_text = g_strdup (text);
+    pane->frame_label_markup = TRUE;
+    g_free (tmp);
+
+    if (pane->frame_label_embedded)
+        gtk_label_set_markup (GTK_LABEL (pane->frame_label_embedded), text);
+    if (pane->frame_label_window)
+        gtk_label_set_markup (GTK_LABEL (pane->frame_label_window), text);
+}
+
+/**
+ * moo_pane_set_frame_text:
+ *
+ * @pane:
+ * @text: (type const-utf8) (allow-none):
+ **/
+void
+moo_pane_set_frame_text (MooPane    *pane,
+                         const char *text)
+{
+    char *tmp;
+
+    g_return_if_fail (MOO_IS_PANE (pane));
+
+    tmp = pane->frame_label_text;
+    pane->frame_label_text = g_strdup (text);
+    pane->frame_label_markup = FALSE;
+    g_free (tmp);
+
+    if (pane->frame_label_embedded)
+        gtk_label_set_text (GTK_LABEL (pane->frame_label_embedded), text);
+    if (pane->frame_label_window)
+        gtk_label_set_text (GTK_LABEL (pane->frame_label_window), text);
+}
+
+
 static GtkWidget *
 create_button (MooPane      *pane,
                GtkWidget    *toolbar,
@@ -570,6 +659,7 @@ create_frame_widget (MooPane        *pane,
                      gboolean        embedded)
 {
     GtkWidget *vbox, *toolbar, *separator, *handle, *table, *child_holder;
+    GtkWidget *handle_hbox, *frame_label;
 
     vbox = gtk_vbox_new (FALSE, 0);
     gtk_widget_show (vbox);
@@ -581,6 +671,29 @@ create_frame_widget (MooPane        *pane,
     gtk_box_pack_start (GTK_BOX (toolbar), handle, TRUE, TRUE, 3);
     pane->handle = handle;
 
+    handle_hbox = gtk_hbox_new (FALSE, 0);
+    gtk_widget_show (handle_hbox);
+    gtk_container_add (GTK_CONTAINER (pane->handle), handle_hbox);
+
+    frame_label = gtk_label_new (NULL);
+    gtk_widget_show (frame_label);
+    gtk_box_pack_start (GTK_BOX (handle_hbox), frame_label, TRUE, TRUE, 0);
+    gtk_misc_set_alignment (GTK_MISC (frame_label), .0, .5);
+    gtk_misc_set_padding (GTK_MISC (frame_label), 6, 0);
+    gtk_label_set_ellipsize (GTK_LABEL (frame_label), PANGO_ELLIPSIZE_END);
+    if (pane->frame_label_markup)
+        gtk_label_set_markup (GTK_LABEL (frame_label), pane->frame_label_text);
+    else
+        gtk_label_set_text (GTK_LABEL (frame_label), pane->frame_label_text);
+    if (embedded)
+        pane->frame_label_embedded = frame_label;
+    else
+        pane->frame_label_window = frame_label;
+
+    pane->small_handle = gtk_event_box_new ();
+    gtk_widget_show (pane->small_handle);
+    gtk_box_pack_start (GTK_BOX (handle_hbox), pane->small_handle, TRUE, TRUE, 0);
+
     if (embedded)
     {
         GtkWidget *hide_button;
@@ -851,6 +964,9 @@ setup_button_dnd (MooPane *pane)
                       G_CALLBACK (button_drag_leave), pane);
 }
 
+/**
+ * moo_pane_set_drag_dest:
+ **/
 void
 moo_pane_set_drag_dest (MooPane *pane)
 {
@@ -864,6 +980,9 @@ moo_pane_set_drag_dest (MooPane *pane)
         setup_button_dnd (pane);
 }
 
+/**
+ * moo_pane_unset_drag_dest:
+ **/
 void
 moo_pane_unset_drag_dest (MooPane *pane)
 {
@@ -938,7 +1057,7 @@ _moo_pane_new (GtkWidget    *child,
 
     g_return_val_if_fail (GTK_IS_WIDGET (child), NULL);
 
-    pane = g_object_new (MOO_TYPE_PANE, NULL);
+    pane = g_object_new (MOO_TYPE_PANE, (const char*) NULL);
     pane->child = g_object_ref (child);
     gtk_widget_show (pane->child);
     g_object_set_data (G_OBJECT (pane->child), "moo-pane", pane);
@@ -949,6 +1068,11 @@ _moo_pane_new (GtkWidget    *child,
     return pane;
 }
 
+/**
+ * moo_pane_get_id:
+ *
+ * Returns: (type const-utf8)
+ **/
 const char *
 moo_pane_get_id (MooPane *pane)
 {
@@ -1043,11 +1167,14 @@ _moo_pane_get_button (MooPane *pane)
     return pane->button;
 }
 
-GtkWidget *
-_moo_pane_get_handle (MooPane *pane)
+void
+_moo_pane_get_handle (MooPane    *pane,
+                      GtkWidget **big,
+                      GtkWidget **small_)
 {
-    g_return_val_if_fail (MOO_IS_PANE (pane), NULL);
-    return pane->handle;
+    g_return_if_fail (MOO_IS_PANE (pane));
+    *big = pane->handle;
+    *small_ = pane->small_handle;
 }
 
 GtkWidget *
@@ -1057,6 +1184,9 @@ _moo_pane_get_window (MooPane *pane)
     return pane->window;
 }
 
+/**
+ * moo_pane_get_child:
+ **/
 GtkWidget *
 moo_pane_get_child (MooPane *pane)
 {
@@ -1121,6 +1251,7 @@ _moo_pane_unparent (MooPane *pane)
         pane->child_holder = NULL;
         pane->frame = NULL;
         pane->handle = NULL;
+        pane->small_handle = NULL;
         pane->button = NULL;
         pane->label_widget = NULL;
         pane->icon_widget = NULL;
@@ -1247,6 +1378,8 @@ create_pane_window (MooPane *pane)
 #endif
 
     set_pane_window_icon_and_title (pane);
+    gtk_window_set_type_hint (GTK_WINDOW (pane->window),
+                              GDK_WINDOW_TYPE_HINT_UTILITY);
 
     switch (_moo_paned_get_position (pane->parent))
     {
@@ -1274,14 +1407,15 @@ create_pane_window (MooPane *pane)
     g_object_set_data (G_OBJECT (pane->window), "moo-pane", pane);
     g_object_set_data (G_OBJECT (pane->keep_on_top_button), "moo-pane", pane);
 
+    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (pane->keep_on_top_button),
+                                  pane->params->keep_on_top);
     g_signal_connect (pane->keep_on_top_button, "toggled",
                       G_CALLBACK (keep_on_top_button_toggled), pane);
     g_signal_connect (pane->window, "configure-event",
                       G_CALLBACK (pane_window_configure), pane);
 }
 
-/* XXX gtk_widget_reparent() doesn't work here for some reasons */
-/* shouldn't it work now, as I fixed GTK_NO_WINDOW flag? */
+/* FIXME use gtk_widget_reparent(), it does work now */
 static void
 reparent (GtkWidget *widget,
           GtkWidget *old_container,
@@ -1386,6 +1520,9 @@ _moo_pane_try_remove (MooPane *pane)
 }
 
 
+/**
+ * moo_pane_open:
+ **/
 void
 moo_pane_open (MooPane *pane)
 {
@@ -1394,6 +1531,9 @@ moo_pane_open (MooPane *pane)
     moo_paned_open_pane (pane->parent, pane);
 }
 
+/**
+ * moo_pane_present:
+ **/
 void
 moo_pane_present (MooPane *pane)
 {
@@ -1402,6 +1542,9 @@ moo_pane_present (MooPane *pane)
     moo_paned_present_pane (pane->parent, pane);
 }
 
+/**
+ * moo_pane_attach:
+ **/
 void
 moo_pane_attach (MooPane *pane)
 {
@@ -1410,6 +1553,9 @@ moo_pane_attach (MooPane *pane)
     moo_paned_attach_pane (pane->parent, pane);
 }
 
+/**
+ * moo_pane_detach:
+ **/
 void
 moo_pane_detach (MooPane *pane)
 {
@@ -1419,6 +1565,9 @@ moo_pane_detach (MooPane *pane)
 }
 
 
+/**
+ * moo_pane_get_index:
+ **/
 int
 moo_pane_get_index (MooPane *pane)
 {
@@ -1456,7 +1605,7 @@ G_DEFINE_TYPE (MooIconWidget, _moo_icon_widget, GTK_TYPE_WIDGET)
 static void
 _moo_icon_widget_init (MooIconWidget *icon)
 {
-    GTK_WIDGET_SET_FLAGS (icon, GTK_NO_WINDOW);
+    GTK_WIDGET_SET_NO_WINDOW (icon);
     icon->pixbufs = NULL;
     icon->data = NULL;
     icon->type = ICON_PIXBUFS;
@@ -1674,7 +1823,7 @@ _moo_create_small_icon (MooSmallIcon icon)
 
     g_return_val_if_fail (data != NULL, NULL);
 
-    icon_widget = g_object_new (_moo_icon_widget_get_type (), NULL);
+    icon_widget = g_object_new (_moo_icon_widget_get_type (), (const char*) NULL);
     icon_widget->data = data;
     gtk_widget_set_size_request (GTK_WIDGET (icon_widget), 7, 7 /* magic */);
 
@@ -1705,7 +1854,7 @@ _moo_create_arrow_icon (GtkArrowType arrow_type)
             g_return_val_if_reached (NULL);
     }
 
-    icon_widget = g_object_new (_moo_icon_widget_get_type (), NULL);
+    icon_widget = g_object_new (_moo_icon_widget_get_type (), (const char*) NULL);
     icon_widget->type = icon_type;
 
     return GTK_WIDGET (icon_widget);
