@@ -1,19 +1,21 @@
 /*
  *   moospawn.c
  *
- *   Copyright (C) 2004-2007 by Yevgen Muntyan <muntyan@math.tamu.edu>
+ *   Copyright (C) 2004-2010 by Yevgen Muntyan <emuntyan@users.sourceforge.net>
  *
- *   This library is free software; you can redistribute it and/or
- *   modify it under the terms of the GNU Lesser General Public
- *   License version 2.1 as published by the Free Software Foundation.
+ *   This file is part of medit.  medit is free software; you can
+ *   redistribute it and/or modify it under the terms of the
+ *   GNU Lesser General Public License as published by the
+ *   Free Software Foundation; either version 2.1 of the License,
+ *   or (at your option) any later version.
  *
- *   See COPYING file that comes with this distribution.
+ *   You should have received a copy of the GNU Lesser General Public
+ *   License along with medit.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "mooutils/moospawn.h"
 #include "marshals.h"
 #include "mooutils/mooutils-misc.h"
-#include "mooutils/moocompat.h"
 #include <string.h>
 
 #ifndef __WIN32__
@@ -202,7 +204,7 @@ _moo_cmd_new (const char *working_dir,
 
     g_return_val_if_fail (argv && *argv, NULL);
 
-    cmd = g_object_new (MOO_TYPE_CMD, NULL);
+    cmd = g_object_new (MOO_TYPE_CMD, (const char*) NULL);
 
     result = moo_cmd_run_command (cmd, working_dir, argv, envp,
                                   flags, cmd_flags, child_setup,
@@ -277,7 +279,7 @@ process_line (MooCmd      *cmd,
 
         if (g_get_charset (&charset))
         {
-            g_warning ("%s: invalid unicode:\n%s", G_STRLOC, line);
+            g_warning ("invalid unicode:\n%s", line);
 
             if (end > line)
                 real_line = g_strndup (line, end - line);
@@ -293,9 +295,9 @@ process_line (MooCmd      *cmd,
 
             if (!real_line)
             {
-                g_warning ("%s: could not convert text to UTF-8:\n%.*s",
-                           G_STRLOC, (int) len, line);
-                g_warning ("%s: %s", G_STRLOC, error->message);
+                g_warning ("could not convert text to UTF-8:\n%.*s",
+                           (int) len, line);
+                g_warning ("%s", moo_error_message (error));
             }
         }
     }
@@ -324,7 +326,7 @@ command_out_or_err (MooCmd         *cmd,
 {
     GSList *lines;
     GError *error = NULL;
-    GIOStatus status;
+    GIOStatus status = G_IO_STATUS_NORMAL;
     gsize count;
 
     count = 0;
@@ -356,7 +358,7 @@ command_out_or_err (MooCmd         *cmd,
 
     if (error)
     {
-        g_warning ("%s: %s", G_STRLOC, error->message);
+        g_warning ("%s", moo_error_message (error));
         g_error_free (error);
         return FALSE;
     }
@@ -528,6 +530,8 @@ moo_cmd_run_command (MooCmd     *cmd,
     else
 #endif
     {
+        flags |= MOO_SPAWN_WIN32_HIDDEN_CONSOLE;
+
         if ((flags & G_SPAWN_STDOUT_TO_DEV_NULL) || (cmd_flags & MOO_CMD_STDOUT_TO_PARENT))
             outp = NULL;
         else
@@ -661,7 +665,7 @@ moo_cmd_cleanup (MooCmd *cmd)
 #ifndef __WIN32__
         kill (-cmd->priv->pid, SIGHUP);
 #else
-        TerminateProcess (cmd->priv->pid, -1);
+        TerminateProcess (cmd->priv->pid, 1);
 #endif
     }
 
@@ -751,3 +755,31 @@ _moo_unix_spawn_async (char      **argv,
     return g_spawn_async (NULL, argv, NULL, g_flags,
                           NULL, NULL, NULL, error);
 }
+
+gboolean
+moo_spawn_command_line_async_with_flags (const gchar *command_line,
+                                         GSpawnFlags  g_flags,
+                                         GError     **error)
+{
+  gboolean retval;
+  gchar **argv = 0;
+
+  g_return_val_if_fail (command_line != NULL, FALSE);
+
+  if (!g_shell_parse_argv (command_line,
+                           NULL, &argv,
+                           error))
+    return FALSE;
+
+  retval = g_spawn_async (NULL,
+                          argv,
+                          NULL,
+                          G_SPAWN_SEARCH_PATH | g_flags,
+                          NULL,
+                          NULL,
+                          NULL,
+                          error);
+  g_strfreev (argv);
+
+  return retval;
+}
