@@ -1,13 +1,28 @@
 /*
  *   moouixml.c
  *
- *   Copyright (C) 2004-2007 by Yevgen Muntyan <muntyan@math.tamu.edu>
+ *   Copyright (C) 2004-2010 by Yevgen Muntyan <emuntyan@users.sourceforge.net>
  *
- *   This library is free software; you can redistribute it and/or
- *   modify it under the terms of the GNU Lesser General Public
- *   License version 2.1 as published by the Free Software Foundation.
+ *   This file is part of medit.  medit is free software; you can
+ *   redistribute it and/or modify it under the terms of the
+ *   GNU Lesser General Public License as published by the
+ *   Free Software Foundation; either version 2.1 of the License,
+ *   or (at your option) any later version.
  *
- *   See COPYING file that comes with this distribution.
+ *   You should have received a copy of the GNU Lesser General Public
+ *   License along with medit.  If not, see <http://www.gnu.org/licenses/>.
+ */
+
+/**
+ * class:MooUiXml: (parent GObject) (constructable) (moo.private 1)
+ **/
+
+/**
+ * pointer:MooUiNode: (moo.private 1)
+ */
+
+/**
+ * enum:MooUiWidgetType: (moo.private 1)
  */
 
 #include "mooutils/mooaction-private.h"
@@ -16,6 +31,9 @@
 #include "mooutils/moomenutoolbutton.h"
 #include "mooutils/mooutils-misc.h"
 #include "mooutils/mooi18n.h"
+#include "mooutils/moomenu.h"
+#include "mooutils/mootype-macros.h"
+#include "mooutils/moocompat.h"
 #include <gtk/gtk.h>
 #include <string.h>
 
@@ -25,12 +43,12 @@
 
 /* XXX weak ref actions */
 
-typedef MooUINodeType NodeType;
-typedef MooUINode Node;
-typedef MooUIWidgetNode Widget;
-typedef MooUIItemNode Item;
-typedef MooUISeparatorNode Separator;
-typedef MooUIPlaceholderNode Placeholder;
+typedef MooUiNodeType NodeType;
+typedef MooUiNode Node;
+typedef MooUiWidgetNode Widget;
+typedef MooUiItemNode Item;
+typedef MooUiSeparatorNode Separator;
+typedef MooUiPlaceholderNode Placeholder;
 
 #define CONTAINER MOO_UI_NODE_CONTAINER
 #define WIDGET MOO_UI_NODE_WIDGET
@@ -48,7 +66,7 @@ static const char *NODE_TYPE_NAME[] = {
     "MOO_UI_NODE_PLACEHOLDER"
 };
 
-struct _MooUIXMLPrivate {
+struct _MooUiXmlPrivate {
     Node *ui;
     GSList *toplevels;  /* Toplevel* */
     guint last_merge_id;
@@ -77,6 +95,8 @@ typedef enum {
 
 #define TOPLEVEL_QUARK (toplevel_quark ())
 #define NODE_QUARK (node_quark ())
+MOO_DEFINE_QUARK_STATIC (moo-ui-xml-toplevel, toplevel_quark)
+MOO_DEFINE_QUARK_STATIC (moo-ui-xml-node, node_quark)
 
 #define SLIST_FOREACH(list,var)                     \
 G_STMT_START {                                      \
@@ -93,10 +113,10 @@ typedef gboolean (*NodeForeachFunc) (Node       *node,
 
 static void     moo_ui_xml_finalize     (GObject        *object);
 
-static void     xml_add_markup          (MooUIXML       *xml,
+static void     xml_add_markup          (MooUiXml       *xml,
                                          MooMarkupNode  *mnode);
 
-static void     update_widgets          (MooUIXML       *xml,
+static void     update_widgets          (MooUiXml       *xml,
                                          UpdateType      type,
                                          Node           *node);
 
@@ -112,7 +132,7 @@ static gboolean item_check              (Node           *node);
 static gboolean widget_check            (Node           *node);
 static gboolean container_check         (Node           *node);
 
-static Merge   *lookup_merge            (MooUIXML       *xml,
+static Merge   *lookup_merge            (MooUiXml       *xml,
                                          guint           id);
 
 static Node    *node_new                (NodeType        type,
@@ -138,36 +158,33 @@ static void     toplevel_free           (Toplevel       *toplevel);
 static GtkWidget *toplevel_get_widget   (Toplevel       *toplevel,
                                          Node           *node);
 
-static GQuark   toplevel_quark          (void);
-static GQuark   node_quark              (void);
-
-static void     xml_add_item_widget     (MooUIXML       *xml,
+static void     xml_add_item_widget     (MooUiXml       *xml,
                                          GtkWidget      *widget);
-static void     xml_add_widget          (MooUIXML       *xml,
+static void     xml_add_widget          (MooUiXml       *xml,
                                          GtkWidget      *widget,
                                          Toplevel       *toplevel,
                                          Node           *node);
-static void     xml_remove_widget       (MooUIXML       *xml,
+static void     xml_remove_widget       (MooUiXml       *xml,
                                          GtkWidget      *widget);
-static void     xml_delete_toplevel     (MooUIXML       *xml,
+static void     xml_delete_toplevel     (MooUiXml       *xml,
                                          Toplevel       *toplevel);
-static void     xml_connect_toplevel    (MooUIXML       *xml,
+static void     xml_connect_toplevel    (MooUiXml       *xml,
                                          Toplevel       *toplevel);
 
-static gboolean create_menu_separator   (MooUIXML       *xml,
+static gboolean create_menu_separator   (MooUiXml       *xml,
                                          Toplevel       *toplevel,
                                          GtkMenuShell   *menu,
                                          Node           *node,
                                          int             index);
-static void     create_menu_item        (MooUIXML       *xml,
+static void     create_menu_item        (MooUiXml       *xml,
                                          Toplevel       *toplevel,
                                          GtkMenuShell   *menu,
                                          Node           *node,
                                          int             index);
-static gboolean create_menu_shell       (MooUIXML       *xml,
+static gboolean create_menu_shell       (MooUiXml       *xml,
                                          Toplevel       *toplevel,
-                                         MooUIWidgetType type);
-static gboolean fill_menu_shell         (MooUIXML       *xml,
+                                         MooUiWidgetType type);
+static gboolean fill_menu_shell         (MooUiXml       *xml,
                                          Toplevel       *toplevel,
                                          Node           *menu_node,
                                          GtkMenuShell   *menu);
@@ -178,33 +195,33 @@ static void     check_empty             (Node           *parent,
                                          GtkWidget      *widget,
                                          Toplevel       *toplevel);
 
-static gboolean create_tool_separator   (MooUIXML       *xml,
+static gboolean create_tool_separator   (MooUiXml       *xml,
                                          Toplevel       *toplevel,
                                          GtkToolbar     *toolbar,
                                          Node           *node,
                                          int             index);
-static gboolean create_tool_item        (MooUIXML       *xml,
+static gboolean create_tool_item        (MooUiXml       *xml,
                                          Toplevel       *toplevel,
                                          GtkToolbar     *toolbar,
                                          Node           *node,
                                          int             index);
-static gboolean fill_toolbar            (MooUIXML       *xml,
+static gboolean fill_toolbar            (MooUiXml       *xml,
                                          Toplevel       *toplevel,
                                          Node           *toolbar_node,
                                          GtkToolbar     *toolbar);
-static gboolean create_toolbar          (MooUIXML       *xml,
+static gboolean create_toolbar          (MooUiXml       *xml,
                                          Toplevel       *toplevel);
 
-static MooUINode *moo_ui_xml_find_node  (MooUIXML       *xml,
+static MooUiNode *moo_ui_xml_find_node  (MooUiXml       *xml,
                                          const char     *path_or_placeholder);
 
 
 /* MOO_TYPE_UI_XML */
-G_DEFINE_TYPE (MooUIXML, moo_ui_xml, G_TYPE_OBJECT)
+G_DEFINE_TYPE (MooUiXml, moo_ui_xml, G_TYPE_OBJECT)
 
 
 static void
-moo_ui_xml_class_init (MooUIXMLClass *klass)
+moo_ui_xml_class_init (MooUiXmlClass *klass)
 {
     GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
 
@@ -221,19 +238,19 @@ node_new (NodeType type, const char *name)
     switch (type)
     {
         case MOO_UI_NODE_CONTAINER:
-            size = sizeof (MooUINode);
+            size = sizeof (MooUiNode);
             break;
         case MOO_UI_NODE_WIDGET:
-            size = sizeof (MooUIWidgetNode);
+            size = sizeof (MooUiWidgetNode);
             break;
         case MOO_UI_NODE_ITEM:
-            size = sizeof (MooUIItemNode);
+            size = sizeof (MooUiItemNode);
             break;
         case MOO_UI_NODE_SEPARATOR:
-            size = sizeof (MooUISeparatorNode);
+            size = sizeof (MooUiSeparatorNode);
             break;
         case MOO_UI_NODE_PLACEHOLDER:
-            size = sizeof (MooUIPlaceholderNode);
+            size = sizeof (MooUiPlaceholderNode);
             break;
     }
 
@@ -245,17 +262,17 @@ node_new (NodeType type, const char *name)
 
 
 static void
-moo_ui_xml_init (MooUIXML *xml)
+moo_ui_xml_init (MooUiXml *xml)
 {
-    xml->priv = g_new0 (MooUIXMLPrivate, 1);
+    xml->priv = g_new0 (MooUiXmlPrivate, 1);
     xml->priv->ui = node_new (CONTAINER, "ui");
 }
 
 
-MooUIXML*
+MooUiXml*
 moo_ui_xml_new (void)
 {
-    return g_object_new (MOO_TYPE_UI_XML, NULL);
+    return g_object_new (MOO_TYPE_UI_XML, (const char*) NULL);
 }
 
 
@@ -270,7 +287,7 @@ parse_object (MooMarkupNode *mnode)
 
     if (!name || !name[0])
     {
-        g_warning ("%s: object name missing", G_STRLOC);
+        g_warning ("object name missing");
         return NULL;
     }
 
@@ -310,7 +327,7 @@ parse_widget (MooMarkupNode *mnode)
 
     if (!name || !name[0])
     {
-        g_warning ("%s: widget name missing", G_STRLOC);
+        g_warning ("widget name missing");
         return NULL;
     }
 
@@ -350,7 +367,7 @@ parse_placeholder (MooMarkupNode *mnode)
 
     if (!name || !name[0])
     {
-        g_warning ("%s: placeholder name missing", G_STRLOC);
+        g_warning ("placeholder name missing");
         return NULL;
     }
 
@@ -530,7 +547,7 @@ parse_markup (MooMarkupNode *mnode)
     else if (!strcmp (mnode->name, "placeholder"))
         return parse_placeholder (mnode);
 
-    g_warning ("%s: unknown element '%s'", G_STRLOC, mnode->name);
+    g_warning ("unknown element '%s'", mnode->name);
     return NULL;
 }
 
@@ -599,10 +616,17 @@ node_free (Node *node)
 }
 
 
+/**
+ * moo_ui_xml_add_ui_from_string:
+ *
+ * @xml:
+ * @buffer: (type const-utf8)
+ * @length: (default -1)
+ */
 void
-moo_ui_xml_add_ui_from_string (MooUIXML       *xml,
-                               const char     *buffer,
-                               gssize          length)
+moo_ui_xml_add_ui_from_string (MooUiXml   *xml,
+                               const char *buffer,
+                               int         length)
 {
     MooMarkupDoc *doc;
     MooMarkupNode *ui_node, *child;
@@ -615,12 +639,8 @@ moo_ui_xml_add_ui_from_string (MooUIXML       *xml,
 
     if (!doc)
     {
-        g_critical ("%s: could not parse markup", G_STRLOC);
-        if (error)
-        {
-            g_critical ("%s: %s", G_STRLOC, error->message);
-            g_error_free (error);
-        }
+        g_critical ("could not parse markup: %s", moo_error_message (error));
+        g_error_free (error);
         return;
     }
 
@@ -681,8 +701,8 @@ placeholder_check (Node *node)
                 break;
 
             default:
-                g_warning ("%s: invalid placeholder child type %s",
-                           G_STRLOC, NODE_TYPE_NAME[child->type]);
+                g_warning ("invalid placeholder child type %s",
+                           NODE_TYPE_NAME[child->type]);
                 return FALSE;
         }
     }
@@ -713,8 +733,8 @@ item_check (Node *node)
                 break;
 
             default:
-                g_warning ("%s: invalid item child type %s",
-                           G_STRLOC, NODE_TYPE_NAME[child->type]);
+                g_warning ("invalid item child type %s",
+                           NODE_TYPE_NAME[child->type]);
                 return FALSE;
         }
     }
@@ -745,8 +765,8 @@ widget_check (Node *node)
                 break;
 
             default:
-                g_warning ("%s: invalid widget child type %s",
-                           G_STRLOC, NODE_TYPE_NAME[child->type]);
+                g_warning ("invalid widget child type %s",
+                           NODE_TYPE_NAME[child->type]);
                 return FALSE;
         }
     }
@@ -776,8 +796,8 @@ container_check (Node *node)
                 break;
 
             default:
-                g_warning ("%s: invalid widget child type %s",
-                           G_STRLOC, NODE_TYPE_NAME[child->type]);
+                g_warning ("invalid widget child type %s",
+                           NODE_TYPE_NAME[child->type]);
                 return FALSE;
         }
     }
@@ -788,7 +808,7 @@ container_check (Node *node)
 
 
 static void
-xml_add_markup (MooUIXML       *xml,
+xml_add_markup (MooUiXml       *xml,
                 MooMarkupNode  *mnode)
 {
     Node *node = parse_markup (mnode);
@@ -817,15 +837,15 @@ xml_add_markup (MooUIXML       *xml,
         case ITEM:
         case SEPARATOR:
         case PLACEHOLDER:
-            g_warning ("%s: invalid toplevel type %s",
-                       G_STRLOC, NODE_TYPE_NAME[node->type]);
+            g_warning ("invalid toplevel type %s",
+                       NODE_TYPE_NAME[node->type]);
             node_free (node);
             return;
     }
 
     if (moo_ui_xml_get_node (xml, node->name))
     {
-        g_warning ("%s: implement me?", G_STRLOC);
+        g_warning ("implement me?");
         node_free (node);
         return;
     }
@@ -835,8 +855,11 @@ xml_add_markup (MooUIXML       *xml,
 }
 
 
+/**
+ * moo_ui_xml_new_merge_id:
+ */
 guint
-moo_ui_xml_new_merge_id (MooUIXML *xml)
+moo_ui_xml_new_merge_id (MooUiXml *xml)
 {
     Merge *merge;
 
@@ -854,7 +877,7 @@ moo_ui_xml_new_merge_id (MooUIXML *xml)
 
 
 static Merge*
-lookup_merge (MooUIXML *xml,
+lookup_merge (MooUiXml *xml,
               guint     merge_id)
 {
     GSList *l;
@@ -870,8 +893,18 @@ lookup_merge (MooUIXML *xml,
 }
 
 
-MooUINode*
-moo_ui_xml_add_item (MooUIXML       *xml,
+/**
+ * moo_ui_xml_add_item:
+ *
+ * @xml:
+ * @merge_id:
+ * @parent_path: (type const-utf8)
+ * @name: (type const-utf8)
+ * @action: (type const-utf8)
+ * @position:
+ */
+MooUiNode*
+moo_ui_xml_add_item (MooUiXml       *xml,
                      guint           merge_id,
                      const char     *parent_path,
                      const char     *name,
@@ -879,7 +912,7 @@ moo_ui_xml_add_item (MooUIXML       *xml,
                      int             position)
 {
     Merge *merge;
-    MooUINode *parent;
+    MooUiNode *parent;
     Item *item;
 
     g_return_val_if_fail (MOO_IS_UI_XML (xml), NULL);
@@ -903,8 +936,8 @@ moo_ui_xml_add_item (MooUIXML       *xml,
 
         case MOO_UI_NODE_CONTAINER:
         case MOO_UI_NODE_SEPARATOR:
-            g_warning ("%s: can't add item to node of type %s",
-                       G_STRLOC, NODE_TYPE_NAME[parent->type]);
+            g_warning ("can't add item to node of type %s",
+                       NODE_TYPE_NAME[parent->type]);
     }
 
     item = item_new (name, action);
@@ -918,10 +951,19 @@ moo_ui_xml_add_item (MooUIXML       *xml,
 }
 
 
+/**
+ * moo_ui_xml_insert:
+ *
+ * @xml:
+ * @merge_id:
+ * @parent:
+ * @position:
+ * @markup: (type const-utf8)
+ */
 void
-moo_ui_xml_insert (MooUIXML       *xml,
+moo_ui_xml_insert (MooUiXml       *xml,
                    guint           merge_id,
-                   MooUINode      *parent,
+                   MooUiNode      *parent,
                    int             position,
                    const char     *markup)
 {
@@ -943,8 +985,8 @@ moo_ui_xml_insert (MooUIXML       *xml,
 
     if (parent->type == MOO_UI_NODE_SEPARATOR)
     {
-        g_warning ("%s: can't add stuff to node of type %s",
-                   G_STRLOC, NODE_TYPE_NAME[parent->type]);
+        g_warning ("can't add stuff to node of type %s",
+                   NODE_TYPE_NAME[parent->type]);
         return;
     }
 
@@ -952,12 +994,8 @@ moo_ui_xml_insert (MooUIXML       *xml,
 
     if (!doc)
     {
-        g_warning ("%s: could not parse markup", G_STRLOC);
-        if (error)
-        {
-            g_warning ("%s: %s", G_STRLOC, error->message);
-            g_error_free (error);
-        }
+        g_warning ("could not parse markup: %s", moo_error_message (error));
+        g_error_free (error);
         return;
     }
 
@@ -989,8 +1027,8 @@ moo_ui_xml_insert (MooUIXML       *xml,
             case CONTAINER:
                 if (parent->type != CONTAINER)
                 {
-                    g_warning ("%s: can not add node of type %s to node of type %s",
-                               G_STRLOC, NODE_TYPE_NAME[node->type],
+                    g_warning ("can not add node of type %s to node of type %s",
+                               NODE_TYPE_NAME[node->type],
                                NODE_TYPE_NAME[parent->type]);
                     node_free (node);
                     continue;
@@ -1002,8 +1040,8 @@ moo_ui_xml_insert (MooUIXML       *xml,
             case SEPARATOR:
                 if (parent->type == SEPARATOR || parent->type == CONTAINER)
                 {
-                    g_warning ("%s: can not add node of type %s to node of type %s",
-                               G_STRLOC, NODE_TYPE_NAME[node->type],
+                    g_warning ("can not add node of type %s to node of type %s",
+                               NODE_TYPE_NAME[node->type],
                                NODE_TYPE_NAME[parent->type]);
                     node_free (node);
                     continue;
@@ -1023,11 +1061,20 @@ moo_ui_xml_insert (MooUIXML       *xml,
 }
 
 
+/**
+ * moo_ui_xml_insert_after:
+ *
+ * @xml:
+ * @merge_id:
+ * @parent:
+ * @after:
+ * @markup: (type const-utf8)
+ */
 void
-moo_ui_xml_insert_after (MooUIXML       *xml,
+moo_ui_xml_insert_after (MooUiXml       *xml,
                          guint           merge_id,
-                         MooUINode      *parent,
-                         MooUINode      *after,
+                         MooUiNode      *parent,
+                         MooUiNode      *after,
                          const char     *markup)
 {
     int position;
@@ -1048,11 +1095,20 @@ moo_ui_xml_insert_after (MooUIXML       *xml,
 }
 
 
+/**
+ * moo_ui_xml_insert_before:
+ *
+ * @xml:
+ * @merge_id:
+ * @parent:
+ * @before:
+ * @markup: (type const-utf8)
+ */
 void
-moo_ui_xml_insert_before (MooUIXML       *xml,
+moo_ui_xml_insert_before (MooUiXml       *xml,
                           guint           merge_id,
-                          MooUINode      *parent,
-                          MooUINode      *before,
+                          MooUiNode      *parent,
+                          MooUiNode      *before,
                           const char     *markup)
 {
     int position;
@@ -1073,8 +1129,17 @@ moo_ui_xml_insert_before (MooUIXML       *xml,
 }
 
 
+/**
+ * moo_ui_xml_insert_markup_after:
+ *
+ * @xml:
+ * @merge_id:
+ * @parent_path: (type const-utf8)
+ * @after_name: (type const-utf8)
+ * @markup: (type const-utf8)
+ */
 void
-moo_ui_xml_insert_markup_after (MooUIXML       *xml,
+moo_ui_xml_insert_markup_after (MooUiXml       *xml,
                                 guint           merge_id,
                                 const char     *parent_path,
                                 const char     *after_name,
@@ -1105,8 +1170,17 @@ moo_ui_xml_insert_markup_after (MooUIXML       *xml,
 }
 
 
+/**
+ * moo_ui_xml_insert_markup_before:
+ *
+ * @xml:
+ * @merge_id:
+ * @parent_path: (type const-utf8)
+ * @before_name: (type const-utf8)
+ * @markup: (type const-utf8)
+ */
 void
-moo_ui_xml_insert_markup_before (MooUIXML       *xml,
+moo_ui_xml_insert_markup_before (MooUiXml       *xml,
                                  guint           merge_id,
                                  const char     *parent_path,
                                  const char     *before_name,
@@ -1137,8 +1211,17 @@ moo_ui_xml_insert_markup_before (MooUIXML       *xml,
 }
 
 
+/**
+ * moo_ui_xml_insert_markup:
+ *
+ * @xml:
+ * @merge_id:
+ * @parent_path: (type const-utf8)
+ * @position:
+ * @markup: (type const-utf8)
+ */
 void
-moo_ui_xml_insert_markup (MooUIXML       *xml,
+moo_ui_xml_insert_markup (MooUiXml       *xml,
                           guint           merge_id,
                           const char     *parent_path,
                           int             position,
@@ -1180,8 +1263,11 @@ node_is_ancestor (Node           *node,
 }
 
 
+/**
+ * moo_ui_xml_remove_ui:
+ */
 void
-moo_ui_xml_remove_ui (MooUIXML       *xml,
+moo_ui_xml_remove_ui (MooUiXml       *xml,
                       guint           merge_id)
 {
     Merge *merge;
@@ -1205,9 +1291,12 @@ moo_ui_xml_remove_ui (MooUIXML       *xml,
 }
 
 
+/**
+ * moo_ui_xml_remove_node:
+ */
 void
-moo_ui_xml_remove_node (MooUIXML       *xml,
-                        MooUINode      *node)
+moo_ui_xml_remove_node (MooUiXml       *xml,
+                        MooUiNode      *node)
 {
     Node *parent;
 
@@ -1284,8 +1373,14 @@ merge_remove_node (Merge          *merge,
 }
 
 
-MooUINode *
-moo_ui_xml_get_node (MooUIXML       *xml,
+/**
+ * moo_ui_xml_get_node:
+ *
+ * @xml:
+ * @path: (type const-utf8)
+ */
+MooUiNode *
+moo_ui_xml_get_node (MooUiXml       *xml,
                      const char     *path)
 {
     g_return_val_if_fail (MOO_IS_UI_XML (xml), NULL);
@@ -1295,11 +1390,11 @@ moo_ui_xml_get_node (MooUIXML       *xml,
 }
 
 
-static MooUINode *
-moo_ui_xml_find_node (MooUIXML       *xml,
+static MooUiNode *
+moo_ui_xml_find_node (MooUiXml       *xml,
                       const char     *path)
 {
-    MooUINode *node;
+    MooUiNode *node;
 
     g_return_val_if_fail (MOO_IS_UI_XML (xml), NULL);
     g_return_val_if_fail (path != NULL, NULL);
@@ -1334,8 +1429,14 @@ find_placeholder_func (Node    *node,
     return FALSE;
 }
 
-MooUINode*
-moo_ui_xml_find_placeholder (MooUIXML       *xml,
+/**
+ * moo_ui_xml_find_placeholder:
+ *
+ * @xml:
+ * @name: (type const-utf8)
+ */
+MooUiNode*
+moo_ui_xml_find_placeholder (MooUiXml       *xml,
                              const char     *name)
 {
     struct {
@@ -1354,8 +1455,13 @@ moo_ui_xml_find_placeholder (MooUIXML       *xml,
 }
 
 
-char*
-moo_ui_node_get_path (MooUINode *node)
+/**
+ * moo_ui_node_get_path:
+ *
+ * Returns: (type utf8)
+ */
+char *
+moo_ui_node_get_path (MooUiNode *node)
 {
     GString *path;
 
@@ -1375,8 +1481,14 @@ moo_ui_node_get_path (MooUINode *node)
 }
 
 
-MooUINode*
-moo_ui_node_get_child (MooUINode      *node,
+/**
+ * moo_ui_node_get_child:
+ *
+ * @node:
+ * @name: (type const-utf8)
+ */
+MooUiNode *
+moo_ui_node_get_child (MooUiNode      *node,
                        const char     *path)
 {
     char **pieces, **p;
@@ -1454,23 +1566,6 @@ toplevel_free (Toplevel *toplevel)
 }
 
 
-static GQuark toplevel_quark (void)
-{
-    static GQuark q = 0;
-    if (!q)
-        q = g_quark_from_static_string ("moo-ui-xml-toplevel");
-    return q;
-}
-
-static GQuark node_quark (void)
-{
-    static GQuark q = 0;
-    if (!q)
-        q = g_quark_from_static_string ("moo-ui-xml-node");
-    return q;
-}
-
-
 static Node *
 get_effective_parent (Node *node)
 {
@@ -1489,7 +1584,7 @@ get_effective_parent (Node *node)
 static void
 visibility_notify (GtkWidget *widget,
                    G_GNUC_UNUSED gpointer whatever,
-                   MooUIXML  *xml)
+                   MooUiXml  *xml)
 {
     Toplevel *toplevel;
     Node *node;
@@ -1511,7 +1606,7 @@ visibility_notify (GtkWidget *widget,
 }
 
 static void
-xml_add_item_widget (MooUIXML       *xml,
+xml_add_item_widget (MooUiXml       *xml,
                      GtkWidget      *widget)
 {
     g_signal_connect (widget, "notify::visible",
@@ -1521,7 +1616,7 @@ xml_add_item_widget (MooUIXML       *xml,
 
 static void
 widget_destroyed (GtkWidget *widget,
-                  MooUIXML  *xml)
+                  MooUiXml  *xml)
 {
     g_return_if_fail (GTK_IS_WIDGET (widget));
     g_return_if_fail (MOO_IS_UI_XML (xml));
@@ -1530,7 +1625,7 @@ widget_destroyed (GtkWidget *widget,
 
 
 static void
-xml_remove_widget (MooUIXML  *xml,
+xml_remove_widget (MooUiXml  *xml,
                    GtkWidget *widget)
 {
     Toplevel *toplevel;
@@ -1558,7 +1653,7 @@ xml_remove_widget (MooUIXML  *xml,
 
 
 static void
-xml_add_widget (MooUIXML  *xml,
+xml_add_widget (MooUiXml  *xml,
                 GtkWidget *widget,
                 Toplevel  *toplevel,
                 Node      *node)
@@ -1590,7 +1685,7 @@ hash_table_list_values (GHashTable *hash_table)
 
 static void
 toplevel_destroyed (GtkWidget *widget,
-                    MooUIXML  *xml)
+                    MooUiXml  *xml)
 {
     Toplevel *toplevel;
 
@@ -1606,7 +1701,7 @@ toplevel_destroyed (GtkWidget *widget,
 
 
 static void
-xml_delete_toplevel (MooUIXML *xml,
+xml_delete_toplevel (MooUiXml *xml,
                      Toplevel *toplevel)
 {
     GSList *children, *l;
@@ -1633,7 +1728,7 @@ xml_delete_toplevel (MooUIXML *xml,
 
 
 static void
-xml_connect_toplevel (MooUIXML  *xml,
+xml_connect_toplevel (MooUiXml  *xml,
                       Toplevel  *toplevel)
 {
     g_signal_connect (toplevel->widget, "destroy",
@@ -1645,7 +1740,7 @@ xml_connect_toplevel (MooUIXML  *xml,
 
 
 static gboolean
-create_menu_separator (MooUIXML       *xml,
+create_menu_separator (MooUiXml       *xml,
                        Toplevel       *toplevel,
                        GtkMenuShell   *menu,
                        Node           *node,
@@ -1684,7 +1779,7 @@ static gboolean node_is_empty (Node *node)
 
 
 static void
-create_menu_item (MooUIXML       *xml,
+create_menu_item (MooUiXml       *xml,
                   Toplevel       *toplevel,
                   GtkMenuShell   *menu,
                   Node           *node,
@@ -1709,8 +1804,7 @@ create_menu_item (MooUIXML       *xml,
         if (!action)
         {
 #if REPORT_UNKNOWN_ACTIONS
-            g_critical ("%s: could not find action '%s'",
-                        G_STRLOC, item->action);
+            g_critical ("could not find action '%s'", item->action);
 #endif
             return;
         }
@@ -1966,7 +2060,7 @@ check_separators (Node           *parent,
 
 
 static gboolean
-fill_menu_shell (MooUIXML       *xml,
+fill_menu_shell (MooUiXml       *xml,
                  Toplevel       *toplevel,
                  Node           *menu_node,
                  GtkMenuShell   *menu)
@@ -1989,8 +2083,8 @@ fill_menu_shell (MooUIXML       *xml,
                 break;
 
             default:
-                g_warning ("%s: invalid menu item type %s",
-                           G_STRLOC, NODE_TYPE_NAME[node->type]);
+                g_warning ("invalid menu item type %s",
+                           NODE_TYPE_NAME[node->type]);
                 return FALSE;
         }
     }
@@ -2004,9 +2098,9 @@ fill_menu_shell (MooUIXML       *xml,
 
 
 static gboolean
-create_menu_shell (MooUIXML       *xml,
+create_menu_shell (MooUiXml       *xml,
                    Toplevel       *toplevel,
-                   MooUIWidgetType type)
+                   MooUiWidgetType type)
 {
     g_return_val_if_fail (toplevel != NULL, FALSE);
     g_return_val_if_fail (toplevel->widget == NULL, FALSE);
@@ -2018,7 +2112,7 @@ create_menu_shell (MooUIXML       *xml,
     }
     else
     {
-        toplevel->widget = gtk_menu_new ();
+        toplevel->widget = moo_menu_new ();
         gtk_menu_set_accel_group (GTK_MENU (toplevel->widget),
                                   toplevel->accel_group);
     }
@@ -2032,7 +2126,7 @@ create_menu_shell (MooUIXML       *xml,
 
 
 static gboolean
-create_tool_separator (MooUIXML       *xml,
+create_tool_separator (MooUiXml       *xml,
                        Toplevel       *toplevel,
                        GtkToolbar     *toolbar,
                        Node           *node,
@@ -2074,7 +2168,7 @@ menu_tool_button_get_menu (GtkWidget *button)
 
 
 static gboolean
-create_tool_item (MooUIXML       *xml,
+create_tool_item (MooUiXml       *xml,
                   Toplevel       *toplevel,
                   GtkToolbar     *toolbar,
                   Node           *node,
@@ -2120,7 +2214,7 @@ create_tool_item (MooUIXML       *xml,
         {
             if (!IS_MENU_TOOL_BUTTON (tool_item))
             {
-                g_critical ("%s: oops", G_STRLOC);
+                g_critical ("oops");
             }
             else
             {
@@ -2173,7 +2267,7 @@ create_tool_item (MooUIXML       *xml,
 
 
 static gboolean
-fill_toolbar (MooUIXML       *xml,
+fill_toolbar (MooUiXml       *xml,
               Toplevel       *toplevel,
               Node           *toolbar_node,
               GtkToolbar     *toolbar)
@@ -2197,8 +2291,8 @@ fill_toolbar (MooUIXML       *xml,
                 break;
 
             default:
-                g_warning ("%s: invalid tool item type %s",
-                           G_STRLOC, NODE_TYPE_NAME[node->type]);
+                g_warning ("invalid tool item type %s",
+                           NODE_TYPE_NAME[node->type]);
                 return FALSE;
         }
 
@@ -2215,7 +2309,7 @@ fill_toolbar (MooUIXML       *xml,
 
 
 static gboolean
-create_toolbar (MooUIXML       *xml,
+create_toolbar (MooUiXml       *xml,
                 Toplevel       *toplevel)
 {
     g_return_val_if_fail (toplevel != NULL, FALSE);
@@ -2232,9 +2326,20 @@ create_toolbar (MooUIXML       *xml,
 }
 
 
+/**
+ * moo_ui_xml_create_widget:
+ *
+ * @xml:
+ * @type:
+ * @path: (type const-utf8)
+ * @actions:
+ * @accel_group:
+ *
+ * Returns: (type GtkWidget*)
+ */
 gpointer
-moo_ui_xml_create_widget (MooUIXML            *xml,
-                          MooUIWidgetType      type,
+moo_ui_xml_create_widget (MooUiXml            *xml,
+                          MooUiWidgetType      type,
                           const char          *path,
                           MooActionCollection *actions,
                           GtkAccelGroup       *accel_group)
@@ -2252,14 +2357,14 @@ moo_ui_xml_create_widget (MooUIXML            *xml,
 
     if (node->type != WIDGET)
     {
-        g_warning ("%s: can create widgets only for nodes of type %s",
-                   G_STRLOC, NODE_TYPE_NAME[WIDGET]);
+        g_warning ("can create widgets only for nodes of type %s",
+                   NODE_TYPE_NAME[WIDGET]);
         return NULL;
     }
 
     if (type < 1 || type > 3)
     {
-        g_warning ("%s: invalid widget type %u", G_STRLOC, type);
+        g_warning ("invalid widget type %u", type);
         return NULL;
     }
 
@@ -2293,13 +2398,20 @@ moo_ui_xml_create_widget (MooUIXML            *xml,
 }
 
 
+/**
+ * moo_ui_xml_get_widget:
+ *
+ * @xml:
+ * @widget:
+ * @path: (type const-utf8)
+ */
 GtkWidget*
-moo_ui_xml_get_widget (MooUIXML       *xml,
+moo_ui_xml_get_widget (MooUiXml       *xml,
                        GtkWidget      *widget,
                        const char     *path)
 {
     Toplevel *toplevel;
-    MooUINode *node;
+    MooUiNode *node;
 
     g_return_val_if_fail (MOO_IS_UI_XML (xml), NULL);
     g_return_val_if_fail (GTK_IS_WIDGET (widget), NULL);
@@ -2343,7 +2455,7 @@ effective_index (Node *parent,
 
 
 static void
-toplevel_add_node (MooUIXML *xml,
+toplevel_add_node (MooUiXml *xml,
                    Toplevel *toplevel,
                    Node     *node)
 {
@@ -2463,7 +2575,7 @@ toplevel_add_node (MooUIXML *xml,
 
 
 static void
-toplevel_remove_node (G_GNUC_UNUSED MooUIXML *xml,
+toplevel_remove_node (G_GNUC_UNUSED MooUiXml *xml,
                       Toplevel *toplevel,
                       Node     *node)
 {
@@ -2480,7 +2592,7 @@ toplevel_remove_node (G_GNUC_UNUSED MooUIXML *xml,
 
 
 static void
-update_widgets (MooUIXML       *xml,
+update_widgets (MooUiXml       *xml,
                 UpdateType      type,
                 Node           *node)
 {
@@ -2511,7 +2623,7 @@ update_widgets (MooUIXML       *xml,
             break;
 
         case UPDATE_CHANGE_NODE:
-            g_warning ("%s: implement me", G_STRLOC);
+            g_warning ("implement me");
             break;
 
         default:
@@ -2523,7 +2635,7 @@ update_widgets (MooUIXML       *xml,
 static void
 moo_ui_xml_finalize (GObject *object)
 {
-    MooUIXML *xml = MOO_UI_XML (object);
+    MooUiXml *xml = MOO_UI_XML (object);
 
     SLIST_FOREACH (xml->priv->toplevels, t)
     {
@@ -2571,7 +2683,7 @@ moo_ui_node_get_type (void)
     static GType type = 0;
 
     if (G_UNLIKELY (!type))
-        type = g_pointer_type_register_static ("MooUINode");
+        type = g_pointer_type_register_static ("MooUiNode");
 
     return type;
 }
