@@ -1,17 +1,22 @@
 /*
  *   mooutils-gobject.c
  *
- *   Copyright (C) 2004-2007 by Yevgen Muntyan <muntyan@math.tamu.edu>
+ *   Copyright (C) 2004-2010 by Yevgen Muntyan <emuntyan@users.sourceforge.net>
  *
- *   This library is free software; you can redistribute it and/or
- *   modify it under the terms of the GNU Lesser General Public
- *   License version 2.1 as published by the Free Software Foundation.
+ *   This file is part of medit.  medit is free software; you can
+ *   redistribute it and/or modify it under the terms of the
+ *   GNU Lesser General Public License as published by the
+ *   Free Software Foundation; either version 2.1 of the License,
+ *   or (at your option) any later version.
  *
- *   See COPYING file that comes with this distribution.
+ *   You should have received a copy of the GNU Lesser General Public
+ *   License along with medit.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "mooutils/mooutils-gobject-private.h"
 #include "mooutils/mooclosure.h"
+#include "mooutils/mootype-macros.h"
+#include <mooutils/mooutils-tests.h>
 #include <gobject/gvaluecollector.h>
 #include <string.h>
 #include <errno.h>
@@ -240,8 +245,10 @@ string_to_int (const char *string,
     if (errno || !end || *end)
         return FALSE;
 
+#if G_MAXINT != G_MAXLONG
     if (val > G_MAXINT || val < G_MININT)
         return FALSE;
+#endif
 
     *dest = val;
     return TRUE;
@@ -740,6 +747,22 @@ _moo_value_convert_to_int (const GValue *val,
     return FALSE;
 }
 
+static gboolean
+_moo_value_convert_to_uint (const GValue *val,
+                            guint        *dest)
+{
+    GValue result = {0};
+
+    g_value_init (&result, G_TYPE_UINT);
+
+    if (_moo_value_convert (val, &result))
+    {
+        *dest = g_value_get_uint (&result);
+        return TRUE;
+    }
+
+    return FALSE;
+}
 
 double
 _moo_value_convert_to_double (const GValue *val)
@@ -850,6 +873,28 @@ _moo_convert_string_to_int (const char *string,
     return int_val;
 }
 
+guint
+_moo_convert_string_to_uint (const char *string,
+                             guint       default_val)
+{
+    guint int_val = default_val;
+
+    if (string && string[0])
+    {
+        GValue str_val = {0};
+
+        g_value_init (&str_val, G_TYPE_STRING);
+        g_value_set_static_string (&str_val, string);
+
+        if (!_moo_value_convert_to_uint (&str_val, &int_val))
+            g_warning ("%s: could not convert string '%s' to uint",
+                       G_STRFUNC, string);
+
+        g_value_unset (&str_val);
+    }
+
+    return int_val;
+}
 
 gboolean
 _moo_convert_string_to_bool (const char *string,
@@ -887,7 +932,7 @@ _moo_convert_bool_to_string (gboolean value)
 }
 
 
-const char*
+const char *
 _moo_convert_int_to_string (int value)
 {
     GValue int_val = {0};
@@ -922,10 +967,6 @@ _moo_value_change_type (GValue *val,
 }
 
 
-#ifdef MOO_ENABLE_UNIT_TESTS
-
-#include <mooutils/mooutils-tests.h>
-
 static double
 string_to_double (const char *string)
 {
@@ -1141,13 +1182,11 @@ moo_test_gobject (void)
 {
     MooTestSuite *suite;
 
-    suite = moo_test_suite_new ("mooutils/mooutils-gobject.c", NULL, NULL, NULL);
+    suite = moo_test_suite_new ("mooutils-gobject", "mooutils/mooutils-gobject.c", NULL, NULL, NULL);
 
-    moo_test_suite_add_test (suite, "test of converting values", (MooTestFunc) test_misc, NULL);
+    moo_test_suite_add_test (suite, "convert", "test of converting values", (MooTestFunc) test_misc, NULL);
 }
 
-#endif
-
 
 /*****************************************************************************/
 /* GParameter array manipulation
@@ -1248,7 +1287,7 @@ _moo_param_array_add_type_valist (GParameter *src,
 
         if (G_TYPE_FUNDAMENTAL (type) == G_TYPE_INVALID)
         {
-            g_warning ("%s: invalid type id passed", G_STRLOC);
+            g_warning ("invalid type id passed");
             _moo_param_array_free ((GParameter*)copy->data, copy->len);
             g_array_free (copy, FALSE);
             return NULL;
@@ -1260,7 +1299,7 @@ _moo_param_array_add_type_valist (GParameter *src,
 
         if (error)
         {
-            g_critical ("%s: %s", G_STRLOC, error);
+            g_critical ("%s", error);
             g_free (error);
             g_value_unset (&param.value);
             g_free ((char*)param.name);
@@ -1311,8 +1350,8 @@ _moo_param_array_add_valist (GType       type,
         GParamSpec *pspec = g_object_class_find_property (klass, name);
 
         if (!pspec) {
-            g_warning ("%s: class '%s' doesn't have property '%s'",
-                       G_STRLOC, g_type_name (type), name);
+            g_warning ("class '%s' doesn't have property '%s'",
+                       g_type_name (type), name);
             _moo_param_array_free ((GParameter*)copy->data, copy->len);
             g_array_free (copy, FALSE);
             return NULL;
@@ -1323,7 +1362,7 @@ _moo_param_array_add_valist (GType       type,
         G_VALUE_COLLECT (&param.value, var_args, 0, &error);
 
         if (error) {
-            g_critical ("%s: %s", G_STRLOC, error);
+            g_critical ("%s", error);
             g_free (error);
             g_value_unset (&param.value);
             g_free ((char*)param.name);
@@ -1371,8 +1410,8 @@ _moo_param_array_collect_valist (GType       type,
 
         if (!pspec)
         {
-            g_warning ("%s: class '%s' doesn't have property '%s'",
-                       G_STRLOC, g_type_name (type), prop_name);
+            g_warning ("class '%s' doesn't have property '%s'",
+                       g_type_name (type), prop_name);
             _moo_param_array_free ((GParameter*)params->data, params->len);
             g_array_free (params, FALSE);
             g_type_class_unref (klass);
@@ -1385,7 +1424,7 @@ _moo_param_array_collect_valist (GType       type,
 
         if (error)
         {
-            g_critical ("%s: %s", G_STRLOC, error);
+            g_critical ("%s", error);
             g_free (error);
             g_value_unset (&param.value);
             g_free ((char*)param.name);
@@ -1598,11 +1637,11 @@ prop_watch_new (GObject            *target,
     if (!source_pspec || !target_pspec)
     {
         if (!source_pspec)
-            g_warning ("%s: no property '%s' in class '%s'",
-                       G_STRLOC, source_prop, g_type_name (G_OBJECT_TYPE (source)));
+            g_warning ("no property '%s' in class '%s'",
+                       source_prop, g_type_name (G_OBJECT_TYPE (source)));
         if (!target_pspec)
-            g_warning ("%s: no property '%s' in class '%s'",
-                       G_STRLOC, target_prop, g_type_name (G_OBJECT_TYPE (target)));
+            g_warning ("no property '%s' in class '%s'",
+                       target_prop, g_type_name (G_OBJECT_TYPE (target)));
         return NULL;
     }
 
@@ -1771,17 +1810,13 @@ _moo_sync_bool_property (gpointer            slave,
 
 
 void
-moo_bind_sensitive (GtkWidget          *btn,
-                    GtkWidget         **dependent,
-                    int                 num_dependent,
-                    gboolean            invert)
+moo_bind_sensitive (GtkWidget *btn,
+                    GtkWidget *dependent,
+                    gboolean   invert)
 {
-    int i;
-
     g_return_if_fail (G_IS_OBJECT (btn));
-
-    for (i = 0; i < num_dependent; ++i)
-        moo_bind_bool_property (dependent[i], "sensitive", btn, "active", invert);
+    g_return_if_fail (G_IS_OBJECT (dependent));
+    moo_bind_bool_property (dependent, "sensitive", btn, "active", invert);
 }
 
 
@@ -1867,8 +1902,8 @@ check_signal (GObject    *obj,
 
     if (!g_signal_parse_name (signal, G_OBJECT_TYPE (obj), signal_id, detail, FALSE))
     {
-        g_warning ("%s: could not parse signal '%s' of '%s' object",
-                   G_STRLOC, signal, g_type_name (G_OBJECT_TYPE (obj)));
+        g_warning ("could not parse signal '%s' of '%s' object",
+                   signal, g_type_name (G_OBJECT_TYPE (obj)));
         return FALSE;
     }
 
@@ -1876,7 +1911,7 @@ check_signal (GObject    *obj,
 
     if (query.n_params > 0)
     {
-        g_warning ("%s: implement me", G_STRLOC);
+        g_warning ("implement me");
         return FALSE;
     }
 
@@ -1889,11 +1924,11 @@ check_signal (GObject    *obj,
         case G_TYPE_INT:
         case G_TYPE_UINT:
             if (!any_return)
-                g_warning ("%s: implement me", G_STRLOC);
+                g_warning ("implement me");
             return FALSE;
 
         default:
-            g_warning ("%s: implement me", G_STRLOC);
+            g_warning ("implement me");
             return FALSE;
     }
 
@@ -1994,7 +2029,6 @@ _moo_ptr_ref (MooPtr *ptr)
     return ptr;
 }
 
-
 static void
 _moo_ptr_unref (MooPtr *ptr)
 {
@@ -2006,6 +2040,7 @@ _moo_ptr_unref (MooPtr *ptr)
     }
 }
 
+MOO_DEFINE_BOXED_TYPE_R (MooPtr, _moo_ptr)
 
 static MooPtr *
 _moo_ptr_new (gpointer        data,
@@ -2025,20 +2060,6 @@ _moo_ptr_new (gpointer        data,
 }
 
 
-GType
-_moo_ptr_get_type (void)
-{
-    static GType type = 0;
-
-    if (G_UNLIKELY (!type))
-        type = g_boxed_type_register_static ("MooPtr",
-                                             (GBoxedCopyFunc) _moo_ptr_ref,
-                                             (GBoxedFreeFunc) _moo_ptr_unref);
-
-    return type;
-}
-
-
 static MooData *
 _moo_data_ref (MooData *data)
 {
@@ -2058,19 +2079,7 @@ _moo_data_unref (MooData *data)
     }
 }
 
-
-GType
-_moo_data_get_type (void)
-{
-    static GType type = 0;
-
-    if (G_UNLIKELY (!type))
-        type = g_boxed_type_register_static ("MooData",
-                                             (GBoxedCopyFunc) _moo_data_ref,
-                                             (GBoxedFreeFunc) _moo_data_unref);
-
-    return type;
-}
+MOO_DEFINE_BOXED_TYPE_R (MooData, _moo_data)
 
 
 static void
