@@ -133,78 +133,4 @@
 #define _MOO_STATIC_ASSERT_MACRO(cond) enum { MOO_CONCAT(_MooStaticAssert_, __LINE__) = 1 / ((cond) ? 1 : 0) }
 #define MOO_STATIC_ASSERT(cond, message) _MOO_STATIC_ASSERT_MACRO(cond)
 
-#ifdef MOO_DEV_MODE
-
-#define __moo_test_func_name MOO_CONCAT(__moo_test_func_, __LINE__)
-#define __moo_test_func __moo_test_func_name
-#define __moo_test_func_a(args) __moo_test_func_name args
-
-MOO_STATIC_ASSERT(sizeof(char) == 1, "test");
-
-void __moo_test_func(void);
-void __moo_test_func(void) MOO_FA_WARNING("warning");
-void MOO_FA_WARNING("warning") __moo_test_func(void);
-void *__moo_test_func(void) MOO_FA_MALLOC;
-void * MOO_FA_MALLOC __moo_test_func(void);
-void * MOO_FA_NONNULL(()) __moo_test_func_a((void *p));
-void * __moo_test_func_a((void *p)) MOO_FA_NONNULL(());
-void * MOO_FA_NONNULL((1)) __moo_test_func_a((void *p));
-void * __moo_test_func_a((void *p)) MOO_FA_NONNULL((1));
-void * MOO_FA_NONNULL((1,2)) __moo_test_func_a((void *p1, void *p2));
-void * __moo_test_func_a((void *p1, void *p2)) MOO_FA_NONNULL((1,2));
-
-void MOO_NORETURN __moo_test_func_name(void);
-void MOO_NOTHROW __moo_test_func_name(void);
-
-inline static void MOO_NOTHROW __moo_test_func_name(void)
-{
-}
-
-void __moo_test_func(void) MOO_FA_NORETURN;
-void MOO_FA_NORETURN __moo_test_func(void);
-
-void __moo_test_func(void) MOO_FA_NOTHROW;
-void MOO_FA_NOTHROW __moo_test_func(void);
-
-void __moo_test_func(void) MOO_FA_UNUSED;
-void MOO_FA_UNUSED __moo_test_func(void);
-
-int __moo_test_func(void) MOO_FA_WARN_UNUSED_RESULT;
-int MOO_FA_WARN_UNUSED_RESULT __moo_test_func(void);
-
-inline static void __moo_test_dummy1(void *p)
-{
-    MOO_UNUSED(p);
-}
-
-inline static void __moo_test_func(void)
-{
-    char *p MOO_VA_CLEANUP(__moo_test_dummy1) = 0;
-    MOO_UNUSED(p);
-}
-
-inline static void __moo_test_func(void)
-{
-    char *p MOO_VAR_CLEANUP_CHECK(__moo_test_dummy1) = 0;
-    MOO_UNUSED(p);
-}
-
-inline static void __moo_test_func(void)
-{
-    char *p MOO_VAR_CLEANUP_CHECKD(__moo_test_dummy1) = 0;
-    MOO_UNUSED(p);
-}
-
-inline static void __moo_test_func(void)
-{
-    MOO_STATIC_ASSERT (sizeof(char) == 1, "test");
-}
-
-inline static void __moo_test_dummy2(void) NOTHROW;
-inline static void NOTHROW __moo_test_dummy2(void)
-{
-}
-
-#endif /* MOO_DEV_MODE */
-
 #endif /* MOO_UTILS_MACROS_H */
