@@ -29,7 +29,7 @@
 #include "mooutils/mooarray.h"
 #include "mooutils/moologwindow-gxml.h"
 #include <gtk/gtk.h>
-#include <glib/gmappedfile.h>
+#include <glib.h>
 #include <glib/gstdio.h>
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
@@ -74,91 +74,20 @@ open_uri (const char *uri,
 
 #else /* ! __WIN32__ */
 
-char *
-_moo_find_script (const char *name,
-                  gboolean    search_path)
-{
-    char *path = NULL;
-
-    g_return_val_if_fail (name != NULL, NULL);
-
-    if (search_path)
-        path = g_find_program_in_path (name);
-
-    if (!path)
-    {
-        char **dirs, **p;
-
-        dirs = moo_get_data_subdirs ("scripts");
-
-        for (p = dirs; p && *p; ++p)
-        {
-            path = g_build_filename (*p, name, NULL);
-
-            if (g_file_test (path, G_FILE_TEST_IS_EXECUTABLE))
-                break;
-
-            g_free (path);
-            path = NULL;
-        }
-
-        g_strfreev (dirs);
-    }
-
-    if (!path)
-    {
-        g_warning ("could not find %s script", name);
-        path = g_strdup (name);
-    }
-
-    _moo_message ("%s: %s", name, path);
-    return path;
-}
-
-enum {
-    SCRIPT_XDG_OPEN,
-    SCRIPT_XDG_EMAIL
-};
-
-static const char *
-get_xdg_script (int type)
-{
-    static char *path[2];
-
-    g_return_val_if_fail (type < 2, NULL);
-
-    if (!path[0])
-    {
-        path[SCRIPT_XDG_OPEN] = _moo_find_script ("xdg-open", TRUE);
-        path[SCRIPT_XDG_EMAIL] = _moo_find_script ("xdg-email", TRUE);
-    }
-
-    return path[type];
-}
-
 static gboolean
 open_uri (const char *uri,
-          gboolean    email)
+          G_GNUC_UNUSED gboolean email)
 {
-    gboolean result = FALSE;
-    const char *argv[3];
     GError *error = NULL;
 
-    argv[0] = get_xdg_script (email ? SCRIPT_XDG_EMAIL : SCRIPT_XDG_OPEN);
-    argv[1] = uri;
-    argv[2] = NULL;
-
-    result = g_spawn_async (NULL, (char**) argv, NULL, (GSpawnFlags) 0, NULL, NULL,
-                            NULL, &error);
-
-    if (!result)
+    if (!gtk_show_uri (NULL, uri, gtk_get_current_event_time (), &error))
     {
-        g_warning ("error in g_spawn_async: %s", moo_error_message (error));
+        g_warning ("Unable to show '%s': %s", uri, error->message);
         g_error_free (error);
-        error = NULL;
+        return FALSE;
     }
 
-    return result;
+    return TRUE;
 }
 
 #endif /* ! __WIN32__ */
@@ -813,39 +742,33 @@ moo_reset_log_func (void)
  * Display log messages in a window
  */
 
-#if !defined(__WIN32__) || !defined(DEBUG)
-
-static void
-on_warning_or_critical_message (G_GNUC_UNUSED const char *message)
-{
-}
-
-#else /* __WIN32__ && DEBUG */
-
-static void
-on_warning_or_critical_message (const char *message)
-{
-    _moo_abort_debug_ignore (MOO_CODE_LOC_UNKNOWN, message);
-}
-
-#endif
-
-static void
-log_func_window (const gchar    *log_domain,
-                 GLogLevelFlags  flags,
-                 const gchar    *message,
-                 G_GNUC_UNUSED gpointer dummy)
-{
-    char *text;
-
 #ifdef __WIN32__
+static void
+win32_filter_fatal_errors (const gchar    *log_domain,
+                           GLogLevelFlags  flags,
+                           const gchar    *message)
+{
     if (flags & (G_LOG_LEVEL_ERROR | G_LOG_FLAG_FATAL | G_LOG_FLAG_RECURSION))
     {
         _moo_win32_show_fatal_error (log_domain, message);
         return;
     }
+}
+#else /* __WIN32__ */
+static void
+win32_filter_fatal_errors (G_GNUC_UNUSED const gchar    *log_domain,
+                           G_GNUC_UNUSED GLogLevelFlags  flags,
+                           G_GNUC_UNUSED const gchar    *message)
+{
+}
 #endif /* __WIN32__ */
 
+static char *
+format_log_message (const char *log_domain,
+                    const char *message)
+{
+    char *text = NULL;
+
     if (!g_utf8_validate (message, -1, NULL))
         message = "<corrupted string, invalid UTF8>";
 
@@ -854,8 +777,20 @@ log_func_window (const gchar    *log_domain,
     else
         text = g_strdup_printf ("%s\n", message);
 
-    if (flags < G_LOG_LEVEL_MESSAGE)
-        on_warning_or_critical_message (text);
+    return text;
+}
+
+static void
+log_func_window (const gchar    *log_domain,
+                 GLogLevelFlags  flags,
+                 const gchar    *message,
+                 G_GNUC_UNUSED gpointer dummy)
+{
+    char *text;
+
+    win32_filter_fatal_errors (log_domain, flags, message);
+
+    text = format_log_message (log_domain, message);
 
     {
         GtkTextTag *tag;
@@ -964,26 +899,21 @@ print_func_file (const char *string)
 
 static void
 log_func_file (const char       *log_domain,
-               G_GNUC_UNUSED GLogLevelFlags flags,
+               GLogLevelFlags    flags,
                const char       *message,
                G_GNUC_UNUSED gpointer dummy)
 {
     char *string;
 
-#ifdef __WIN32__
-    if (flags & (G_LOG_LEVEL_ERROR | G_LOG_FLAG_FATAL | G_LOG_FLAG_RECURSION))
-    {
-        _moo_win32_show_fatal_error (log_domain, message);
-        return;
-    }
-#endif /* __WIN32__ */
-
     if (log_domain)
         string = g_strdup_printf ("%s: %s\n", log_domain, message);
     else
         string = g_strdup_printf ("%s\n", message);
 
     print_func_file (string);
+
+    win32_filter_fatal_errors (log_domain, flags, message);
+
     g_free (string);
 }
 
@@ -1004,18 +934,12 @@ moo_set_log_func_file (const char *log_file)
  */
 
 static void
-log_func_silent (G_GNUC_UNUSED const gchar    *log_domain,
-                 G_GNUC_UNUSED GLogLevelFlags  flags,
-                 G_GNUC_UNUSED const gchar    *message,
-                 G_GNUC_UNUSED gpointer        data_unused)
+log_func_silent (const gchar    *log_domain,
+                 GLogLevelFlags  flags,
+                 const gchar    *message,
+                 G_GNUC_UNUSED gpointer data_unused)
 {
-#ifdef __WIN32__
-    if (flags & (G_LOG_LEVEL_ERROR | G_LOG_FLAG_FATAL | G_LOG_FLAG_RECURSION))
-    {
-        _moo_win32_show_fatal_error (log_domain, message);
-        return;
-    }
-#endif /* __WIN32__ */
+    win32_filter_fatal_errors (log_domain, flags, message);
 }
 
 static void
@@ -1697,105 +1621,10 @@ _moo_widget_set_tooltip (GtkWidget  *widget,
  * Asserts
  */
 
-#if !defined(__WIN32__)
-
-void
-_moo_abort_debug_ignore (MooCodeLoc loc, const char *message)
-{
-    if (moo_code_loc_valid (loc))
-	g_error ("file '%s', function '%s', line %d: %s\n", loc.file, loc.func, loc.line, message);
-    else
-        g_error ("%s\n", message);
-}
-
-#else /* !__WIN32__ */
-
-static void
-break_into_debugger (void)
-{
-    RaiseException(0, EXCEPTION_NONCONTINUABLE, 0, NULL);
-}
-
-void
-_moo_abort_debug_ignore (MooCodeLoc loc, const char *message)
-{
-    gboolean skip = FALSE;
-    static GHashTable *locs_hash;
-    char *loc_id = NULL;
-    gboolean use_location;
-
-    use_location = moo_code_loc_valid (loc);
-
-    if (use_location)
-        loc_id = g_strdup_printf ("%s#%d#%d", loc.file, loc.line, loc.counter);
-    else
-        loc_id = g_strdup (message);
-
-    if (loc_id != NULL)
-    {
-        if (!locs_hash)
-            locs_hash = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, NULL);
-        if (g_hash_table_lookup (locs_hash, loc_id))
-            skip = TRUE;
-    }
-
-    if (!skip)
-    {
-        int ret;
-
-        if (use_location)
-            ret = _moo_win32_message_box (NULL, MB_ABORTRETRYIGNORE, NULL,
-                                          Q_("console message|in file %s, line %d, function %s:\n%s"),
-                                          loc.file, loc.line, loc.func, message);
-        else
-            ret = _moo_win32_message_box (NULL, MB_ABORTRETRYIGNORE, NULL,
-                                          "%s", message);
-
-        switch (ret)
-        {
-            case IDABORT:
-                TerminateProcess (GetCurrentProcess (), 1);
-                break;
-
-            case IDRETRY:
-                break_into_debugger ();
-                break;
-
-            case IDIGNORE:
-                if (GetKeyState(VK_CONTROL) < 0)
-                    skip = TRUE;
-                break;
-
-            default:
-                moo_abort ();
-                break;
-        }
-
-        if (skip && loc_id)
-        {
-            g_hash_table_insert (locs_hash, loc_id, GINT_TO_POINTER (1));
-            loc_id = NULL;
-        }
-    }
-
-    g_free (loc_id);
-}
-
-#endif /* !__WIN32__ */
-
-#if !defined(MOO_DEV_MODE) && !defined(DEBUG)
-NORETURN
-#endif
-void
+NORETURN void
 _moo_assert_message (MooCodeLoc loc, const char *message)
 {
-#if defined(MOO_DEV_MODE) && defined(__WIN32__)
-    _moo_abort_debug_ignore (loc, message);
-#elif defined(MOO_DEV_MODE) || defined(DEBUG)
-    g_critical ("file '%s', function '%s', line %d: %s\n", loc.file, loc.func, loc.line, message);
-#else
     g_error ("file '%s', function '%s', line %d: %s\n", loc.file, loc.func, loc.line, message);
-#endif
 }
 
 void
@@ -1834,14 +1663,6 @@ void _moo_logv (MooCodeLoc loc, GLogLevelFlags flags, const char *format, va_lis
 
     message = g_strdup_vprintf (format, args);
 
-#if defined(MOO_DEV_MODE) && !defined(__WIN32__)
-    if (flags < G_LOG_LEVEL_MESSAGE)
-    {
-        _moo_abort_debug_ignore (loc, message);
-        flags = G_LOG_LEVEL_MESSAGE;
-    }
-#endif
-
     if (moo_code_loc_valid (loc))
         g_log (G_LOG_DOMAIN, flags,
                Q_("console message|in file %s, line %d, function %s: %s"),
