@@ -1,18 +1,22 @@
 /*
  *   mooutils-treeview.c
  *
- *   Copyright (C) 2004-2008 by Yevgen Muntyan <muntyan@tamu.edu>
+ *   Copyright (C) 2004-2010 by Yevgen Muntyan <emuntyan@users.sourceforge.net>
  *
- *   This library is free software; you can redistribute it and/or
- *   modify it under the terms of the GNU Lesser General Public
- *   License version 2.1 as published by the Free Software Foundation.
+ *   This file is part of medit.  medit is free software; you can
+ *   redistribute it and/or modify it under the terms of the
+ *   GNU Lesser General Public License as published by the
+ *   Free Software Foundation; either version 2.1 of the License,
+ *   or (at your option) any later version.
  *
- *   See COPYING file that comes with this distribution.
+ *   You should have received a copy of the GNU Lesser General Public
+ *   License along with medit.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "mooutils/mooutils-treeview.h"
 #include "mooutils/mooutils-misc.h"
 #include "mooutils/mooutils-gobject.h"
+#include "mooutils/moocompat.h"
 #include "mooutils/mootype-macros.h"
 #include "marshals.h"
 #include <string.h>
@@ -380,11 +384,9 @@ _moo_tree_helper_set_modified (MooTreeHelper *helper,
 {
     g_return_if_fail (MOO_IS_TREE_HELPER (helper));
 
-#ifdef MOO_DEBUG_ENABLED
+#ifdef MOO_DEBUG
     if (!helper->modified && modified)
-    {
-        g_message ("%s: helper modified", G_STRLOC);
-    }
+        g_message ("helper modified");
 #endif
 
     helper->modified = modified != 0;
@@ -607,7 +609,7 @@ moo_tree_helper_delete_row (MooTreeHelper *helper)
 
     if (!gtk_tree_selection_get_selected (selection, &model, &iter))
     {
-        g_critical ("%s: oops", G_STRLOC);
+        g_critical ("oops");
         return;
     }
 
@@ -644,7 +646,7 @@ moo_tree_helper_row_move (MooTreeHelper *helper,
 
     if (!gtk_tree_selection_get_selected (selection, &model, &iter))
     {
-        g_critical ("%s: oops", G_STRLOC);
+        g_critical ("oops");
         return;
     }
 
@@ -765,8 +767,8 @@ _moo_tree_helper_new (GtkWidget *widget,
 
     g_return_val_if_fail (GTK_IS_TREE_VIEW (widget) || GTK_IS_COMBO_BOX (widget), NULL);
 
-    helper = g_object_new (MOO_TYPE_TREE_HELPER, NULL);
-    MOO_OBJECT_REF_SINK (helper);
+    helper = g_object_new (MOO_TYPE_TREE_HELPER, (const char*) NULL);
+    g_object_ref_sink (helper);
 
     _moo_tree_helper_connect (helper, widget, new_btn, delete_btn, up_btn, down_btn);
 
@@ -841,7 +843,7 @@ moo_tree_helper_set_valist (MooTreeHelper *helper,
 
         if (column >= n_columns)
         {
-            g_warning ("%s: invalid column number %d", G_STRLOC, column);
+            g_warning ("invalid column number %d", column);
             break;
         }
 
@@ -852,7 +854,7 @@ moo_tree_helper_set_valist (MooTreeHelper *helper,
 
         if (error)
         {
-            g_warning ("%s: %s", G_STRLOC, error);
+            g_warning ("%s", error);
             g_free (error);
             break;
         }
@@ -1114,7 +1116,7 @@ expander_data_free (ExpanderData *data)
         if (data->cell)
             g_object_unref (data->cell);
         expander_data_disconnect_model (data);
-        moo_free (ExpanderData, data);
+        g_slice_free (ExpanderData, data);
     }
 }
 
@@ -1200,22 +1202,18 @@ _moo_tree_view_setup_expander (GtkTreeView       *tree_view,
     g_return_if_fail (GTK_IS_TREE_VIEW (tree_view));
     g_return_if_fail (GTK_IS_TREE_VIEW_COLUMN (column));
 
-#if !GTK_CHECK_VERSION(2,12,0)
-    return;
-#endif
-
     g_object_set (tree_view,
                   "show-expanders", FALSE,
                   "level-indentation", LEVEL_INDENTATION,
                   NULL);
 
-    cell = g_object_new (MOO_TYPE_EXPANDER_CELL, NULL);
+    cell = g_object_new (MOO_TYPE_EXPANDER_CELL, (const char*) NULL);
     gtk_tree_view_column_pack_start (column, cell, FALSE);
     gtk_tree_view_column_set_cell_data_func (column, cell,
                                              (GtkTreeCellDataFunc) expander_cell_data_func,
                                              tree_view, NULL);
 
-    data = moo_new0 (ExpanderData);
+    data = g_slice_new0 (ExpanderData);
     data->column = g_object_ref (column);
     data->cell = g_object_ref (cell);
     g_object_set_data_full (G_OBJECT (tree_view), "moo-tree-view-expander-data",
